@@ -2,9 +2,10 @@
 
 """Generate the code reference pages and navigation."""
 
-import mkdocs_gen_files
 from pathlib import Path
 
+import mkdocs_gen_files
+
 nav = mkdocs_gen_files.Nav()
 
 for path in sorted(Path("src/kiara").rglob("*.py")):
@@ -21,7 +22,7 @@ for path in sorted(Path("src/kiara").rglob("*.py")):
     elif parts[-1] == "__main__":
         continue
 
-    nav[parts] = doc_path  #
+    nav[parts] = doc_path
 
     with mkdocs_gen_files.open(full_doc_path, "w") as fd:
         ident = ".".join(parts)
@@ -29,5 +30,5 @@ for path in sorted(Path("src/kiara").rglob("*.py")):
 
     mkdocs_gen_files.set_edit_path(full_doc_path, path)
 
-with mkdocs_gen_files.open("reference/SUMMARY.md", "w") as nav_file:  #
-    nav_file.writelines(nav.build_literate_nav())  #
+with mkdocs_gen_files.open("reference/SUMMARY.md", "w") as nav_file:
+    nav_file.writelines(nav.build_literate_nav())
