@@ -7,36 +7,47 @@ import contextlib
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 import os
 import uuid
-from enum import Enum
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Mapping, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Iterable,
+    List,
+    Literal,
+    Mapping,
+    Type,
+    Union,
+)
 
 import structlog
-from pydantic import BaseModel, root_validator, validator
-from pydantic.config import Extra
-from pydantic.env_settings import BaseSettings
+from pydantic import BaseModel, ConfigDict, field_validator, model_validator
 from pydantic.fields import Field, PrivateAttr
+from pydantic_settings import BaseSettings, SettingsConfigDict
 from ruamel import yaml as r_yaml
 
+from kiara.context.runtime_config import KiaraRuntimeConfig
 from kiara.defaults import (
     DEFAULT_ALIAS_STORE_MARKER,
     DEFAULT_CONTEXT_NAME,
     DEFAULT_DATA_STORE_MARKER,
     DEFAULT_JOB_STORE_MARKER,
+    DEFAULT_METADATA_STORE_MARKER,
     DEFAULT_WORKFLOW_STORE_MARKER,
     KIARA_CONFIG_FILE_NAME,
     KIARA_MAIN_CONFIG_FILE,
     KIARA_MAIN_CONTEXTS_PATH,
-    METADATA_DESTINY_STORE_MARKER,
     kiara_app_dirs,
 )
-from kiara.registries.environment import EnvironmentRegistry
+from kiara.exceptions import KiaraException
 from kiara.registries.ids import ID_REGISTRY
+from kiara.utils import log_message
 from kiara.utils.files import get_data_from_file
 
 if TYPE_CHECKING:
     from kiara.context import Kiara
     from kiara.models.context import ContextInfo
+    from kiara.registries import BaseArchive, KiaraArchive
 
 logger = structlog.getLogger()
 
@@ -46,7 +57,9 @@ yaml.default_flow_style = False
 
 def config_file_settings_source(settings: BaseSettings) -> Dict[str, Any]:
     if os.path.isfile(KIARA_MAIN_CONFIG_FILE):
-        config = get_data_from_file(KIARA_MAIN_CONFIG_FILE, content_type="yaml")
+        config: Dict[str, Any] = get_data_from_file(
+            KIARA_MAIN_CONFIG_FILE, content_type="yaml"
+        )
         if not isinstance(config, Mapping):
             raise ValueError(
                 f"Invalid config file format, can't parse file: {KIARA_MAIN_CONFIG_FILE}"
@@ -57,21 +70,238 @@ def config_file_settings_source(settings: BaseSettings) -> Dict[str, Any]:
 
 
 class KiaraArchiveConfig(BaseModel):
+    """Configuration data that can be used to load an existing kiara archive."""
 
-    archive_id: str = Field(description="The unique archive id.")
+    # archive_alias: str = Field(description="The unique archive id.")
     archive_type: str = Field(description="The archive type.")
     config: Mapping[str, Any] = Field(
         description="Archive type specific config.", default_factory=dict
     )
 
+
+class KiaraArchiveReference(BaseModel):
+    @classmethod
+    def load_existing_archive(
+        cls,
+        archive_uri: str,
+        store_type: Union[str, None, Iterable[str]] = None,
+        allow_write_access: bool = False,
+        archive_name: Union[str, None] = None,
+        **kwargs: Any,
+    ) -> "KiaraArchiveReference":
+        from kiara.utils.class_loading import find_all_archive_types
+
+        archive_types = find_all_archive_types()
+
+        archive_configs: List[KiaraArchiveConfig] = []
+        archives: List[KiaraArchive] = []
+
+        if store_type:
+            if isinstance(store_type, str):
+                archive_cls: Union[Type[KiaraArchive], None] = archive_types.get(
+                    store_type, None
+                )
+                if archive_cls is None:
+                    raise Exception(
+                        f"Can't create context: no archive type '{store_type}' available. Available types: {', '.join(archive_types.keys())}"
+                    )
+                data = archive_cls.load_archive_config(
+                    archive_uri=archive_uri,
+                    allow_write_access=allow_write_access,
+                    **kwargs,
+                )
+                archive_config = archive_cls._config_cls(**data)
+                archive: KiaraArchive = archive_cls(
+                    archive_config=archive_config, archive_name=archive_name
+                )
+                wrapped_archive_config = KiaraArchiveConfig(
+                    archive_type=store_type, config=data
+                )
+                archive_configs.append(wrapped_archive_config)
+                archives.append(archive)
+            else:
+                for st in store_type:
+                    archive_cls = archive_types.get(st, None)
+                    if archive_cls is None:
+                        raise Exception(
+                            f"Can't create context: no archive type '{store_type}' available. Available types: {', '.join(archive_types.keys())}"
+                        )
+                    data = archive_cls.load_archive_config(
+                        archive_uri=archive_uri,
+                        allow_write_access=allow_write_access,
+                        **kwargs,
+                    )
+                    archive_config = archive_cls._config_cls(**data)
+                    archive = archive_cls(
+                        archive_config=archive_config, archive_name=archive_name
+                    )
+                    wrapped_archive_config = KiaraArchiveConfig(
+                        archive_type=st, config=data
+                    )
+                    archive_configs.append(wrapped_archive_config)
+                    archives.append(archive)
+        else:
+            for archive_type, archive_cls in archive_types.items():
+                data = archive_cls.load_archive_config(
+                    archive_uri=archive_uri,
+                    allow_write_access=allow_write_access,
+                    **kwargs,
+                )
+
+                if data is None:
+                    continue
+
+                archive_config = archive_cls._config_cls(**data)
+                archive = archive_cls(
+                    archive_config=archive_config, archive_name=archive_name
+                )
+                wrapped_archive_config = KiaraArchiveConfig(
+                    archive_type=archive_type, config=data
+                )
+                archive_configs.append(wrapped_archive_config)
+                archives.append(archive)
+
+        if archives is None:
+            raise Exception(
+                f"Can't create context: no valid archive found at '{archive_uri}'"
+            )
+
+        result = cls(
+            archive_uri=archive_uri,
+            allow_write_access=allow_write_access,
+            archive_configs=archive_configs,
+            # archive_alias=archive_alias,
+        )
+        result._archives = archives
+        return result
+
+    archive_uri: str = Field(description="The uri that points to the archive.")
+    # archive_alias: str = Field(
+    #     description="The alias that is used for the archives contained in here."
+    # )
+    allow_write_access: bool = Field(
+        description="Whether to allow write access to the archives contained here.",
+        default=False,
+    )
+    archive_configs: List[KiaraArchiveConfig] = Field(
+        description="All the archives this kiara context can use and the aliases they are registered with."
+    )
+    _archives: Union[None, List["KiaraArchive"]] = PrivateAttr(default=None)
+
     @property
-    def archive_uuid(self) -> uuid.UUID:
-        return uuid.UUID(self.archive_id)
+    def archives(self) -> List["KiaraArchive"]:
+        if self._archives is not None:
+            return self._archives
+
+        from kiara.utils.class_loading import find_all_archive_types
+
+        archive_types = find_all_archive_types()
+
+        archive_alias = None
+
+        result = []
+        for config in self.archive_configs:
+            if config.archive_type not in archive_types.keys():
+                raise Exception(
+                    f"Can't create context: no archive type '{config.archive_type}' available. Available types: {', '.join(archive_types.keys())}"
+                )
+
+            archive_cls = archive_types[config.archive_type]
+            archive_config_data = archive_cls.load_archive_config(
+                archive_uri=self.archive_uri,
+                allow_write_access=self.allow_write_access,
+            )
+            archive_config = archive_cls._config_cls(**archive_config_data)
+            archive = archive_cls(
+                archive_config=archive_config, archive_name=archive_alias
+            )
+            result.append(archive)
+
+        self._archives = result
+        return self._archives
 
 
 class KiaraContextConfig(BaseModel):
-    class Config:
-        extra = Extra.forbid
+    @classmethod
+    def create_from_sqlite_db(cls, db_path: Path) -> "KiaraContextConfig":
+        import sqlite3
+
+        if not db_path.exists():
+            context_id = str(uuid.uuid4())
+            conn = sqlite3.connect(db_path)
+            c = conn.cursor()
+            c.execute(
+                """CREATE TABLE context_metadata
+                         (key text PRIMARY KEY , value text NOT NULL)"""
+            )
+            c.execute(
+                "INSERT INTO context_metadata VALUES ('context_id', ?)", (context_id,)
+            )
+            c.execute(
+                """CREATE TABLE archive_metadata
+                         (key text PRIMARY KEY , value text NOT NULL)"""
+            )
+            c.execute(
+                "INSERT INTO archive_metadata VALUES ('archive_id', ?)", (context_id,)
+            )
+
+            conn.commit()
+            conn.close()
+        else:
+            try:
+                with sqlite3.connect(db_path) as conn:
+                    context_id = conn.execute(
+                        "SELECT value FROM context_metadata WHERE key = 'context_id'"
+                    ).fetchone()[0]
+            except Exception as e:
+                raise KiaraException(
+                    f"Can't read context from sqlite db '{db_path}': {e}"
+                )
+
+        base_path = os.path.abspath(kiara_app_dirs.user_data_dir)
+        stores_base_path = os.path.join(base_path, "stores")
+        workflow_base_path = os.path.join(
+            stores_base_path, "filesystem_stores", "workflows"
+        )
+        workflow_store_path = os.path.join(workflow_base_path, context_id)
+
+        data_store_config = KiaraArchiveConfig(
+            archive_type="sqlite_data_store",
+            config={"sqlite_db_path": db_path.as_posix()},
+        )
+        alias_store_config = KiaraArchiveConfig(
+            archive_type="sqlite_alias_store",
+            config={"sqlite_db_path": db_path.as_posix()},
+        )
+        job_store_config = KiaraArchiveConfig(
+            archive_type="sqlite_job_store",
+            config={"sqlite_db_path": db_path.as_posix()},
+        )
+        workflow_store_config = KiaraArchiveConfig(
+            archive_type="filesystem_workflow_store",
+            config={"archive_path": workflow_store_path},
+        )
+        metadata_store_config = KiaraArchiveConfig(
+            archive_type="sqlite_metadata_store",
+            config={"sqlite_db_path": db_path.as_posix()},
+        )
+
+        archives = {
+            DEFAULT_DATA_STORE_MARKER: data_store_config,
+            DEFAULT_ALIAS_STORE_MARKER: alias_store_config,
+            DEFAULT_JOB_STORE_MARKER: job_store_config,
+            DEFAULT_WORKFLOW_STORE_MARKER: workflow_store_config,
+            DEFAULT_METADATA_STORE_MARKER: metadata_store_config,
+        }
+
+        context_config = cls(
+            context_id=context_id,
+            archives=archives,
+        )
+
+        return context_config
+
+    model_config = ConfigDict(extra="forbid")
 
     context_id: str = Field(description="A globally unique id for this kiara context.")
 
@@ -85,7 +315,6 @@ class KiaraContextConfig(BaseModel):
     _context_config_path: Union[Path, None] = PrivateAttr(default=None)
 
     def add_pipelines(self, *pipelines: str):
-
         for pipeline in pipelines:
             if os.path.exists(pipeline):
                 self.extra_pipelines.append(pipeline)
@@ -94,27 +323,28 @@ class KiaraContextConfig(BaseModel):
                     "ignore.pipeline", reason="path does not exist", path=pipeline
                 )
 
-    # @property
-    # def db_url(self):
-    #     return get_kiara_db_url(self.context_folder)
+    # def create_archive(
+    #     self, archive_alias: str, allow_write_access: bool = False
+    # ) -> "KiaraArchive":
+    #     """Create the kiara archive with the specified alias.
     #
-    # @property
-    # def data_directory(self) -> str:
-    #     return os.path.join(self.context_folder, "data")
-
-
-class JobCacheStrategy(Enum):
-
-    no_cache = "no_cache"
-    value_id = "value_id"
-    data_hash = "data_hash"
+    #     Make sure you know what you are doing when setting 'allow_write_access' to True.
+    #     """
+    #
+    #     store_config = self.archives[archive_alias]
+    #     store = create_store(
+    #         archive_id=store_config.archive_uuid,
+    #         store_type=store_config.archive_type,
+    #         store_config=store_config.config,
+    #         allow_write_access=allow_write_access,
+    #     )
+    #     return store
 
 
 class KiaraSettings(BaseSettings):
-    class Config:
-        extra = Extra.forbid
-        validate_assignment = True
-        env_prefix = "kiara_setting_"
+    model_config = SettingsConfigDict(
+        extra="forbid", validate_assignment=True, env_prefix="kiara_setting_"
+    )
 
     syntax_highlight_background: str = Field(
         description="The background color for code blocks when rendering to terminal, Jupyter, etc.",
@@ -125,34 +355,52 @@ class KiaraSettings(BaseSettings):
 KIARA_SETTINGS = KiaraSettings()
 
 
-class KiaraRuntimeConfig(BaseSettings):
-    class Config:
-        extra = Extra.forbid
-        validate_assignment = True
-        env_prefix = "kiara_runtime_"
+def create_default_store_config(
+    store_type: str, stores_base_path: str, use_wal_mode: bool = False
+) -> KiaraArchiveConfig:
+    from kiara.utils.archives import find_archive_types
 
-    job_cache: JobCacheStrategy = Field(
-        description="Name of the strategy that determines when to re-run jobs or use cached results.",
-        default=JobCacheStrategy.data_hash,
+    # env_registry = EnvironmentRegistry.instance()
+    # find_archive_types = find_archive_types()
+    # kiara_types: "KiaraTypesRuntimeEnvironment" = env_registry.environments["kiara_types"]  # type: ignore
+    available_archives = find_archive_types()
+
+    assert store_type in available_archives.item_infos.keys()
+
+    from kiara.models.archives import ArchiveTypeInfo
+
+    archive_info: ArchiveTypeInfo = available_archives.item_infos[store_type]
+    cls: Type[BaseArchive] = archive_info.python_class.get_class()  # type: ignore
+
+    log_message(
+        "create_new_store",
+        stores_base_path=stores_base_path,
+        store_type=cls.__name__,
     )
-    allow_external: bool = Field(
-        description="Whether to allow external external pipelines.", default=True
+
+    config = cls._config_cls.create_new_store_config(
+        store_base_path=stores_base_path, use_wal_mode=use_wal_mode
     )
-    # ignore_errors: bool = Field(
-    #     description="If set, kiara will try to ignore most errors (that can be ignored).",
-    #     default=False,
-    # )
+
+    # store_id: uuid.UUID = config.get_archive_id()
+
+    data_store = KiaraArchiveConfig(
+        archive_type=store_type,
+        config=config.model_dump(),
+    )
+    return data_store
+
+
+DEFAULT_STORE_TYPE: Literal["sqlite"] = "sqlite"
 
 
 class KiaraConfig(BaseSettings):
-    class Config:
-        env_prefix = "kiara_"
-        extra = Extra.forbid
-        use_enum_values = True
+    model_config = SettingsConfigDict(
+        env_prefix="kiara_", extra="forbid", use_enum_values=True
+    )
 
     @classmethod
     def create_in_folder(cls, path: Union[Path, str]) -> "KiaraConfig":
-
         if isinstance(path, str):
             path = Path(path)
         path = path.absolute()
@@ -169,10 +417,11 @@ class KiaraConfig(BaseSettings):
         return config
 
     @classmethod
-    def load_from_file(cls, path: Union[Path, None] = None) -> "KiaraConfig":
-
+    def load_from_file(cls, path: Union[Path, str, None] = None) -> "KiaraConfig":
         if path is None:
             path = Path(KIARA_MAIN_CONFIG_FILE)
+        elif isinstance(path, str):
+            path = Path(path)
 
         if not path.exists():
             raise Exception(
@@ -208,6 +457,10 @@ class KiaraConfig(BaseSettings):
         description="The name of the default context to use if none is provided.",
         default=DEFAULT_CONTEXT_NAME,
     )
+    # default_store_type: Literal["sqlite", "filesystem"] = Field(
+    #     description="The default store type to use when creating new stores.",
+    #     default=DEFAULT_STORE_TYPE,
+    # )
     auto_generate_contexts: bool = Field(
         description="Whether to auto-generate requested contexts if they don't exist yet.",
         default=True,
@@ -217,21 +470,21 @@ class KiaraConfig(BaseSettings):
     )
 
     _contexts: Dict[uuid.UUID, "Kiara"] = PrivateAttr(default_factory=dict)
-    _available_context_files: Dict[str, Path] = PrivateAttr(default=None)
+    _available_context_files: Union[Dict[str, Path], None] = PrivateAttr(default=None)
     _context_data: Dict[str, KiaraContextConfig] = PrivateAttr(default_factory=dict)
     _config_path: Union[Path, None] = PrivateAttr(default=None)
 
-    @validator("context_search_paths")
+    @field_validator("context_search_paths")
+    @classmethod
     def validate_context_search_paths(cls, v):
-
         if not v or not v[0]:
             v = [KIARA_MAIN_CONTEXTS_PATH]
 
         return v
 
-    @root_validator(pre=True)
+    @model_validator(mode="before")
+    @classmethod
     def _set_paths(cls, values: Any):
-
         base_path = values.get("base_data_path", None)
         if not base_path:
             base_path = os.path.abspath(kiara_app_dirs.user_data_dir)
@@ -254,7 +507,6 @@ class KiaraConfig(BaseSettings):
 
     @property
     def available_context_names(self) -> Iterable[str]:
-
         if self._available_context_files is not None:
             return self._available_context_files.keys()
 
@@ -271,7 +523,6 @@ class KiaraConfig(BaseSettings):
 
     @property
     def context_configs(self) -> Mapping[str, KiaraContextConfig]:
-
         return {a: self.get_context_config(a) for a in self.available_context_names}
 
     def get_context_config(
@@ -279,7 +530,6 @@ class KiaraConfig(BaseSettings):
         context_name: Union[str, None] = None,
         auto_generate: Union[bool, None] = None,
     ) -> KiaraContextConfig:
-
         if auto_generate is None:
             auto_generate = self.auto_generate_contexts
 
@@ -297,9 +547,16 @@ class KiaraConfig(BaseSettings):
         if context_name in self._context_data.keys():
             return self._context_data[context_name]
 
+        assert self._available_context_files is not None
+
         context_file = self._available_context_files[context_name]
         context_data = get_data_from_file(context_file, content_type="yaml")
 
+        if not context_data:
+            raise KiaraException(
+                f"Empty/corrupted context file '{context_file.as_posix()}': delete file and try again."
+            )
+
         changed = False
         if "extra_pipeline_folders" in context_data.keys():
             epf = context_data.pop("extra_pipeline_folders")
@@ -320,7 +577,7 @@ class KiaraConfig(BaseSettings):
             )
             context_file.parent.mkdir(parents=True, exist_ok=True)
             with open(context_file, "wt") as f:
-                yaml.dump(context.dict(), f)
+                yaml.dump(context.model_dump(), f)
 
         context._context_config_path = context_file
 
@@ -328,120 +585,186 @@ class KiaraConfig(BaseSettings):
         return context
 
     def _validate_context(self, context_config: KiaraContextConfig) -> bool:
-
-        env_registry = EnvironmentRegistry.instance()
-        from kiara.models.runtime_environment.kiara import KiaraTypesRuntimeEnvironment
-
-        kiara_types: KiaraTypesRuntimeEnvironment = env_registry.environments["kiara_types"]  # type: ignore
-        available_archives = kiara_types.archive_types
-
         changed = False
-        if DEFAULT_DATA_STORE_MARKER not in context_config.archives.keys():
-            data_store_type = "filesystem_data_store"
-            assert data_store_type in available_archives.item_infos.keys()
 
-            data_store_id = ID_REGISTRY.generate(comment="default data store id")
-            data_archive_config = {
-                "archive_path": os.path.abspath(
-                    os.path.join(
-                        self.stores_base_path, data_store_type, str(data_store_id)
-                    )
-                )
-            }
-            data_store = KiaraArchiveConfig.construct(
-                archive_id=str(data_store_id),
-                archive_type=data_store_type,
-                config=data_archive_config,
+        sqlite_base_path = os.path.join(self.stores_base_path, "sqlite_stores")
+        filesystem_base_path = os.path.join(self.stores_base_path, "filesystem_stores")
+
+        def create_default_sqlite_archive_config(use_wal_mode: bool) -> Dict[str, Any]:
+            store_id = str(uuid.uuid4())
+            file_name = f"{store_id}.karchive"
+            archive_path = Path(
+                os.path.abspath(os.path.join(sqlite_base_path, file_name))
             )
-            context_config.archives[DEFAULT_DATA_STORE_MARKER] = data_store
 
+            if archive_path.exists():
+                raise Exception(
+                    f"Archive path '{archive_path.as_posix()}' already exists."
+                )
+
+            archive_path.parent.mkdir(exist_ok=True, parents=True)
+
+            # Connect to the SQLite database (or create it if it doesn't exist)
+            import sqlite3
+
+            conn = sqlite3.connect(archive_path)
+
+            # Create a cursor object
+            c = conn.cursor()
+            # Create table
+            c.execute(
+                """CREATE TABLE archive_metadata
+                         (key text PRIMARY KEY , value text NOT NULL)"""
+            )
+            c.execute(
+                "INSERT INTO archive_metadata VALUES ('archive_id', ?)", (store_id,)
+            )
+            conn.commit()
+            conn.close()
+
+            return {
+                "sqlite_db_path": archive_path.as_posix(),
+                "use_wal_mode": use_wal_mode,
+            }
+
+        default_sqlite_config: Union[Dict[str, Any], None] = None
+
+        use_wal_mode: bool = True
+        default_store_type = "sqlite"
+
+        if default_store_type == "auto":
+            # if windows, we want sqlite as default, because although it's slower, it does not
+            # need the user to enable developer mode
+            if os.name == "nt":
+                data_store_type = "sqlite"
+            else:
+                data_store_type = "filesystem"
+
+            metadata_store_type = "sqlite"
+            alias_store_type = "sqlite"
+            job_store_type = "sqlite"
+            workflow_store_type = "sqlite"
+        elif default_store_type == "filesystem":
+            metadata_store_type = "filesystem"
+            data_store_type = "filesystem"
+            alias_store_type = "filesystem"
+            job_store_type = "filesystem"
+            workflow_store_type = "filesystem"
+        elif default_store_type == "sqlite":
+            metadata_store_type = "sqlite"
+            data_store_type = "sqlite"
+            alias_store_type = "sqlite"
+            job_store_type = "sqlite"
+            workflow_store_type = "sqlite"
+        else:
+            raise Exception(f"Unknown store type: {default_store_type}")
+
+        if DEFAULT_METADATA_STORE_MARKER not in context_config.archives.keys():
+            if metadata_store_type == "sqlite":
+                default_sqlite_config = create_default_sqlite_archive_config(
+                    use_wal_mode=use_wal_mode
+                )
+                metaddata_store = KiaraArchiveConfig(
+                    archive_type="sqlite_metadata_store", config=default_sqlite_config
+                )
+            elif metadata_store_type == "filesystem":
+                default_sqlite_config = create_default_sqlite_archive_config(
+                    use_wal_mode=use_wal_mode
+                )
+                metaddata_store = KiaraArchiveConfig(
+                    archive_type="sqlite_metadata_store", config=default_sqlite_config
+                )
+            else:
+                raise Exception(
+                    f"Can't create default metadata store: invalid default store type '{metadata_store_type}'"
+                )
+
+            context_config.archives[DEFAULT_METADATA_STORE_MARKER] = metaddata_store
+            changed = True
+
+        if DEFAULT_DATA_STORE_MARKER not in context_config.archives.keys():
+            if data_store_type == "sqlite":
+                if default_sqlite_config is None:
+                    default_sqlite_config = create_default_sqlite_archive_config(
+                        use_wal_mode=use_wal_mode
+                    )
+
+                data_store = KiaraArchiveConfig(
+                    archive_type="sqlite_data_store", config=default_sqlite_config
+                )
+            elif data_store_type == "filesystem":
+                data_store_type = "filesystem_data_store"
+                data_store = create_default_store_config(
+                    store_type=data_store_type,
+                    stores_base_path=os.path.join(filesystem_base_path, "data"),
+                )
+            else:
+                raise Exception(
+                    f"Can't create default data store: invalid default store type '{data_store_type}'."
+                )
+
+            context_config.archives[DEFAULT_DATA_STORE_MARKER] = data_store
             changed = True
 
         if DEFAULT_JOB_STORE_MARKER not in context_config.archives.keys():
-            job_store_type = "filesystem_job_store"
-            assert job_store_type in available_archives.item_infos.keys()
-
-            job_store_id = ID_REGISTRY.generate(comment="default job store id")
-            job_archive_config = {
-                "archive_path": os.path.abspath(
-                    os.path.join(
-                        self.stores_base_path, job_store_type, str(job_store_id)
+            if job_store_type == "sqlite":
+                if default_sqlite_config is None:
+                    default_sqlite_config = create_default_sqlite_archive_config(
+                        use_wal_mode=use_wal_mode
                     )
-                )
-            }
-            job_store = KiaraArchiveConfig.construct(
-                archive_id=str(job_store_id),
-                archive_type=job_store_type,
-                config=job_archive_config,
-            )
-            context_config.archives[DEFAULT_JOB_STORE_MARKER] = job_store
 
+                job_store = KiaraArchiveConfig(
+                    archive_type="sqlite_job_store", config=default_sqlite_config
+                )
+            elif job_store_type == "filesystem":
+                job_store_type = "filesystem_job_store"
+                job_store = create_default_store_config(
+                    store_type=job_store_type,
+                    stores_base_path=os.path.join(filesystem_base_path, "jobs"),
+                )
+            else:
+                raise Exception(
+                    f"Can't create default job store: invalid default store type '{job_store_type}'."
+                )
+
+            context_config.archives[DEFAULT_JOB_STORE_MARKER] = job_store
             changed = True
 
         if DEFAULT_ALIAS_STORE_MARKER not in context_config.archives.keys():
-
-            alias_store_type = "filesystem_alias_store"
-            assert alias_store_type in available_archives.item_infos.keys()
-            alias_store_id = ID_REGISTRY.generate(comment="default alias store id")
-            alias_store_config = {
-                "archive_path": os.path.abspath(
-                    os.path.join(
-                        self.stores_base_path, alias_store_type, str(alias_store_id)
+            if alias_store_type == "sqlite":
+                if default_sqlite_config is None:
+                    default_sqlite_config = create_default_sqlite_archive_config(
+                        use_wal_mode=use_wal_mode
                     )
-                )
-            }
-            alias_store = KiaraArchiveConfig.construct(
-                archive_id=str(alias_store_id),
-                archive_type=alias_store_type,
-                config=alias_store_config,
-            )
-            context_config.archives[DEFAULT_ALIAS_STORE_MARKER] = alias_store
 
+                alias_store = KiaraArchiveConfig(
+                    archive_type="sqlite_alias_store", config=default_sqlite_config
+                )
+            elif alias_store_type == "filesystem":
+                alias_store_type = "filesystem_alias_store"
+                alias_store = create_default_store_config(
+                    store_type=alias_store_type,
+                    stores_base_path=os.path.join(filesystem_base_path, "aliases"),
+                )
+            else:
+                raise Exception(
+                    f"Can't create default alias store: invalid default store type '{alias_store_type}'."
+                )
+
+            context_config.archives[DEFAULT_ALIAS_STORE_MARKER] = alias_store
             changed = True
 
         if DEFAULT_WORKFLOW_STORE_MARKER not in context_config.archives.keys():
+            # TODO: impolement sqlite type, or remove workflows entirely
 
             workflow_store_type = "filesystem_workflow_store"
-            assert workflow_store_type in available_archives.item_infos.keys()
-            workflow_store_id = ID_REGISTRY.generate(
-                comment="default workflow store id"
-            )
-            workflow_store_config = {
-                "archive_path": os.path.abspath(
-                    os.path.join(
-                        self.stores_base_path,
-                        workflow_store_type,
-                        str(workflow_store_id),
-                    )
-                )
-            }
-            workflow_store = KiaraArchiveConfig.construct(
-                archive_id=str(workflow_store_id),
-                archive_type=workflow_store_type,
-                config=workflow_store_config,
+            # workflow_store_type = "sqlite_workflow_store"
+
+            workflow_store = create_default_store_config(
+                store_type=workflow_store_type,
+                stores_base_path=os.path.join(filesystem_base_path, "workflows"),
             )
             context_config.archives[DEFAULT_WORKFLOW_STORE_MARKER] = workflow_store
-
-            changed = True
-
-        if METADATA_DESTINY_STORE_MARKER not in context_config.archives.keys():
-            destiny_store_type = "filesystem_destiny_store"
-            assert destiny_store_type in available_archives.item_infos.keys()
-            destiny_store_id = ID_REGISTRY.generate(comment="default destiny store id")
-            destiny_store_config = {
-                "archive_path": os.path.abspath(
-                    os.path.join(
-                        self.stores_base_path, destiny_store_type, str(destiny_store_id)
-                    )
-                )
-            }
-            destiny_store = KiaraArchiveConfig.construct(
-                archive_id=str(destiny_store_id),
-                archive_type=destiny_store_type,
-                config=destiny_store_config,
-            )
-            context_config.archives[METADATA_DESTINY_STORE_MARKER] = destiny_store
-
             changed = True
 
         return changed
@@ -449,42 +772,54 @@ class KiaraConfig(BaseSettings):
     def create_context_config(
         self, context_alias: Union[str, None] = None
     ) -> KiaraContextConfig:
-
         if not context_alias:
             context_alias = DEFAULT_CONTEXT_NAME
+
         if context_alias in self.available_context_names:
             raise Exception(
                 f"Can't create kiara context '{context_alias}': context with that alias already registered."
             )
 
-        if os.path.sep in context_alias:
-            raise Exception(
-                f"Can't create context with alias '{context_alias}': no special characters allowed."
+        if context_alias.endswith(".kontext"):
+            context_db_file = Path(context_alias)
+            context_config: KiaraContextConfig = (
+                KiaraContextConfig.create_from_sqlite_db(db_path=context_db_file)
+            )
+            self._validate_context(context_config=context_config)
+            context_config._context_config_path = context_db_file
+        else:
+            if os.path.sep in context_alias:
+                raise Exception(
+                    f"Can't create context with alias '{context_alias}': no special characters allowed."
+                )
+
+            context_file = (
+                Path(os.path.join(self.context_search_paths[0]))
+                / f"{context_alias}.yaml"
             )
 
-        context_file = (
-            Path(os.path.join(self.context_search_paths[0])) / f"{context_alias}.yaml"
-        )
+            archives: Dict[str, KiaraArchiveConfig] = {}
+            # create_default_archives(kiara_config=self)
+            context_id = ID_REGISTRY.generate(
+                obj_type=KiaraContextConfig,
+                comment=f"new kiara context '{context_alias}'",
+            )
 
-        archives: Dict[str, KiaraArchiveConfig] = {}
-        # create_default_archives(kiara_config=self)
-        context_id = ID_REGISTRY.generate(
-            obj_type=KiaraContextConfig, comment=f"new kiara context '{context_alias}'"
-        )
+            context_config = KiaraContextConfig(
+                context_id=str(context_id), archives=archives, extra_pipelines=[]
+            )
 
-        context_config = KiaraContextConfig(
-            context_id=str(context_id), archives=archives, extra_pipelines=[]
-        )
+            self._validate_context(context_config=context_config)
 
-        self._validate_context(context_config=context_config)
+            context_file.parent.mkdir(parents=True, exist_ok=True)
+            with open(context_file, "wt") as f:
+                yaml.dump(context_config.model_dump(), f)
 
-        context_file.parent.mkdir(parents=True, exist_ok=True)
-        with open(context_file, "wt") as f:
-            yaml.dump(context_config.dict(), f)
+            context_config._context_config_path = context_file
 
-        context_config._context_config_path = context_file
+            assert self._available_context_files is not None
+            self._available_context_files[context_alias] = context_file
 
-        self._available_context_files[context_alias] = context_file
         self._context_data[context_alias] = context_config
 
         return context_config
@@ -494,20 +829,31 @@ class KiaraConfig(BaseSettings):
         context: Union[None, str, uuid.UUID, Path] = None,
         extra_pipelines: Union[None, str, Iterable[str]] = None,
     ) -> "Kiara":
-
         if not context:
             context = self.default_context
         else:
             with contextlib.suppress(Exception):
                 context = uuid.UUID(context)  # type: ignore
 
-        if isinstance(context, str) and os.path.exists(context):
-            context = Path(context)
+        if isinstance(context, str) and (
+            os.path.exists(context) or context.endswith(".kontext")
+        ):
+            context = Path(os.path.abspath(context))
 
         if isinstance(context, Path):
-            with context.open("rt") as f:
-                data = yaml.load(f)
-            context_config = KiaraContextConfig(**data)
+            if context.name.endswith(".kontext"):
+                context_config = KiaraContextConfig.create_from_sqlite_db(
+                    db_path=context
+                )
+            else:
+                try:
+                    with context.open("rt") as f:
+                        data = yaml.load(f)
+                except Exception as e:
+                    raise KiaraException(
+                        f"Can't read context from file '{context}': {e}"
+                    )
+                context_config = KiaraContextConfig(**data)
         elif isinstance(context, str):
             context_config = self.get_context_config(context_name=context)
         elif isinstance(context, uuid.UUID):
@@ -546,18 +892,20 @@ class KiaraConfig(BaseSettings):
 
         path.parent.mkdir(parents=True, exist_ok=True)
 
+        data = self.model_dump(
+            exclude={
+                "context",
+                "auto_generate_contexts",
+                "stores_base_path",
+                "context_search_paths",
+                "default_context",
+                "runtime_config",
+            }
+        )
+
         with path.open("wt") as f:
             yaml.dump(
-                self.dict(
-                    exclude={
-                        "context",
-                        "auto_generate_contexts",
-                        "stores_base_path",
-                        "context_search_paths",
-                        "default_context",
-                        "runtime_config",
-                    }
-                ),
+                data,
                 f,
             )
 
@@ -565,7 +913,8 @@ class KiaraConfig(BaseSettings):
 
     def delete(
         self, context_name: Union[str, None] = None, dry_run: bool = True
-    ) -> "ContextInfo":
+    ) -> Union["ContextInfo", None]:
+        """Deletes the context with the specified name."""
 
         if context_name is None:
             context_name = self.default_context
@@ -576,20 +925,27 @@ class KiaraConfig(BaseSettings):
         context_config = self.get_context_config(
             context_name=context_name, auto_generate=False
         )
-        kiara = Kiara(config=context_config, runtime_config=self.runtime_config)
 
-        context_summary = ContextInfo.create_from_context(
-            kiara=kiara, context_name=context_name
-        )
+        context_summary = None
 
-        if dry_run:
-            return context_summary
+        try:
+            kiara = Kiara(config=context_config, runtime_config=self.runtime_config)
 
-        for archive in kiara.get_all_archives().keys():
-            archive.delete_archive(archive_id=archive.archive_id)
+            context_summary = ContextInfo.create_from_context(
+                kiara=kiara, context_name=context_name
+            )
 
-        if context_config._context_config_path is not None:
-            os.unlink(context_config._context_config_path)
+            if dry_run:
+                return context_summary
+
+            for archive in kiara.get_all_archives().keys():
+                archive.delete_archive(archive_id=archive.archive_id)
+        except Exception as e:
+            log_message("delete.context.error", context_name=context_name, error=e)
+
+        if not dry_run:
+            if context_config._context_config_path is not None:
+                os.unlink(context_config._context_config_path)
 
         return context_summary
 
