@@ -9,9 +9,11 @@ import abc
 from typing import (
     TYPE_CHECKING,
     Any,
+    ClassVar,
     Dict,
     Generic,
     Iterable,
+    List,
     Mapping,
     Type,
     TypeVar,
@@ -19,7 +21,7 @@ from typing import (
 )
 
 import orjson
-from pydantic import BaseModel
+from pydantic import BaseModel, Field
 from rich import box
 from rich.console import RenderableType
 from rich.syntax import Syntax
@@ -29,7 +31,7 @@ from kiara.data_types import TYPE_CONFIG_CLS, TYPE_PYTHON_CLS, DataType, DataTyp
 from kiara.defaults import INVALID_HASH_MARKER, SpecialValue
 from kiara.exceptions import DataTypeUnknownException, KiaraProcessingException
 from kiara.models import KiaraModel
-from kiara.models.data_types import DictModel
+from kiara.models.data_types import KiaraDict
 from kiara.models.python_class import PythonClass
 from kiara.models.rendering import RenderScene, RenderValueResult
 from kiara.models.values import DataTypeCharacteristics
@@ -46,9 +48,9 @@ SCALAR_CHARACTERISTICS = DataTypeCharacteristics(
 
 
 class NoneType(DataType[SpecialValue, DataTypeConfig]):
-    """Type indicating a 'None' value"""
+    """Type indicating a 'None' value."""
 
-    _data_type_name = "none"
+    _data_type_name: ClassVar[str] = "none"
 
     @classmethod
     def python_class(cls) -> Type:
@@ -69,20 +71,20 @@ class NoneType(DataType[SpecialValue, DataTypeConfig]):
     def pretty_print_as__string(
         self, value: "Value", render_config: Mapping[str, Any]
     ) -> Any:
-
         return "None"
 
     def pretty_print_as__terminal_renderable(
         self, value: "Value", render_config: Mapping[str, Any]
     ):
-
         return "None"
 
 
 class AnyType(
-    DataType[TYPE_PYTHON_CLS, DataTypeConfig], Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS]
+    DataType[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS],
+    Generic[TYPE_PYTHON_CLS, TYPE_CONFIG_CLS],
 ):
-    """'Any' type, the parent type for most other types.
+    """
+    'Any' type, the parent type for most other types.
 
     This type acts as the parents for all (or at least most) non-internal value types. There are some generic operations
     (like 'persist_value', or 'pretty_print') which are implemented for this type, so it's descendents have a fallback
@@ -91,7 +93,7 @@ class AnyType(
     this might or might not change).
     """
 
-    _data_type_name = "any"
+    _data_type_name: ClassVar[str] = "any"
 
     @classmethod
     def python_class(cls) -> Type:
@@ -99,10 +101,11 @@ class AnyType(
 
     def pretty_print_as__string(
         self, value: "Value", render_config: Mapping[str, Any]
-    ) -> Any:
-
+    ) -> str:
         if hasattr(self, "_pretty_print_as__string"):
-            return self._pretty_print_as__string(value=value, render_config=render_config)  # type: ignore
+            return self._pretty_print_as__string(  # type: ignore
+                value=value, render_config=render_config
+            )  # type: ignore
 
         try:
             return str(value.data)
@@ -111,10 +114,11 @@ class AnyType(
 
     def pretty_print_as__terminal_renderable(
         self, value: "Value", render_config: Mapping[str, Any]
-    ):
-
+    ) -> RenderableType:
         if hasattr(self, "_pretty_print_as__terminal_renderable"):
-            return self._pretty_print_as__terminal_renderable(value=value, render_config=render_config)  # type: ignore
+            return self._pretty_print_as__terminal_renderable(  # type: ignore
+                value=value, render_config=render_config
+            )  # type: ignore
 
         try:
             data = value.data
@@ -150,18 +154,18 @@ class AnyType(
 
     def render_as__string(
         self, value: "Value", render_config: Mapping[str, Any], manifest: "Manifest"
-    ):
+    ) -> str:
         if hasattr(self, "_render_as__string"):
-            return self._render_as__string(value=value, render_scene=render_config, manifest=manifest)  # type: ignore
+            return self._render_as__string(  # type: ignore
+                value=value, render_scene=render_config, manifest=manifest
+            )  # type: ignore
         else:
             return self.pretty_print_as__string(value=value, render_config={})
 
     def render_as__terminal_renderable(
         self, value: "Value", render_config: Mapping[str, Any], manifest: "Manifest"
     ) -> RenderableType:
-
         if not hasattr(self, "_render_as__terminal_renderable"):
-
             try:
                 value.data
                 return self.render_as__string(
@@ -173,32 +177,33 @@ class AnyType(
                 )
 
         else:
-            return self._render_as__terminal_renderable(value=value, render_config=render_config, manifest=manifest)  # type: ignore
+            return self._render_as__terminal_renderable(  # type: ignore
+                value=value, render_config=render_config, manifest=manifest
+            )  # type: ignore
 
 
 class BytesType(AnyType[bytes, DataTypeConfig]):
     """An array of bytes."""
 
-    _data_type_name = "bytes"
+    _data_type_name: ClassVar[str] = "bytes"
 
     @classmethod
     def python_class(cls) -> Type:
         return bytes
 
     def serialize(self, data: bytes) -> "SerializedData":
-
         _data = {"bytes": {"type": "chunk", "chunk": data, "codec": "raw"}}
 
         serialized_data = {
             "data_type": self.data_type_name,
-            "data_type_config": self.type_config.dict(),
+            "data_type_config": self.type_config.model_dump(),
             "data": _data,
             "serialization_profile": "raw",
             "metadata": {
                 "environment": {},
                 "deserialize": {
                     "python_object": {
-                        "module_name": "load.bytes",
+                        "module_type": "load.bytes",
                         "module_config": {
                             "value_type": "bytes",
                             "target_profile": "python_object",
@@ -211,34 +216,48 @@ class BytesType(AnyType[bytes, DataTypeConfig]):
         from kiara.models.values.value import SerializationResult
 
         serialized = SerializationResult(**serialized_data)
+
         return serialized
 
     def _pretty_print_as__string(
         self, value: "Value", render_config: Mapping[str, Any]
     ) -> Any:
-
         data: bytes = value.data
         return data.decode()
 
 
-class StringType(AnyType[str, DataTypeConfig]):
-    """A string."""
+class StringTypeConfig(DataTypeConfig):
+    allowed_strings: Union[None, List[str]] = Field(
+        description="A list of allowed strings, if empty or None, any string is allowed.",
+        default=None,
+    )
 
-    _data_type_name = "string"
+
+class StringType(AnyType[str, StringTypeConfig]):
+    """A string.
+
+    Can be configured to only allow a list of specific strings by using the `allowed_strings` configuration option.
+    """
+
+    _data_type_name: ClassVar[str] = "string"
+
+    @classmethod
+    def data_type_config_class(cls) -> Type[TYPE_CONFIG_CLS]:
+        """The Python class that holds the (optional) configuration for a data type instance."""
+        return StringTypeConfig  # type: ignore
 
     @classmethod
     def python_class(cls) -> Type:
         return str
 
     def serialize(self, data: str) -> "SerializedData":
-
         _data = {
             "string": {"type": "chunk", "chunk": data.encode("utf-8"), "codec": "raw"}
         }
 
         serialized_data = {
             "data_type": self.data_type_name,
-            "data_type_config": self.type_config.dict(),
+            "data_type_config": self.type_config.model_dump(),
             "data": _data,
             "serialization_profile": "raw",
             "metadata": {
@@ -263,11 +282,22 @@ class StringType(AnyType[str, DataTypeConfig]):
     def _retrieve_characteristics(self) -> DataTypeCharacteristics:
         return SCALAR_CHARACTERISTICS
 
-    def _validate(cls, value: Any) -> None:
+    def parse_python_obj(self, data: Any) -> str:
+        # TODO: check if this is actually ok to do always
+        return str(data)
 
+    def _validate(self, value: Any) -> None:
         if not isinstance(value, str):
             raise ValueError(f"Invalid type '{type(value)}': string required")
 
+        if (
+            self.type_config.allowed_strings
+            and value not in self.type_config.allowed_strings
+        ):
+            raise ValueError(
+                f"Invalid value '{value}': not in allowed values {self.type_config.allowed_strings}"
+            )
+
     def pretty_print_as__bytes(self, value: "Value", render_config: Mapping[str, Any]):
         value_str: str = value.data
         return value_str.encode()
@@ -276,7 +306,7 @@ class StringType(AnyType[str, DataTypeConfig]):
 class BooleanType(AnyType[bool, DataTypeConfig]):
     "A boolean."
 
-    _data_type_name = "boolean"
+    _data_type_name: ClassVar[str] = "boolean"
 
     @classmethod
     def python_class(cls) -> Type:
@@ -296,7 +326,6 @@ class BooleanType(AnyType[bool, DataTypeConfig]):
     #     return 1 if data else 0
 
     def parse_python_obj(self, data: Any) -> bool:
-
         if data is True or data is False:
             return data
         elif data == 0:
@@ -314,19 +343,20 @@ class BooleanType(AnyType[bool, DataTypeConfig]):
         pass
 
 
-class DictValueType(AnyType[DictModel, DataTypeConfig]):
-    """A dictionary.
+class DictValueType(AnyType[KiaraDict, DataTypeConfig]):
+    """
+    A dictionary.
 
     In addition to the actual dictionary value, this value type comes also with an optional schema, describing the
     dictionary. In case no schema was attached, a simple generic one is attached. This data type is backed by the
     [DictModel][kiara_plugin.core_types.models.DictModel] class.
     """
 
-    _data_type_name = "dict"
+    _data_type_name: ClassVar[str] = "dict"
 
     @classmethod
     def python_class(cls) -> Type:
-        return DictModel
+        return KiaraDict
 
     # def calculate_size(self, data: DictModel) -> int:
     #     return data.size
@@ -337,21 +367,19 @@ class DictValueType(AnyType[DictModel, DataTypeConfig]):
     def _retrieve_characteristics(self) -> DataTypeCharacteristics:
         return DataTypeCharacteristics(is_scalar=False, is_json_serializable=True)
 
-    def parse_python_obj(self, data: Any) -> DictModel:
-
+    def parse_python_obj(self, data: Any) -> KiaraDict:
         python_cls = data.__class__
         dict_data = None
         schema = None
 
         if isinstance(data, Mapping):
-
             if (
                 len(data) == 3
                 and "dict_data" in data.keys()
                 and "data_schema" in data.keys()
                 and "python_class" in data.keys()
             ):
-                dict_model = DictModel(
+                dict_model = KiaraDict(
                     dict_data=data["dict_data"],
                     data_schema=data["data_schema"],
                     python_class=data["python_class"],
@@ -361,14 +389,21 @@ class DictValueType(AnyType[DictModel, DataTypeConfig]):
             schema = {"title": "dict", "type": "object"}
             dict_data = data
         elif isinstance(data, BaseModel):
-            dict_data = data.dict()
-            schema = data.schema()
+            dict_data = data.model_dump()
+            schema = data.model_json_schema()
         elif isinstance(data, str):
             try:
                 dict_data = orjson.loads(data)
                 schema = {"title": "dict", "type": "object"}
             except Exception:
-                pass
+                try:
+                    from kiara.utils.cli import dict_from_cli_args
+
+                    tokens = data.split(",")
+                    dict_data = dict_from_cli_args(*tokens)
+                    schema = {"title": "dict", "type": "object"}
+                except Exception:
+                    pass
 
         if dict_data is None or schema is None:
             raise Exception(f"Invalid data for value type 'dict': {data}")
@@ -376,13 +411,12 @@ class DictValueType(AnyType[DictModel, DataTypeConfig]):
         result = {
             "dict_data": dict_data,
             "data_schema": schema,
-            "python_class": PythonClass.from_class(python_cls).dict(),
+            "python_class": PythonClass.from_class(python_cls).model_dump(),
         }
-        return DictModel(**result)
+        return KiaraDict(**result)
 
-    def _validate(self, data: DictModel) -> None:
-
-        if not isinstance(data, DictModel):
+    def _validate(self, data: KiaraDict) -> None:
+        if not isinstance(data, KiaraDict):
             raise Exception(f"Invalid type: {type(data)}.")
 
     # def render_as__string(self, value: Value, render_config: Mapping[str, Any]) -> str:
@@ -393,15 +427,16 @@ class DictValueType(AnyType[DictModel, DataTypeConfig]):
     def _pretty_print_as__terminal_renderable(
         self, value: "Value", render_config: Mapping[str, Any]
     ):
-
         show_schema = render_config.get("show_schema", True)
 
         table = Table(show_header=False, box=box.SIMPLE)
         table.add_column("key", style="i")
         table.add_column("value")
 
-        data: DictModel = value.data
-        data_json = orjson_dumps(data.dict_data, option=orjson.OPT_INDENT_2)
+        data: KiaraDict = value.data
+        data_json = orjson_dumps(
+            data.dict_data, option=orjson.OPT_INDENT_2 | orjson.OPT_NON_STR_KEYS
+        )
         table.add_row(
             "dict data", Syntax(data_json, "json", background_color="default")
         )
@@ -414,15 +449,13 @@ class DictValueType(AnyType[DictModel, DataTypeConfig]):
 
         return table
 
-    def serialize(self, data: DictModel) -> "SerializedData":
-
-        result = self.serialize_as_json(data.dict())
+    def serialize(self, data: KiaraDict) -> "SerializedData":
+        result = self.serialize_as_json(data.model_dump())
         return result
 
     def render_as__terminal_renderable(
         self, value: "Value", render_config: Mapping[str, Any], manifest: "Manifest"
     ) -> RenderableType:
-
         render_item = render_config.get("render_item", "data")
         width = render_config.get("display_width", 0)
 
@@ -443,7 +476,7 @@ class DictValueType(AnyType[DictModel, DataTypeConfig]):
 
             rendered = Syntax(json_string, "json")
             related_scenes["data"] = None
-            related_scenes["schema"] = RenderScene.construct(
+            related_scenes["schema"] = RenderScene(
                 title="schema",
                 description="The (json) schema for the data.",
                 manifest_hash=manifest.manifest_hash,
@@ -455,7 +488,7 @@ class DictValueType(AnyType[DictModel, DataTypeConfig]):
             json_string = orjson_dumps(schema, option=orjson.OPT_INDENT_2)
 
             rendered = Syntax(json_string, "json")
-            related_scenes["data"] = RenderScene.construct(
+            related_scenes["data"] = RenderScene(
                 title="data",
                 description="The actual data of the dictionary.",
                 manifest_hash=manifest.manifest_hash,
@@ -483,34 +516,33 @@ class DictValueType(AnyType[DictModel, DataTypeConfig]):
 KIARA_MODEL_CLS = TypeVar("KIARA_MODEL_CLS", bound=KiaraModel)
 
 
-class KiaraModelValueType(
+class KiaraModelValueBaseType(
     AnyType[KIARA_MODEL_CLS, TYPE_CONFIG_CLS], Generic[KIARA_MODEL_CLS, TYPE_CONFIG_CLS]
 ):
-    """A value type that is used internally.
+    """
+    A value type that is used internally.
 
     This type should not be used by user-facing modules and/or operations.
     """
 
-    _data_type_name = None  # type: ignore
+    _data_type_name: ClassVar[str] = None  # type: ignore
 
     @classmethod
-    def data_type_config_class(cls) -> Type[DataTypeConfig]:
-        return DataTypeConfig
+    def data_type_config_class(cls) -> Type[TYPE_CONFIG_CLS]:
+        return DataTypeConfig  # type: ignore
 
     @abc.abstractmethod
     def create_model_from_python_obj(self, data: Any) -> KIARA_MODEL_CLS:
         pass
 
     def parse_python_obj(self, data: Any) -> KIARA_MODEL_CLS:
-
         if isinstance(data, self.__class__.python_class()):
             return data  # type: ignore
 
-        data = self.create_model_from_python_obj(data)
-        return data
+        _data = self.create_model_from_python_obj(data)
+        return _data
 
     def _validate(self, data: KiaraModel) -> None:
-
         if not isinstance(data, self.__class__.python_class()):
             raise Exception(
                 f"Invalid type '{type(data)}', must be: {self.__class__.python_class().__name__}, or subclass."
