@@ -14,7 +14,8 @@ from kiara.models.values.value import SerializedData, Value
 
 
 class PythonObjectType(InternalType[object, DataTypeConfig]):
-    """A 'plain' Python object.
+    """
+    A 'plain' Python object.
 
     This data type is mostly used internally, for hading over data in (de-)serialization operations.
     """
@@ -36,5 +37,4 @@ class PythonObjectType(InternalType[object, DataTypeConfig]):
     def _pretty_print_as__terminal_renderable(
         self, value: Value, render_config: Mapping[str, Any]
     ):
-
         return str(value.data)
