@@ -9,8 +9,8 @@ import os
 import typing
 import uuid
 from enum import Enum
+from pathlib import Path
 
-import mmh3
 from appdirs import AppDirs
 
 kiara_app_dirs = AppDirs("kiara", "DHARPA")
@@ -37,6 +37,18 @@ KIARA_DEV_CONFIG_FILE = os.path.join(
     kiara_app_dirs.user_config_dir, KIARA_DEV_CONFIG_FILE_NAME
 )
 KIARA_MAIN_CONTEXTS_PATH = os.path.join(kiara_app_dirs.user_config_dir, "contexts")
+KIARA_MAIN_CONTEXT_DATA_PATH = os.path.join(
+    kiara_app_dirs.user_data_dir, "context_data"
+)
+KIARA_MAIN_CONTEXT_LOCKS_PATH = os.path.join(
+    kiara_app_dirs.user_data_dir, "context_locks"
+)
+
+
+KIARA_DEFAULT_STAGES_EXTRACTION_TYPE = "early"
+
+INIT_EXAMPLE_NAME = "init"
+
 # USER_PIPELINES_FOLDER = os.path.join(kiara_app_dirs.user_config_dir, "pipelines")
 
 
@@ -69,11 +81,23 @@ INVALID_VALUE_NAMES = [
     "kiara",
     "callbacks",
 ]
+INVALID_ALIAS_NAMES = [
+    "kiara",
+    "__default__",
+    "alias",
+    "value",
+    "value_id",
+    "kiarchive",
+]
 """List of reserved names, inputs/outputs can't use those."""
+DEFAULT_STORE_MARKER = "default_store"
 
 DEFAULT_DATA_STORE_MARKER = "default_data_store"
 """Name for the default context data store."""
 
+DEFAULT_METADATA_STORE_MARKER = "default_metadata_store"
+"""Name for the default context metadata store."""
+
 DEFAULT_JOB_STORE_MARKER = "default_job_store"
 """Name for the default context job store."""
 
@@ -83,7 +107,7 @@ DEFAULT_ALIAS_STORE_MARKER = "default_alias_store"
 DEFAULT_WORKFLOW_STORE_MARKER = "default_workflow_store"
 """Name for the default context workflow store."""
 
-METADATA_DESTINY_STORE_MARKER = "metadata"
+METADATA_PROPERTY_MARKER = "metadata"
 """Name for the default context destiny store."""
 
 PIPELINE_PARENT_MARKER = "__pipeline__"
@@ -111,9 +135,17 @@ KIARA_MODULE_METADATA_ATTRIBUTE = "KIARA_METADATA"
 
 KIARA_DEFAULT_ROOT_NODE_ID = "__self__"
 
+KIARA_SQLITE_STORE_EXTENSION = "kiara"
+
+VALUE_ATTR_DELIMITER = "::"
+VALID_VALUE_QUERY_CATEGORIES = ["data", "properties"]
+
+CHUNK_CACHE_BASE_DIR = Path(kiara_app_dirs.user_cache_dir) / "data" / "chunks"
+CHUNK_CACHE_DIR_DEPTH = 2
+CHUNK_CACHE_DIR_WIDTH = 1
+
 
 class SpecialValue(Enum):
-
     NOT_SET = "__not_set__"
     NO_VALUE = "__no_value__"
 
@@ -160,14 +192,12 @@ INVALID_SIZE_MARKER = -1
 NO_SERIALIZATION_MARKER = "-- serialization not supported --"
 KIARA_ROOT_TYPE_NAME = "__kiara__"
 
-SERIALIZED_DATA_TYPE_NAME = "serialized_data"
-LOAD_CONFIG_DATA_TYPE_NAME = "load_config"
+SERIALIZED_data_type_name = "serialized_data"
+LOAD_CONFIG_data_type_name = "load_config"
 
 PYDANTIC_USE_CONSTRUCT: bool = False
 STRICT_CHECKS: bool = False
 
-KIARA_HASH_FUNCTION = mmh3.hash
-
 ANY_TYPE_NAME = "any"
 
 DEFAULT_ENV_HASH_KEY = "default"
@@ -236,6 +266,12 @@ ARRAY_MODEL_CATEOGORY_ID = "instance.model.array"
 TABLE_MODEL_CATEOGORY_ID = "instance.model.table"
 DEFAULT_CONTEXT_NAME = "default"
 
+KIARA_MODEL_ID_KEY = "kiara_model_id"
+KIARA_MODEL_DATA_KEY = "data"
+KIARA_MODEL_SCHEMA_KEY = "schema"
+
+ENVIRONMENT_MARKER_KEY = "environment"
+"""Constant string to indicate this is a metadata entry of type 'environment'."""
 
 SYMLINK_ISSUE_MSG = """Your operating system does not support symlinks, which is a requirement for kiara to work.
 
@@ -260,3 +296,58 @@ OFFICIAL_KIARA_PLUGINS = [
     "network_analysis",
     "language_processing",
 ]
+
+
+class CHUNK_COMPRESSION_TYPE(Enum):
+    NONE = 0
+    ZSTD = 1
+    LZMA = 2
+    LZ4 = 3
+
+
+DEFAULT_CHUNK_COMPRESSION = CHUNK_COMPRESSION_TYPE.ZSTD
+
+ARCHIVE_NAME_MARKER = "archive_name"
+DATA_ARCHIVE_DEFAULT_VALUE_MARKER = "default_value"
+TABLE_NAME_ARCHIVE_METADATA = "archive_metadata"
+TABLE_NAME_DATA_METADATA = "data_value_metadata"
+TABLE_NAME_DATA_SERIALIZATION_METADATA = "data_serialization_metadata"
+TABLE_NAME_DATA_CHUNKS = "data_chunks"
+TABLE_NAME_DATA_PEDIGREE = "data_value_pedigree"
+TABLE_NAME_DATA_DESTINIES = "data_value_destiny"
+REQUIRED_TABLES_DATA_ARCHIVE = {
+    TABLE_NAME_ARCHIVE_METADATA,
+    TABLE_NAME_DATA_METADATA,
+    TABLE_NAME_DATA_SERIALIZATION_METADATA,
+    TABLE_NAME_DATA_CHUNKS,
+    TABLE_NAME_DATA_PEDIGREE,
+    TABLE_NAME_DATA_DESTINIES,
+}
+
+TABLE_NAME_ALIASES = "aliases"
+REQUIRED_TABLES_ALIAS_ARCHIVE = {
+    TABLE_NAME_ARCHIVE_METADATA,
+    TABLE_NAME_ALIASES,
+}
+
+TABLE_NAME_JOB_RECORDS = "job_records"
+REQUIRED_TABLES_JOB_ARCHIVE = {
+    TABLE_NAME_ARCHIVE_METADATA,
+    TABLE_NAME_JOB_RECORDS,
+}
+
+TABLE_NAME_METADATA = "metadata"
+TABLE_NAME_METADATA_SCHEMAS = "metadata_schemas"
+TABLE_NAME_METADATA_REFERENCES = "metadata_references"
+REQUIRED_TABLES_METADATA = {
+    TABLE_NAME_ARCHIVE_METADATA,
+    TABLE_NAME_METADATA,
+    TABLE_NAME_METADATA_SCHEMAS,
+    TABLE_NAME_METADATA_REFERENCES,
+}
+
+
+ALL_REQUIRED_TABLES = set(REQUIRED_TABLES_DATA_ARCHIVE)
+ALL_REQUIRED_TABLES.update(REQUIRED_TABLES_ALIAS_ARCHIVE)
+ALL_REQUIRED_TABLES.update(REQUIRED_TABLES_JOB_ARCHIVE)
+ALL_REQUIRED_TABLES.update(REQUIRED_TABLES_METADATA)
