@@ -11,26 +11,24 @@ from typing import Any, Type, Union
 
 from pydantic import BaseModel
 
-from kiara.context import Kiara, KiaraContextInfo
+from kiara.context import KiaraContextInfo
 from kiara.utils.yaml import StringYAML
 
 yaml = StringYAML()
-kiara_obj = Kiara.instance()
+# kiara_obj = Kiara.instance()
 
 
 def define_env(env):
     """
-    This is the hook for defining variables, macros and filters
+    This is the hook for defining variables, macros and filters.
 
     - variables: the dictionary that contains the environment variables
     - macro: a decorator function, to declare a macro.
     """
-
     # env.variables["baz"] = "John Doe"
 
     @env.macro
     def get_schema_for_model(model_class: Union[str, Type[BaseModel]]):
-
         if isinstance(model_class, str):
             _class: Type[BaseModel] = locate(model_class)  # type: ignore
         else:
@@ -42,7 +40,6 @@ def define_env(env):
 
     @env.macro
     def get_src_of_object(obj: Union[str, Any]):
-
         try:
             if isinstance(obj, str):
                 _obj: Type[BaseModel] = locate(obj)  # type: ignore
@@ -52,11 +49,10 @@ def define_env(env):
             src = inspect.getsource(_obj)
             return src
         except Exception as e:
-            return f"Can't render object source: {str(e)}"
+            return f"Can't render object source: {e}"
 
     @env.macro
     def get_context_info() -> KiaraContextInfo:
-
         return builtins.plugin_package_context_info  # type: ignore
 
     # @env.macro
