@@ -7,21 +7,16 @@
 
 import os
 import sys
-from typing import Iterable
+from typing import TYPE_CHECKING, Iterable, Union
 
 import rich_click as click
 
-from kiara import KiaraAPI
-from kiara.context import Kiara
-from kiara.interfaces.python_api.models.info import (
-    OperationGroupInfo,
-    OperationTypeClassesInfo,
-)
-
-# from kiara.interfaces.python_api.operation import KiaraOperation
 from kiara.utils.cli import output_format_option, terminal_print, terminal_print_model
 from kiara.utils.cli.exceptions import handle_exception
 
+if TYPE_CHECKING:
+    from kiara.interfaces.python_api.base_api import BaseAPI, Kiara
+
 
 @click.group()
 @click.pass_context
@@ -40,8 +35,7 @@ def operation(ctx):
 @output_format_option()
 @click.pass_context
 def list_types(ctx, full_doc: bool, format: str, filter: Iterable[str]):
-
-    kiara_obj: Kiara = ctx.obj["kiara"]
+    kiara_obj: Kiara = ctx.obj.kiara
 
     op_mgmt = kiara_obj.operation_registry
 
@@ -61,6 +55,8 @@ def list_types(ctx, full_doc: bool, format: str, filter: Iterable[str]):
                 temp[k] = v
         op_types = temp
 
+    from kiara.interfaces.python_api.models.info import OperationTypeClassesInfo
+
     operation_types_info = OperationTypeClassesInfo.create_from_type_items(
         kiara=kiara_obj, group_title="all_items", **op_types
     )
@@ -74,8 +70,7 @@ def list_types(ctx, full_doc: bool, format: str, filter: Iterable[str]):
 @click.pass_context
 @handle_exception()
 def explain_type(ctx, operation_type: str, format: str):
-
-    kiara_api: KiaraAPI = ctx.obj["kiara_api"]
+    kiara_api: BaseAPI = ctx.obj.base_api
 
     op_type = kiara_api.retrieve_operation_type_info(operation_type)
 
@@ -104,6 +99,12 @@ def explain_type(ctx, operation_type: str, format: str):
     help="Whether to include operations that are mainly used internally.",
     is_flag=True,
 )
+@click.option(
+    "--python-package",
+    "-p",
+    help="Only return modules from this package.",
+    required=False,
+)
 @output_format_option()
 @click.pass_context
 def list_operations(
@@ -112,36 +113,43 @@ def list_operations(
     filter: Iterable[str],
     full_doc: bool,
     include_internal: bool,
+    python_package: Union[str, None],
     format: str,
 ):
+    kiara_obj: Kiara = ctx.obj.kiara
+    api: BaseAPI = ctx.obj.base_api
 
-    kiara_obj: Kiara = ctx.obj["kiara"]
+    operations = api.list_operations(
+        filter=filter, include_internal=include_internal, python_packages=python_package
+    )
 
-    operations = kiara_obj.operation_registry.operations
+    # operations = kiara_obj.operation_registry.operations
     title = "Available operations"
-    if filter:
-        title = "Filtered operations"
-        temp = {}
-        for op_id, op in operations.items():
-            match = True
-            for f in filter:
-                if f.lower() not in op_id.lower():
-                    match = False
-                    break
-            if match:
-                temp[op_id] = op
-        operations = temp
+    # if filter:
+    #     title = "Filtered operations"
+    #     temp = {}
+    #     for op_id, op in operations.items():
+    #         match = True
+    #         for f in filter:
+    #             if f.lower() not in op_id.lower():
+    #                 match = False
+    #                 break
+    #         if match:
+    #             temp[op_id] = op
+    #     operations = temp
+    #
+    # if not include_internal:
+    #     temp = {}
+    #     for op_id, op in operations.items():
+    #         if not op.operation_details.is_internal_operation:
+    #             temp[op_id] = op
+    #
+    #     operations = temp
 
-    if not include_internal:
-        temp = {}
-        for op_id, op in operations.items():
-            if not op.operation_details.is_internal_operation:
-                temp[op_id] = op
-
-        operations = temp
+    from kiara.interfaces.python_api.models.info import OperationGroupInfo
 
     ops_info = OperationGroupInfo.create_from_operations(
-        kiara=kiara_obj, group_title=title, **operations
+        kiara=kiara_obj, group_title=title, **operations.root
     )
 
     terminal_print_model(
@@ -169,9 +177,8 @@ def list_operations(
 @click.pass_context
 @handle_exception()
 def explain(ctx, operation_id: str, source: bool, format: str, module_info: bool):
-
-    kiara_obj: Kiara = ctx.obj["kiara"]
-    api: KiaraAPI = ctx.obj["kiara_api"]
+    kiara_obj: Kiara = ctx.obj.kiara
+    api: BaseAPI = ctx.obj.base_api
 
     if os.path.isfile(os.path.realpath(operation_id)):
         operation = api.get_operation(operation_id)
