@@ -6,18 +6,19 @@
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 
 """Pipeline-related subcommands for the cli."""
+
 import typing
 
 import rich_click as click
 from rich import box
 from rich.table import Table
 
-from kiara.context import Kiara
-from kiara.interfaces.python_api.models.info import OperationGroupInfo
+from kiara.defaults import KIARA_DEFAULT_STAGES_EXTRACTION_TYPE
 from kiara.utils.cli import output_format_option, terminal_print_model
 from kiara.utils.cli.exceptions import handle_exception
-from kiara.utils.graphs import print_ascii_graph
-from kiara.utils.pipelines import get_pipeline_config
+
+if typing.TYPE_CHECKING:
+    from kiara.api import Kiara
 
 
 @click.group()
@@ -38,8 +39,7 @@ def pipeline(ctx):
 @click.pass_context
 def list_pipelines(ctx, full_doc: bool, filter: typing.Iterable[str], format: str):
     """List available module data_types."""
-
-    kiara_obj: Kiara = ctx.obj["kiara"]
+    kiara_obj: Kiara = ctx.obj.kiara
 
     kiara_obj.operation_registry.get_operation_type("pipeline")
 
@@ -69,6 +69,8 @@ def list_pipelines(ctx, full_doc: bool, filter: typing.Iterable[str], format: st
             op_id: kiara_obj.operation_registry.get_operation(op_id) for op_id in op_ids
         }
 
+    from kiara.interfaces.python_api.models.info import OperationGroupInfo
+
     ops_info = OperationGroupInfo.create_from_operations(
         kiara=kiara_obj, group_title=title, **operations
     )
@@ -77,17 +79,61 @@ def list_pipelines(ctx, full_doc: bool, filter: typing.Iterable[str], format: st
 
 @pipeline.command()
 @click.argument("pipeline-name-or-path", nargs=1)
+@click.option(
+    "--stages-extraction-type",
+    "-s",
+    default=KIARA_DEFAULT_STAGES_EXTRACTION_TYPE,
+    help="How to extract the stages from the pipeline structure. Available: 'late', 'early', as well as pipeline specific profiles (if in pipeline metadata).",
+)
 @output_format_option()
 @click.pass_context
 @handle_exception()
-def explain(ctx, pipeline_name_or_path: str, format: str):
+def explain(ctx, pipeline_name_or_path: str, format: str, stages_extraction_type: str):
     """Print details about pipeline inputs, outputs, and overall structure."""
+    kiara_obj: Kiara = ctx.obj.kiara
 
-    kiara_obj: Kiara = ctx.obj["kiara"]
+    from kiara.utils.pipelines import get_pipeline_config
 
     pc = get_pipeline_config(kiara=kiara_obj, pipeline=pipeline_name_or_path)
     terminal_print_model(
-        pc, format=format, in_panel=f"Pipeline: [b i]{pipeline_name_or_path}[/b i]"
+        pc,
+        format=format,
+        in_panel=f"Pipeline: [b i]{pipeline_name_or_path}[/b i]",
+        stages_extraction_type=stages_extraction_type,
+        show_pipeline_inputs_for_steps=False,
+    )
+
+
+@pipeline.command()
+@click.argument("pipeline-name-or-path", nargs=1)
+@click.option(
+    "--stages-extraction-type",
+    "-s",
+    default=KIARA_DEFAULT_STAGES_EXTRACTION_TYPE,
+    help="How to extract the stages from the pipeline structure. Available: 'late', 'early', as well as pipeline specific profiles (if in pipeline metadata).",
+)
+@output_format_option()
+@click.pass_context
+@handle_exception()
+def explain_stages(
+    ctx, pipeline_name_or_path: str, format: str, stages_extraction_type: str
+):
+    """Print details about pipeline inputs, outputs, and overall structure."""
+    from kiara.models.module.pipeline.stages import PipelineStages
+    from kiara.utils.pipelines import get_pipeline_config
+
+    kiara_obj: Kiara = ctx.obj.kiara
+
+    pc = get_pipeline_config(kiara=kiara_obj, pipeline=pipeline_name_or_path)
+    structure = pc.structure
+
+    stages = PipelineStages.create(
+        structure=structure, stages_extraction_type=stages_extraction_type
+    )
+    terminal_print_model(
+        stages,
+        format=format,
+        in_panel=f"Stages for pipeline: [b i]{pipeline_name_or_path}[/b i]",
     )
 
 
@@ -96,12 +142,15 @@ def explain(ctx, pipeline_name_or_path: str, format: str):
 @click.pass_context
 def execution_graph(ctx, pipeline_name_or_path: str):
     """Print the execution graph for a pipeline structure."""
+    from kiara.utils.graphs import print_ascii_graph
+    from kiara.utils.pipelines import get_pipeline_config
 
-    kiara_obj = ctx.obj["kiara"]
+    kiara_obj = ctx.obj.kiara
 
     pc = get_pipeline_config(kiara=kiara_obj, pipeline=pipeline_name_or_path)
 
     structure = pc.structure
+
     print_ascii_graph(
         structure.execution_graph, restart_interpreter_if_asciinet_installed=True
     )
@@ -119,7 +168,10 @@ def execution_graph(ctx, pipeline_name_or_path: str):
 def data_flow_graph(ctx, pipeline_name_or_path: str, full: bool):
     """Print the data flow graph for a pipeline structure."""
 
-    kiara_obj = ctx.obj["kiara"]
+    from kiara.utils.graphs import print_ascii_graph
+    from kiara.utils.pipelines import get_pipeline_config
+
+    kiara_obj = ctx.obj.kiara
 
     pc = get_pipeline_config(kiara=kiara_obj, pipeline=pipeline_name_or_path)
 
@@ -136,6 +188,35 @@ def data_flow_graph(ctx, pipeline_name_or_path: str, full: bool):
         )
 
 
+@pipeline.command()
+@click.argument("pipeline-name-or-path", nargs=1)
+@click.option(
+    "--stages-extraction-type",
+    "-s",
+    default=KIARA_DEFAULT_STAGES_EXTRACTION_TYPE,
+    help="How to extract the stages from the pipeline structure. Available: 'late', 'early', as well as pipeline specific profiles (if in pipeline metadata).",
+)
+@click.pass_context
+def stages_graph(ctx, pipeline_name_or_path: str, stages_extraction_type: str):
+    """Print the data flow graph for a pipeline structure."""
+    from kiara.utils.graphs import print_ascii_graph
+    from kiara.utils.pipelines import get_pipeline_config
+
+    kiara_obj = ctx.obj.kiara
+
+    pc = get_pipeline_config(kiara=kiara_obj, pipeline=pipeline_name_or_path)
+
+    structure = pc.structure
+    stages_graph = structure.get_stages_graph(
+        stages_extraction_type=stages_extraction_type
+    )
+
+    print_ascii_graph(
+        stages_graph,
+        restart_interpreter_if_asciinet_installed=True,
+    )
+
+
 # @pipeline.command()
 # @click.argument("pipeline-type", nargs=1)
 # @click.pass_context
