@@ -7,7 +7,15 @@ import os
 import uuid
 from typing import TYPE_CHECKING, Any, Dict, List, Mapping, Union
 
-from pydantic import BaseModel, Field, PrivateAttr, root_validator, validator
+from pydantic import (
+    BaseModel,
+    ConfigDict,
+    Field,
+    PrivateAttr,
+    field_validator,
+    model_validator,
+)
+from pydantic_core.core_schema import ValidationInfo
 
 from kiara.context import Kiara
 from kiara.interfaces.python_api.utils import create_save_config
@@ -22,8 +30,7 @@ if TYPE_CHECKING:
 
 
 class BatchOperation(BaseModel):
-    class Config:
-        validate_assignment = True
+    model_config = ConfigDict(validate_assignment=True)
 
     @classmethod
     def from_file(
@@ -31,7 +38,6 @@ class BatchOperation(BaseModel):
         path: str,
         kiara: Union["Kiara", None] = None,
     ):
-
         data = get_data_from_file(path)
         pipeline_id = data.get("pipeline_name", None)
         if pipeline_id is None:
@@ -53,7 +59,6 @@ class BatchOperation(BaseModel):
         data: Mapping[str, Any],
         kiara: Union["Kiara", None],
     ):
-
         data = dict(data)
         inputs = data.pop("inputs", {})
         save = data.pop("save", False)
@@ -88,13 +93,14 @@ class BatchOperation(BaseModel):
     save_defaults: Dict[str, List[str]] = Field(
         description="Configuration which values to save, under which alias(es).",
         default_factory=dict,
+        validate_default=True,
     )
 
-    _kiara: Kiara = PrivateAttr(default=None)
+    _kiara: Union[Kiara, None] = PrivateAttr(default=None)
 
-    @root_validator(pre=True)
+    @model_validator(mode="before")
+    @classmethod
     def add_alias(cls, values):
-
         if not values.get("alias", None):
             pc = values.get("pipeline_config", None)
             if not pc:
@@ -107,13 +113,13 @@ class BatchOperation(BaseModel):
 
         return values
 
-    @validator("save_defaults", always=True, pre=True)
-    def validate_save(cls, save, values):
-
-        alias = values["alias"]
-        pipeline_config = values["pipeline_config"]
+    @field_validator("save_defaults", mode="before")
+    @classmethod
+    def validate_save(cls, save_defaults: Dict[str, List[str]], info: ValidationInfo):
+        alias = info.data["alias"]
+        pipeline_config = info.data["pipeline_config"]
         return cls.create_save_aliases(
-            save=save, alias=alias, pipeline_config=pipeline_config
+            save=save_defaults, alias=alias, pipeline_config=pipeline_config
         )
 
     @classmethod
@@ -123,7 +129,6 @@ class BatchOperation(BaseModel):
         alias: str,
         pipeline_config: PipelineConfig,
     ) -> Mapping[str, Any]:
-
         assert isinstance(pipeline_config, PipelineConfig)
 
         if save in [False, None]:
@@ -148,7 +153,7 @@ class BatchOperation(BaseModel):
         inputs: Union[Mapping[str, Any], None] = None,
         save: Union[None, bool, str, Mapping[str, Any]] = None,
     ) -> ValueMap:
-
+        assert self._kiara is not None
         pipeline = Pipeline(
             structure=self.pipeline_config.structure,
             kiara=self._kiara,
