@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
-"""This module contains the metadata (and other) models that are used in the ``kiara_plugin.core_types`` package.
+"""
+This module contains the metadata (and other) models that are used in the ``kiara_plugin.core_types`` package.
 
 Those models are convenience wrappers that make it easier for *kiara* to find, create, manage and version metadata -- but also
 other type of models -- that is attached to data, as well as *kiara* modules.
@@ -9,34 +10,26 @@ Metadata models must be a sub-class of [kiara.metadata.MetadataModel][kiara.meta
 sub-class a pydantic BaseModel or implement custom base classes.
 """
 
-from typing import Any, Dict, Mapping
+from typing import Any, Dict, Mapping, Union
 
-import orjson
 from pydantic import BaseModel, Field, PrivateAttr
 
 from kiara.models.python_class import PythonClass
 from kiara.utils.hashing import compute_cid
-from kiara.utils.json import orjson_dumps
 
 
-class DictModel(BaseModel, Mapping):
-    """A dict implentation that contains (optional) schema information of the dicts items."""
-
-    class Config:
-        json_loads = orjson.loads
-        json_dumps = orjson_dumps
-
+class KiaraDict(BaseModel, Mapping):
     dict_data: Dict[str, Any] = Field(description="The data.")
     data_schema: Dict[str, Any] = Field(description="The schema.")
     python_class: PythonClass = Field(
         description="The python class of which model instances are created. This is mostly meant as a hint for client applications."
     )
 
-    _size_cache: int = PrivateAttr(default=None)
-    _hash_cache: int = PrivateAttr(default=None)
-    _data_hash: int = PrivateAttr(default=None)
-    _schema_hash: int = PrivateAttr(default=None)
-    _value_hash: int = PrivateAttr(default=None)
+    _size_cache: Union[int, None] = PrivateAttr(default=None)
+    _hash_cache: Union[int, None] = PrivateAttr(default=None)
+    _data_hash: Union[int, None] = PrivateAttr(default=None)
+    _schema_hash: Union[int, None] = PrivateAttr(default=None)
+    _value_hash: Union[int, None] = PrivateAttr(default=None)
 
     @property
     def size(self):
@@ -51,7 +44,8 @@ class DictModel(BaseModel, Mapping):
         if self._data_hash is not None:
             return self._data_hash
 
-        self._data_hash = compute_cid(self.dict_data)
+        _data_hash, _ = compute_cid(self.dict_data)
+        self._data_hash = int.from_bytes(_data_hash, byteorder="big")
         return self._data_hash
 
     @property
@@ -59,7 +53,8 @@ class DictModel(BaseModel, Mapping):
         if self._schema_hash is not None:
             return self._schema_hash
 
-        self._schema_hash = compute_cid(self.data_schema)
+        _schema_hash, _ = compute_cid(self.data_schema)
+        self._schema_hash = int.from_bytes(_schema_hash, byteorder="big")
         return self._schema_hash
 
     @property
@@ -68,7 +63,8 @@ class DictModel(BaseModel, Mapping):
             return self._value_hash
 
         obj = {"data": self.data_hash, "data_schema": self.schema_hash}
-        self._value_hash = compute_cid(obj)
+        _value_hash, _ = compute_cid(obj)  # type: ignore
+        self._value_hash = int.from_bytes(_value_hash, byteorder="big")
         return self._value_hash
 
     def __getitem__(self, item):
@@ -79,3 +75,10 @@ class DictModel(BaseModel, Mapping):
 
     def __len__(self):
         return self.dict_data.__len__()
+
+    def __repr__(self):
+        full = {
+            "dict_data": self.dict_data,
+            "data_schema": self.data_schema,
+        }
+        return full.__repr__()
