@@ -9,7 +9,7 @@ import os
 import shutil
 import tempfile
 from pathlib import Path
-from typing import Any, Dict, List, Mapping, Union
+from typing import Any, Callable, ClassVar, Dict, List, Mapping, Union
 
 import structlog
 from deepdiff import DeepHash
@@ -19,10 +19,15 @@ from rich import box
 from rich.console import RenderableType
 from rich.table import Table
 
-from kiara.defaults import DEFAULT_EXCLUDE_FILES, KIARA_HASH_FUNCTION
+from kiara.defaults import (
+    DEFAULT_EXCLUDE_DIRS,
+    DEFAULT_EXCLUDE_FILES,
+)
+from kiara.exceptions import KiaraException
 from kiara.models import KiaraModel
 from kiara.utils import log_message
-from kiara.utils.hashing import compute_cid_from_file
+from kiara.utils.files import unpack_archive
+from kiara.utils.hashing import KIARA_HASH_FUNCTION, compute_cid_from_file
 
 logger = structlog.getLogger()
 
@@ -37,10 +42,20 @@ FILE_BUNDLE_IMPORT_AVAILABLE_COLUMNS = [
 ]
 
 
-class FileModel(KiaraModel):
+class KiaraFile(KiaraModel):
     """Describes properties for the 'file' value type."""
 
-    _kiara_model_id = "instance.data.file"
+    _kiara_model_id: ClassVar = "instance.data.file"
+
+    @classmethod
+    def from_bytes(cls, content_bytes: bytes, file_name: str):
+        temp_f = tempfile.mkstemp()
+        os.close(temp_f[0])
+        with open(temp_f[1], "wb") as f:
+            f.write(content_bytes)
+        file = cls.load_file(temp_f[1], file_name=file_name)
+        atexit.register(os.remove, temp_f[1])
+        return file
 
     @classmethod
     def load_file(
@@ -48,9 +63,8 @@ class FileModel(KiaraModel):
         source: str,
         file_name: Union[str, None] = None,
         # import_time: Optional[datetime.datetime] = None,
-    ):
+    ) -> "KiaraFile":
         """Utility method to read metadata of a file from disk."""
-
         import mimetypes
 
         import filetype
@@ -86,7 +100,7 @@ class FileModel(KiaraModel):
             else:
                 mime_type = _mime_type.MIME
 
-        m = FileModel(
+        m = KiaraFile(
             # import_time=file_import_time,
             mime_type=mime_type,
             size=size,
@@ -102,11 +116,16 @@ class FileModel(KiaraModel):
     file_name: str = Field("The name of the file.")
     size: int = Field(description="The size of the file.")
     metadata: Dict[str, Any] = Field(
-        description="Additional, ustructured, user-defined metadata.",
+        description="Additional, unstructured, user-defined metadata.",
+        default_factory=dict,
+    )
+    metadata_schemas: Dict[str, str] = Field(
+        description="The metadata schemas for each of the metadata values (if available).",
         default_factory=dict,
     )
 
     _path: Union[str, None] = PrivateAttr(default=None)
+    _path_resolver: Union[Callable, None] = PrivateAttr(default=None)
     _file_hash: Union[str, None] = PrivateAttr(default=None)
     _file_cid: Union[CID, None] = PrivateAttr(default=None)
 
@@ -117,7 +136,10 @@ class FileModel(KiaraModel):
     @property
     def path(self) -> str:
         if self._path is None:
-            raise Exception("File path not set for file model.")
+            if self._path_resolver is not None:
+                self._path = self._path_resolver()
+            else:
+                raise Exception("File path not set for file model.")
         return self._path
 
     def _retrieve_data_to_hash(self) -> Any:
@@ -133,13 +155,12 @@ class FileModel(KiaraModel):
     def get_category_alias(self) -> str:
         return "instance.file_model"
 
-    def copy_file(self, target: str, new_name: Union[str, None] = None) -> "FileModel":
-
+    def copy_file(self, target: str, new_name: Union[str, None] = None) -> "KiaraFile":
         target_path: str = os.path.abspath(target)
         os.makedirs(os.path.dirname(target_path), exist_ok=True)
 
         shutil.copy2(self.path, target_path)
-        fm = FileModel.load_file(target, file_name=new_name)
+        fm = KiaraFile.load_file(target, file_name=new_name)
 
         if self._file_hash is not None:
             fm._file_hash = self._file_hash
@@ -148,7 +169,6 @@ class FileModel(KiaraModel):
 
     @property
     def file_hash(self) -> str:
-
         if self._file_hash is not None:
             return self._file_hash
 
@@ -157,7 +177,6 @@ class FileModel(KiaraModel):
 
     @property
     def file_cid(self) -> CID:
-
         if self._file_cid is not None:
             return self._file_cid
 
@@ -167,12 +186,14 @@ class FileModel(KiaraModel):
 
     @property
     def file_name_without_extension(self) -> str:
-
         return self.file_name.split(".")[0]
 
+    @property
+    def file_extension(self) -> str:
+        return self.file_name.split(".")[-1]
+
     def read_text(self, max_lines: int = -1) -> str:
         """Read the content of a file."""
-
         with open(self.path, "rt") as f:
             if max_lines <= 0:
                 content = f.read()
@@ -182,7 +203,6 @@ class FileModel(KiaraModel):
 
     def read_bytes(self, length: int = -1) -> bytes:
         """Read the content of a file."""
-
         with open(self.path, "rb") as f:
             if length <= 0:
                 content = f.read()
@@ -191,13 +211,16 @@ class FileModel(KiaraModel):
         return content
 
     def __repr__(self):
-        return f"FileModel(name={self.file_name})"
+        return f"KiaraFile(name={self.file_name})"
 
     def __str__(self):
         return self.__repr__()
 
 
 class FolderImportConfig(BaseModel):
+    sub_path: Union[str, None] = Field(
+        description="The sub-path to import from the folder.", default=None
+    )
 
     include_files: Union[List[str], None] = Field(
         description="A list of strings, include all files where the filename ends with that string.",
@@ -205,7 +228,7 @@ class FolderImportConfig(BaseModel):
     )
     exclude_dirs: Union[List[str], None] = Field(
         description="A list of strings, exclude all folders whose name ends with that string.",
-        default=None,
+        default=DEFAULT_EXCLUDE_DIRS,
     )
     exclude_files: Union[List[str], None] = Field(
         description=f"A list of strings, exclude all files that match those (takes precedence over 'include_files'). Defaults to: {DEFAULT_EXCLUDE_FILES}.",
@@ -213,15 +236,14 @@ class FolderImportConfig(BaseModel):
     )
 
 
-class FileBundle(KiaraModel):
+class KiaraFileBundle(KiaraModel):
     """Describes properties for the 'file_bundle' value type."""
 
-    _kiara_model_id = "instance.data.file_bundle"
+    _kiara_model_id: ClassVar = "instance.data.file_bundle"
 
     @classmethod
     def create_tmp_dir(self) -> Path:
         """Utility method to create a temp folder that gets deleted when kiara exits."""
-
         temp_f = tempfile.mkdtemp()
 
         def cleanup():
@@ -231,6 +253,52 @@ class FileBundle(KiaraModel):
 
         return Path(temp_f)
 
+    @classmethod
+    def from_archive(
+        cls,
+        archive_path: str,
+        import_config: Union[FolderImportConfig, None] = None,
+        bundle_name: Union[str, None] = None,
+    ) -> "KiaraFileBundle":
+        """Extracts the contents of an archive file to a target folder."""
+
+        if not os.path.isfile(archive_path):
+            raise KiaraException(
+                msg=f"Archive file '{archive_path}' does not exist or is not a file."
+            )
+
+        out_dir = tempfile.mkdtemp()
+
+        def del_out_dir():
+            shutil.rmtree(out_dir, ignore_errors=True)
+
+        atexit.register(del_out_dir)
+
+        unpack_archive(archive_path, out_dir)
+
+        bundle = KiaraFileBundle.import_folder(
+            out_dir, import_config=import_config, bundle_name=bundle_name
+        )
+        return bundle
+
+    @classmethod
+    def from_archive_file(
+        cls,
+        archive_file: KiaraFile,
+        import_config: Union[FolderImportConfig, None] = None,
+    ) -> "KiaraFileBundle":
+        """Extracts the contents of an archive file to a target folder."""
+
+        bundle = KiaraFileBundle.from_archive(
+            archive_path=archive_file.path,
+            bundle_name=archive_file.file_name,
+            import_config=import_config,
+        )
+
+        bundle.metadata = archive_file.metadata
+        bundle.metadata_schemas = archive_file.metadata_schemas
+        return bundle
+
     @classmethod
     def import_folder(
         cls,
@@ -238,8 +306,7 @@ class FileBundle(KiaraModel):
         bundle_name: Union[str, None] = None,
         import_config: Union[None, Mapping[str, Any], FolderImportConfig] = None,
         # import_time: Optional[datetime.datetime] = None,
-    ) -> "FileBundle":
-
+    ) -> "KiaraFileBundle":
         if not source:
             raise ValueError("No source path provided.")
 
@@ -252,8 +319,6 @@ class FileBundle(KiaraModel):
         if source.endswith(os.path.sep):
             source = source[0:-1]
 
-        abs_path = os.path.abspath(source)
-
         if import_config is None:
             _import_config = FolderImportConfig()
         elif isinstance(import_config, Mapping):
@@ -265,21 +330,19 @@ class FileBundle(KiaraModel):
                 f"Invalid type for folder import config: {type(import_config)}."
             )
 
-        included_files: Dict[str, FileModel] = {}
+        abs_path = os.path.abspath(source)
+        if _import_config.sub_path:
+            abs_path = os.path.join(abs_path, _import_config.sub_path)
+
+        included_files: Dict[str, KiaraFile] = {}
         exclude_dirs = _import_config.exclude_dirs
         invalid_extensions = _import_config.exclude_files
 
         valid_extensions = _import_config.include_files
 
-        # if import_time:
-        #     bundle_import_time = import_time
-        # else:
-        #     bundle_import_time = datetime.datetime.now()  # TODO: timezone
-
         sum_size = 0
 
         def include_file(filename: str) -> bool:
-
             if invalid_extensions and any(
                 filename.endswith(ext) for ext in invalid_extensions
             ):
@@ -289,28 +352,31 @@ class FileBundle(KiaraModel):
             else:
                 return any(filename.endswith(ext) for ext in valid_extensions)
 
-        for root, dirnames, filenames in os.walk(abs_path, topdown=True):
+        if os.path.isfile(abs_path):
+            file_model = KiaraFile.load_file(abs_path)
+            sum_size = file_model.size
+            included_files[file_model.file_name] = file_model
+        else:
+            for root, dirnames, filenames in os.walk(abs_path, topdown=True):
+                if exclude_dirs:
+                    dirnames[:] = [d for d in dirnames if d not in exclude_dirs]
 
-            if exclude_dirs:
-                dirnames[:] = [d for d in dirnames if d not in exclude_dirs]
+                for filename in [
+                    f
+                    for f in filenames
+                    if os.path.isfile(os.path.join(root, f)) and include_file(f)
+                ]:
+                    full_path = os.path.join(root, filename)
+                    rel_path = os.path.relpath(full_path, abs_path)
 
-            for filename in [
-                f
-                for f in filenames
-                if os.path.isfile(os.path.join(root, f)) and include_file(f)
-            ]:
-
-                full_path = os.path.join(root, filename)
-                rel_path = os.path.relpath(full_path, abs_path)
-
-                file_model = FileModel.load_file(full_path)
-                sum_size = sum_size + file_model.size
-                included_files[rel_path] = file_model
+                    file_model = KiaraFile.load_file(full_path)
+                    sum_size = sum_size + file_model.size
+                    included_files[rel_path] = file_model
 
         if bundle_name is None:
             bundle_name = os.path.basename(source)
 
-        bundle = FileBundle.create_from_file_models(
+        bundle = KiaraFileBundle.create_from_file_models(
             files=included_files,
             path=abs_path,
             bundle_name=bundle_name,
@@ -321,13 +387,12 @@ class FileBundle(KiaraModel):
     @classmethod
     def create_from_file_models(
         cls,
-        files: Mapping[str, FileModel],
+        files: Mapping[str, KiaraFile],
         bundle_name: str,
         path: Union[str, None] = None,
         sum_size: Union[int, None] = None,
         # import_time: Optional[datetime.datetime] = None,
-    ) -> "FileBundle":
-
+    ) -> "KiaraFileBundle":
         # if import_time:
         #     bundle_import_time = import_time
         # else:
@@ -348,7 +413,7 @@ class FileBundle(KiaraModel):
                 sum_size = sum_size + f.size
         result["size"] = sum_size
 
-        bundle = FileBundle(**result)
+        bundle = KiaraFileBundle(**result)
         bundle._path = path
         return bundle
 
@@ -361,7 +426,7 @@ class FileBundle(KiaraModel):
     number_of_files: int = Field(
         description="How many files are included in this bundle."
     )
-    included_files: Dict[str, FileModel] = Field(
+    included_files: Dict[str, KiaraFile] = Field(
         description="A map of all the included files, incl. their properties. Uses the relative path of each file as key."
     )
     size: int = Field(description="The size of all files in this folder, combined.")
@@ -369,7 +434,10 @@ class FileBundle(KiaraModel):
         description="Additional, ustructured, user-defined metadata.",
         default_factory=dict,
     )
-
+    metadata_schemas: Dict[str, str] = Field(
+        description="The metadata schemas for each metadata value (if available).",
+        default_factory=dict,
+    )
     _path: Union[str, None] = PrivateAttr(default=None)
 
     @property
@@ -389,7 +457,6 @@ class FileBundle(KiaraModel):
     #     return self.file_bundle_hash
 
     def _retrieve_data_to_hash(self) -> Any:
-
         return {
             "bundle_name": self.bundle_name,
             "included_files": {
@@ -397,11 +464,10 @@ class FileBundle(KiaraModel):
             },
         }
 
-    def get_relative_path(self, file: FileModel):
+    def get_relative_path(self, file: KiaraFile):
         return os.path.relpath(file.path, self.path)
 
     def read_text_file_contents(self, ignore_errors: bool = False) -> Mapping[str, str]:
-
         content_dict: Dict[str, str] = {}
 
         def read_file(rel_path: str, full_path: str):
@@ -428,7 +494,6 @@ class FileBundle(KiaraModel):
 
     @property
     def file_bundle_hash(self) -> int:
-
         # TODO: use sha256?
         if self._file_bundle_hash is not None:
             return self._file_bundle_hash
@@ -441,8 +506,7 @@ class FileBundle(KiaraModel):
 
     def copy_bundle(
         self, target_path: str, bundle_name: Union[str, None] = None
-    ) -> "FileBundle":
-
+    ) -> "KiaraFileBundle":
         if target_path == self.path:
             raise Exception(f"Target path and current path are the same: {target_path}")
 
@@ -455,7 +519,7 @@ class FileBundle(KiaraModel):
         if bundle_name is None:
             bundle_name = os.path.basename(target_path)
 
-        fb = FileBundle.create_from_file_models(
+        fb = KiaraFileBundle.create_from_file_models(
             files=result,
             bundle_name=bundle_name,
             path=target_path,
@@ -468,7 +532,6 @@ class FileBundle(KiaraModel):
         return fb
 
     def create_renderable(self, **config: Any) -> RenderableType:
-
         show_bundle_hash = config.get("show_bundle_hash", False)
 
         table = Table(show_header=False, box=box.SIMPLE)
@@ -488,7 +551,6 @@ class FileBundle(KiaraModel):
         return table
 
     def _create_content_table(self, **render_config: Any) -> Table:
-
         # show_content = render_config.get("show_content_preview", False)
         max_no_included_files = render_config.get("max_no_files", 40)
 
