@@ -2,11 +2,11 @@
 import os
 import uuid
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Mapping, Union
+from typing import TYPE_CHECKING, Any, ClassVar, Dict, Iterable, List, Mapping, Union
 
 import orjson
 from boltons.strutils import slugify
-from pydantic import Extra, Field, PrivateAttr, root_validator, validator
+from pydantic import ConfigDict, Field, PrivateAttr, field_validator, model_validator
 from rich import box
 from rich.console import RenderableType
 from rich.markdown import Markdown
@@ -19,14 +19,22 @@ from kiara.models.documentation import DocumentationMetadataModel
 from kiara.models.module import KiaraModuleConfig
 from kiara.models.module.jobs import ExecutionContext
 from kiara.models.module.manifest import Manifest
-from kiara.models.module.pipeline.value_refs import StepValueAddress
+from kiara.models.module.pipeline.value_refs import (
+    PipelineInputRef,
+    PipelineOutputRef,
+    StepValueAddress,
+)
 from kiara.models.python_class import KiaraModuleInstance
 from kiara.utils import find_free_id, is_jupyter
+from kiara.utils.data import get_data_from_string
 from kiara.utils.files import get_data_from_file
 from kiara.utils.json import orjson_dumps
 from kiara.utils.modules import module_config_is_empty
 from kiara.utils.output import create_table_from_field_schemas
-from kiara.utils.pipelines import ensure_step_value_addresses
+from kiara.utils.pipelines import (
+    ensure_step_value_addresses,
+    extract_data_to_hash_from_pipeline_config,
+)
 from kiara.utils.string_vars import replace_var_names_in_obj
 
 #  Copyright (c) 2021, Markus Binsteiner
@@ -36,6 +44,7 @@ from kiara.utils.string_vars import replace_var_names_in_obj
 
 if TYPE_CHECKING:
     from kiara.context import Kiara
+    from kiara.models.module.pipeline.pipeline import Pipeline
     from kiara.models.module.pipeline.structure import PipelineStructure
     from kiara.modules import KiaraModule
 
@@ -47,15 +56,20 @@ class StepStatus(Enum):
     INPUTS_READY = "inputs_ready"
     RESULTS_READY = "results_ready"
 
+    def to_console_renderable(self) -> RenderableType:
+        if self == StepStatus.INPUTS_INVALID:
+            return "[red]inputs invalid[/red]"
+        elif self == StepStatus.INPUTS_READY:
+            return "[yellow]inputs ready[/yellow]"
+        elif self == StepStatus.RESULTS_READY:
+            return "[green]results ready[/green]"
+
 
 class PipelineStep(Manifest):
     """A step within a pipeline-structure, includes information about it's connection(s) and other metadata."""
 
-    _kiara_model_id = "instance.pipeline_step"
-
-    class Config:
-        validate_assignment = True
-        extra = Extra.forbid
+    _kiara_model_id: ClassVar = "instance.pipeline_step"
+    model_config = ConfigDict(validate_assignment=True, extra="forbid")
 
     @classmethod
     def create_step(
@@ -66,11 +80,11 @@ class PipelineStep(Manifest):
         auto_step_id: bool = False,
         taken_step_ids: Union[List[str], None] = None,
     ):
-        """Create a step object from step data.
+        """
+        Create a step object from step data.
 
         Beware: the provided 'module_map' dictionary will be modified, as will the 'taken_step_ids'.
         """
-
         if module_map is None:
             module_map = {}
 
@@ -78,7 +92,6 @@ class PipelineStep(Manifest):
             taken_step_ids = []
 
         if not isinstance(step, PipelineStep):
-
             module_type = step.get("module_type", None)
 
             if not module_type:
@@ -93,14 +106,26 @@ class PipelineStep(Manifest):
             )
 
             if module_type not in kiara.module_type_names:
-
                 if module_type in module_map.keys():
                     resolved_module_type = module_map[module_type]["module_type"]
                     resolved_module_config = module_map[module_type]["module_config"]
+
+                    if module_config:
+                        merged_module_config = dict(resolved_module_config)
+                        merged_module_config.setdefault("defaults", {})
+                        merged_module_config.setdefault("constants", {})
+                        defaults = module_config.get("defaults", {})
+                        constants = module_config.get("constants", {})
+                        merged_module_config["defaults"].update(defaults)
+                        merged_module_config["constants"].update(constants)
+                    else:
+                        merged_module_config = resolved_module_config
+
                     manifest = kiara.create_manifest(
                         module_or_operation=resolved_module_type,
-                        config=resolved_module_config,
+                        config=merged_module_config,
                     )
+
                 elif (
                     kiara.operation_registry.is_initialized
                     and module_type in kiara.operation_registry.operation_ids
@@ -108,9 +133,21 @@ class PipelineStep(Manifest):
                     op = kiara.operation_registry.operations[module_type]
                     resolved_module_type = op.module_type
                     resolved_module_config = op.module_config
+
+                    if module_config:
+                        merged_module_config = dict(resolved_module_config)
+                        merged_module_config.setdefault("defaults", {})
+                        merged_module_config.setdefault("constants", {})
+                        defaults = module_config.get("defaults", {})
+                        constants = module_config.get("constants", {})
+                        merged_module_config["defaults"].update(defaults)
+                        merged_module_config["constants"].update(constants)
+                    else:
+                        merged_module_config = dict(resolved_module_config)
+
                     manifest = kiara.create_manifest(
                         module_or_operation=resolved_module_type,
-                        config=resolved_module_config,
+                        config=merged_module_config,
                     )
                 else:
                     raise InvalidPipelineStepConfig(
@@ -151,6 +188,12 @@ class PipelineStep(Manifest):
                 input_links[input_field] = sources
 
             doc = step.get("doc", None)
+            if doc is None:
+                if module.module_type_name == "pipeline":
+                    pc: PipelineConfig = module.config
+                    doc = pc.doc
+                else:
+                    doc = module.doc
 
             # TODO: do we really need the deepcopy here?
             _s = PipelineStep(
@@ -176,7 +219,6 @@ class PipelineStep(Manifest):
         module_map: Union[Mapping[str, Any], None] = None,
         auto_step_ids: bool = False,
     ) -> List["PipelineStep"]:
-
         if module_map is None:
             module_map = {}
         else:
@@ -186,7 +228,6 @@ class PipelineStep(Manifest):
 
         step_ids: List[str] = []
         for step in steps:
-
             _s = cls.create_step(
                 step=step,
                 kiara=kiara,
@@ -198,9 +239,9 @@ class PipelineStep(Manifest):
 
         return result
 
-    @validator("step_id")
+    @field_validator("step_id")
+    @classmethod
     def _validate_step_id(cls, v):
-
         assert isinstance(v, str)
         if "." in v:
             raise ValueError("Step ids can't contain '.' characters.")
@@ -239,9 +280,9 @@ class PipelineStep(Manifest):
     )
     _module: Union["KiaraModule", None] = PrivateAttr(default=None)
 
-    @root_validator(pre=True)
+    @model_validator(mode="before")
+    @classmethod
     def create_step_id(cls, values):
-
         if "module_type" not in values:
             raise ValueError("No 'module_type' specified.")
         if "step_id" not in values or not values["step_id"]:
@@ -249,14 +290,22 @@ class PipelineStep(Manifest):
 
         return values
 
-    @validator("doc", pre=True)
+    def _retrieve_data_to_hash(self) -> Any:
+        data = extract_data_to_hash_from_pipeline_config(self.module_config)
+        return {
+            "module_type": self.module_type,
+            "module_config": data,
+        }
+
+    @field_validator("doc", mode="before")
+    @classmethod
     def validate_doc(cls, value):
         doc = DocumentationMetadataModel.create(value)
         return doc
 
-    @validator("step_id")
+    @field_validator("step_id")
+    @classmethod
     def ensure_valid_id(cls, v):
-
         # TODO: check with regex
         if "." in v or " " in v:
             raise ValueError(
@@ -265,22 +314,21 @@ class PipelineStep(Manifest):
 
         return v
 
-    @validator("module_config", pre=True)
+    @field_validator("module_config", mode="before")
+    @classmethod
     def ensure_dict(cls, v):
-
         if v is None:
             v = {}
         return v
 
-    @validator("input_links", pre=True)
+    @field_validator("input_links", mode="before")
+    @classmethod
     def ensure_input_links_valid(cls, v):
-
         if v is None:
             v = {}
 
         result = {}
         for input_name, output in v.items():
-
             input_links = ensure_step_value_addresses(
                 default_field_name=input_name, link=output
             )
@@ -295,16 +343,40 @@ class PipelineStep(Manifest):
             self._module = m_cls(module_config=self.module_config)
         return self._module
 
-    def __repr__(self):
+    def find_pipeline_inputs(
+        self, pipeline: Union["Pipeline", "PipelineStructure"]
+    ) -> Dict[str, PipelineInputRef]:
+        """Return all the pipeline inputs that are connected to this step.
 
+        Returns a dictionary with the name of the step input as key, and a reference to the pipeline input as value.
+        """
+
+        result = {}
+        for field, field_ref in pipeline.pipeline_input_refs.items():
+            for con_inp in field_ref.connected_inputs:
+                if con_inp.step_id == self.step_id:
+                    result[con_inp.value_name] = field_ref
+
+        return result
+
+    def find_pipeline_outputs(
+        self, pipeline: Union["Pipeline", "PipelineStructure"]
+    ) -> Dict[str, PipelineOutputRef]:
+        result = {}
+        for field, field_ref in pipeline.pipeline_output_refs.items():
+            if field_ref.connected_output.step_id == self.step_id:
+                result[field] = field_ref
+        return result
+
+    def __repr__(self):
         return f"{self.__class__.__name__}(step_id={self.step_id} module_type={self.module_type})"
 
     def __str__(self):
         return f"step: {self.step_id} (module: {self.module_type})"
 
     def create_renderable(self, **config: Any) -> RenderableType:
-
         in_panel = config.get("in_panel", None)
+        display_step_id = config.get("display_step_id", True)
         if in_panel is None:
             if is_jupyter():
                 in_panel = True
@@ -317,7 +389,8 @@ class PipelineStep(Manifest):
 
         if self.doc.is_set:
             table.add_row("", Markdown(self.doc.full_doc))
-        table.add_row("step_id", self.step_id)
+        if display_step_id:
+            table.add_row("step_id", self.step_id)
         table.add_row("module type", self.module_type)
         if not module_config_is_empty(self.module_config):
             mc = dict(self.module_config)
@@ -371,7 +444,6 @@ class PipelineStep(Manifest):
 
 
 def create_input_alias_map(steps: Iterable[PipelineStep]) -> Dict[str, str]:
-
     aliases: Dict[str, str] = {}
     for step in steps:
         field_names = step.module.input_names
@@ -386,7 +458,6 @@ def create_input_alias_map(steps: Iterable[PipelineStep]) -> Dict[str, str]:
 
 
 def create_output_alias_map(steps: Iterable[PipelineStep]) -> Dict[str, str]:
-
     aliases: Dict[str, str] = {}
     for step in steps:
         field_names = step.module.output_names
@@ -401,7 +472,8 @@ def create_output_alias_map(steps: Iterable[PipelineStep]) -> Dict[str, str]:
 
 
 class PipelineConfig(KiaraModuleConfig):
-    """A class to hold the configuration for a [PipelineModule][kiara.pipeline.module.PipelineModule].
+    """
+    A class to hold the configuration for a [PipelineModule][kiara.pipeline.module.PipelineModule].
 
     If you want to control the pipeline input and output names, you need to have to provide a map that uses the
     autogenerated field name ([step_id]__[alias] -- 2 underscores!!) as key, and the desired field name
@@ -419,7 +491,7 @@ class PipelineConfig(KiaraModuleConfig):
     to the shortest possible names for each case.
 
     Examples:
-
+    --------
         Configuration for a pipeline module that functions as a ``nand`` logic gate (in Python):
 
         ``` python
@@ -428,11 +500,11 @@ class PipelineConfig(KiaraModuleConfig):
         nand_p_conf = PipelineConfig(doc="Returns 'False' if both inputs are 'True'.",
                             steps=[and_step, not_step],
                             input_aliases={
-                                "and__a": "a",
-                                "and__b": "b"
+                                "and.a": "a",
+                                "and.b": "b"
                             },
                             output_aliases={
-                                "not__y": "y"
+                                "not.y": "y"
                             }}
         ```
 
@@ -456,17 +528,17 @@ class PipelineConfig(KiaraModuleConfig):
             }
           ],
           "input_aliases": {
-            "and__a": "a",
-            "and__b": "b"
+            "and.a": "a",
+            "and.b": "b"
           },
           "output_aliases": {
-            "not__y": "y"
+            "not.y": "y"
           }
         }
         ```
     """
 
-    _kiara_model_id = "instance.module_config.pipeline"
+    _kiara_model_id: ClassVar = "instance.module_config.pipeline"
 
     @classmethod
     def from_file(
@@ -475,8 +547,7 @@ class PipelineConfig(KiaraModuleConfig):
         kiara: Union["Kiara", None] = None,
         pipeline_name: Union[None, str] = None,
         # module_map: Optional[Mapping[str, Any]] = None,
-    ):
-
+    ) -> "PipelineConfig":
         data = get_data_from_file(path)
         _pipeline_name = data.pop("pipeline_name", None)
 
@@ -496,6 +567,26 @@ class PipelineConfig(KiaraModuleConfig):
             execution_context=execution_context,
         )
 
+    @classmethod
+    def from_string(
+        cls,
+        string_data: str,
+        kiara: Union["Kiara", None] = None,
+        pipeline_name: Union[None, str] = None,
+        # module_map: Optional[Mapping[str, Any]] = None,
+    ) -> "PipelineConfig":
+        data = get_data_from_string(string_data)
+        _pipeline_name = data.pop("pipeline_name", None)
+
+        if pipeline_name:
+            _pipeline_name = pipeline_name
+
+        return cls.from_config(
+            pipeline_name=_pipeline_name,
+            data=data,
+            kiara=kiara,
+        )
+
     @classmethod
     def from_config(
         cls,
@@ -505,8 +596,7 @@ class PipelineConfig(KiaraModuleConfig):
         module_map: Union[Mapping[str, Any], None] = None,
         execution_context: Union[ExecutionContext, None] = None,
         auto_step_ids: bool = False,
-    ):
-
+    ) -> "PipelineConfig":
         if kiara is None:
             from kiara.context import Kiara
 
@@ -537,12 +627,11 @@ class PipelineConfig(KiaraModuleConfig):
         module_map: Union[Mapping[str, Any], None] = None,
         execution_context: Union[ExecutionContext, None] = None,
         auto_step_ids: bool = False,
-    ):
-
+    ) -> "PipelineConfig":
         if execution_context is None:
             execution_context = ExecutionContext()
 
-        repl_dict = execution_context.dict()
+        repl_dict = execution_context.model_dump()
 
         data = dict(data)
 
@@ -584,9 +673,7 @@ class PipelineConfig(KiaraModuleConfig):
         result = cls(pipeline_name=_pipeline_name, **data)
         return result
 
-    class Config:
-        extra = Extra.ignore
-        validate_assignment = True
+    model_config = ConfigDict(extra="ignore", validate_assignment=True)
 
     pipeline_name: str = Field(description="The name of this pipeline.")
     steps: List[PipelineStep] = Field(
@@ -599,20 +686,22 @@ class PipelineConfig(KiaraModuleConfig):
         description="A map of output aliases, with the location of the output (in the format '[step_id].[output_field]') as key, and the pipeline output field name as value.",
     )
     doc: DocumentationMetadataModel = Field(
-        default="-- n/a --", description="Documentation about what the pipeline does."  # type: ignore
+        default="-- n/a --",
+        description="Documentation about what the pipeline does.",  # type: ignore
     )
     context: Dict[str, Any] = Field(
         default_factory=dict, description="Metadata for this workflow."
     )
     _structure: Union["PipelineStructure", None] = PrivateAttr(default=None)
 
-    @validator("doc", pre=True)
+    @field_validator("doc", mode="before")
+    @classmethod
     def validate_doc(cls, value):
         return DocumentationMetadataModel.create(value)
 
-    @validator("steps", pre=True)
+    @field_validator("steps", mode="before")
+    @classmethod
     def _validate_steps(cls, v):
-
         steps = []
         for step in v:
             if not step:
@@ -627,7 +716,6 @@ class PipelineConfig(KiaraModuleConfig):
 
     @property
     def structure(self) -> "PipelineStructure":
-
         if self._structure is not None:
             return self._structure
 
@@ -637,7 +725,6 @@ class PipelineConfig(KiaraModuleConfig):
         return self._structure
 
     def get_raw_config(self) -> Dict[str, Any]:
-
         steps = []
         for step in self.steps:
             src: Dict[str, Any] = {
@@ -648,9 +735,9 @@ class PipelineConfig(KiaraModuleConfig):
             src["step_id"] = step.step_id
             for field, links in step.input_links.items():
                 for link in links:
-                    src.setdefault("input_links", {})[
-                        field
-                    ] = f"{link.step_id}.{link.value_name}"
+                    src.setdefault("input_links", {})[field] = (
+                        f"{link.step_id}.{link.value_name}"
+                    )
             steps.append(src)
 
         return {
@@ -661,14 +748,24 @@ class PipelineConfig(KiaraModuleConfig):
             "output_aliases": self.output_aliases,
         }
 
-    def create_renderable(self, **config: Any) -> RenderableType:
+    def _retrieve_data_to_hash(self) -> Any:
+        data = {
+            "defaults": self.defaults,
+            "constants": self.constants,
+            "steps": [step.model_dump() for step in self.steps],
+            "input_aliases": self.input_aliases,
+            "output_aliases": self.output_aliases,
+        }
+        hash_data = extract_data_to_hash_from_pipeline_config(data)
+        return hash_data
 
+    def create_renderable(self, **config: Any) -> RenderableType:
         table = Table(show_header=False, box=box.SIMPLE)
         table.add_column("key", style="i")
         table.add_column("value")
 
         table.add_row("doc", self.doc.full_doc)
-        table.add_row("structure", self.structure)
+        table.add_row("structure", self.structure.create_renderable(**config))
         return table
 
         # return create_table_from_model_object(self, exclude_fields={"steps"})
@@ -709,5 +806,4 @@ class PipelineConfig(KiaraModuleConfig):
 
 
 def generate_pipeline_endpoint_name(step_id: str, value_name: str):
-
     return f"{step_id}__{value_name}"
