@@ -7,19 +7,30 @@
 
 import abc
 import uuid
-from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Mapping, Type, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    ClassVar,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    Type,
+    Union,
+)
 
 import dpath
 import networkx as nx
 from pydantic import Field, PrivateAttr
 from rich import box
 from rich.console import RenderableType
+from rich.markdown import Markdown
 from rich.panel import Panel
 from rich.table import Table
 
 from kiara.defaults import NONE_VALUE_ID, NOT_SET_VALUE_ID, SpecialValue
 from kiara.exceptions import InvalidValuesException
-from kiara.interfaces.python_api.models.info import ItemInfo
+from kiara.interfaces.python_api.models.info import InfoItemGroup, ItemInfo
 from kiara.models.aliases import AliasValueMap
 from kiara.models.documentation import (
     AuthorsMetadataModel,
@@ -35,7 +46,11 @@ from kiara.models.events.pipeline import (
 from kiara.models.module.jobs import JobConfig
 from kiara.models.module.pipeline import PipelineConfig, StepStatus
 from kiara.models.module.pipeline.structure import PipelineStep, PipelineStructure
-from kiara.models.module.pipeline.value_refs import ValueRef
+from kiara.models.module.pipeline.value_refs import (
+    PipelineInputRef,
+    PipelineOutputRef,
+    ValueRef,
+)
 from kiara.models.values.value import ORPHAN
 from kiara.models.values.value_schema import ValueSchema
 from kiara.registries.data import DataRegistry
@@ -49,6 +64,7 @@ from kiara.utils.yaml import StringYAML
 
 if TYPE_CHECKING:
     from kiara.context import Kiara
+    from kiara.interfaces.python_api.base_api import BaseAPI
 
 yaml = StringYAML()
 
@@ -65,9 +81,13 @@ class Pipeline(object):
     @classmethod
     def create_pipeline(
         cls,
-        kiara: "Kiara",
+        kiara: Union["Kiara", "BaseAPI"],
         pipeline: Union[PipelineConfig, PipelineStructure, Mapping, str],
     ) -> "Pipeline":
+        from kiara.interfaces.python_api.base_api import BaseAPI
+
+        if isinstance(kiara, BaseAPI):
+            kiara = kiara.context
 
         if isinstance(pipeline, Mapping):
             pipeline_structure: PipelineStructure = PipelineConfig.from_config(
@@ -83,7 +103,31 @@ class Pipeline(object):
             if isinstance(module.config, PipelineConfig):
                 config: PipelineConfig = module.config
             else:
-                raise NotImplementedError()
+                step_id = module.module_type_name.replace(".", "_")
+                input_aliases = {}
+                output_aliases = {}
+                for input_name in module.input_names:
+                    input_aliases[f"{step_id}.{input_name}"] = input_name
+                for output_name in module.output_names:
+                    output_aliases[f"{step_id}.{output_name}"] = output_name
+                pipeline_config_data = {
+                    "doc": module.doc,
+                    "steps": [
+                        {
+                            "module_type": module.module_type_name,
+                            "module_config": module.config.model_dump(),
+                            "step_id": step_id,
+                        },
+                    ],
+                    "input_aliases": input_aliases,
+                    "output_aliases": output_aliases,
+                }
+                config = PipelineConfig.from_config(
+                    pipeline_name=module.module_type_name,
+                    data=pipeline_config_data,
+                    kiara=kiara,
+                )
+
             pipeline_structure = config.structure
         else:
             raise Exception(f"Invalid type for argument 'pipeline': {type(pipeline)}")
@@ -92,7 +136,6 @@ class Pipeline(object):
         return pipeline_obj
 
     def __init__(self, structure: PipelineStructure, kiara: "Kiara"):
-
         self._id: uuid.UUID = uuid.uuid4()
 
         self._structure: PipelineStructure = structure
@@ -128,15 +171,19 @@ class Pipeline(object):
         return self._kiara.id
 
     def _init_values(self):
-        """Initialize this object. This should only be called once.
+        """
+        Initialize this object. This should only be called once.
 
         Basically, this goes through all the inputs and outputs of all steps, and 'allocates' a PipelineValueInfo object
         for each of them. In case where output/input or pipeline-input/input points are connected, only one
         value item is allocated, since those refer to the same value.
         """
-
         values = AliasValueMap(
-            alias=str(self.id), version=0, assoc_value=None, values_schema={}
+            alias=str(self.id),
+            version=0,
+            assoc_value=None,
+            values_schema={},
+            assoc_schema=None,
         )
         values._data_registry = self._data_registry
         inputs_schema = self._structure.pipeline_inputs_schema
@@ -164,25 +211,21 @@ class Pipeline(object):
 
         self._all_values = values
 
-        initial_inputs = {
-            k: SpecialValue.NOT_SET
-            for k in self._structure.pipeline_inputs_schema.keys()
-        }
+        initial_inputs = dict.fromkeys(
+            self._structure.pipeline_inputs_schema, SpecialValue.NOT_SET
+        )
         self.set_pipeline_inputs(inputs=initial_inputs)
 
     def __eq__(self, other):
-
         if not isinstance(other, Pipeline):
             return False
 
         return self._id == other._id
 
     def __hash__(self):
-
         return hash(self._id)
 
     def add_listener(self, listener: PipelineListener):
-
         self._listeners.append(listener)
 
     @property
@@ -193,13 +236,24 @@ class Pipeline(object):
     def structure(self) -> PipelineStructure:
         return self._structure
 
+    @property
+    def pipeline_inputs_schema(self):
+        return self._structure.pipeline_inputs_schema
+
+    @property
+    def pipeline_outputs_schema(self):
+        return self._structure.pipeline_outputs_schema
+
+    @property
+    def config(self) -> PipelineConfig:
+        return self._structure.pipeline_config
+
     @property
     def doc(self) -> DocumentationMetadataModel:
         return self.structure.pipeline_config.doc
 
     def get_current_pipeline_inputs(self) -> Dict[str, uuid.UUID]:
         """All (pipeline) input values of this pipeline."""
-
         if not self._structure.steps:
             return {}
 
@@ -208,7 +262,6 @@ class Pipeline(object):
 
     def get_current_pipeline_outputs(self) -> Dict[str, uuid.UUID]:
         """All (pipeline) output values of this pipeline."""
-
         if not self._structure.steps:
             return {}
 
@@ -216,18 +269,15 @@ class Pipeline(object):
         return alias_map.get_all_value_ids()  # type: ignore
 
     def get_current_step_inputs(self, step_id) -> Dict[str, uuid.UUID]:
-
         alias_map = self._all_values.get_alias(f"steps.{step_id}.inputs")
         return alias_map.get_all_value_ids()  # type: ignore
 
     def get_current_step_outputs(self, step_id) -> Dict[str, uuid.UUID]:
-
         alias_map = self._all_values.get_alias(f"steps.{step_id}.outputs")
         return alias_map.get_all_value_ids()  # type: ignore
 
     def get_inputs_for_steps(self, *step_ids: str) -> Dict[str, Dict[str, uuid.UUID]]:
         """Retrieve value ids for the inputs of the specified steps (or all steps, if no argument provided."""
-
         result = {}
         for step_id in self._structure.step_ids:
             if step_ids and step_id not in step_ids:
@@ -238,7 +288,6 @@ class Pipeline(object):
 
     def get_outputs_for_steps(self, *step_ids: str) -> Dict[str, Dict[str, uuid.UUID]]:
         """Retrieve value ids for the outputs of the specified steps (or all steps, if no argument provided."""
-
         result = {}
         for step_id in self._structure.step_ids:
             if step_ids and step_id not in step_ids:
@@ -248,12 +297,10 @@ class Pipeline(object):
         return result
 
     def _notify_pipeline_listeners(self, event: PipelineEvent):
-
         for listener in self._listeners:
             listener._pipeline_event_occurred(event=event)
 
     def get_pipeline_details(self) -> PipelineState:
-
         pipeline_inputs = self._all_values.get_alias("pipeline.inputs")
         pipeline_outputs = self._all_values.get_alias("pipeline.outputs")
 
@@ -285,7 +332,7 @@ class Pipeline(object):
             d = self.get_step_details(step_id)
             step_states[step_id] = d
 
-        details = PipelineState.construct(
+        details = PipelineState(
             kiara_id=self._data_registry.kiara_id,
             pipeline_id=self.pipeline_id,
             pipeline_status=status,
@@ -294,11 +341,11 @@ class Pipeline(object):
             invalid_details=invalid,
             step_states=step_states,
         )
+        details._kiara = self._kiara
 
         return details
 
     def get_step_details(self, step_id: str) -> StepDetails:
-
         step_input_ids = self.get_current_step_inputs(step_id=step_id)
         step_output_ids = self.get_current_step_outputs(step_id=step_id)
         step_inputs = self._all_values.get_alias(f"steps.{step_id}.inputs")
@@ -319,7 +366,7 @@ class Pipeline(object):
         else:
             status = StepStatus.INPUTS_INVALID
 
-        details = StepDetails.construct(
+        details = StepDetails(
             kiara_id=self._data_registry.kiara_id,
             pipeline_id=self.pipeline_id,
             step=self._structure.get_step(step_id=step_id),
@@ -330,14 +377,40 @@ class Pipeline(object):
             invalid_details=invalid,
             processing_stage=processing_stage,
         )
+        details._kiara = self._kiara
         return details
 
+    def set_pipeline_input(
+        self,
+        pipeline_input_field: str,
+        input_value: Any,
+        sync_to_step_inputs: bool = True,
+        notify_listeners: bool = True,
+    ) -> Mapping[str, Mapping[str, Mapping[str, ChangedValue]]]:
+        """Just a utility method, check 'set_pipeline_inputs` for more details."""
+
+        return self.set_pipeline_inputs(
+            {pipeline_input_field: input_value},
+            sync_to_step_inputs=sync_to_step_inputs,
+            notify_listeners=notify_listeners,
+        )
+
     def set_pipeline_inputs(
         self,
         inputs: Mapping[str, Any],
         sync_to_step_inputs: bool = True,
         notify_listeners: bool = True,
     ) -> Mapping[str, Mapping[str, Mapping[str, ChangedValue]]]:
+        """Set one or several pipeline inputs.
+
+        The 'sync_to_step_inputs' parameter determines whether the inputs should be synced to the respective step inputs, which is what you usually want. Only in cases where you don't want to reset/clear any intermediate or end-result values you would set this to False.
+
+        Arguments:
+            inputs: the inputs to set
+            sync_to_step_inputs: whether to sync the inputs to the respective step inputs
+            notify_listeners: whether to notify listeners about the change, in most cases, this would be a PipelineController instance.
+
+        """
 
         values_to_set: Dict[str, uuid.UUID] = {}
 
@@ -381,6 +454,7 @@ class Pipeline(object):
     def sync_pipeline_inputs(
         self, notify_listeners: bool = True
     ) -> Mapping[str, Mapping[str, Mapping[str, ChangedValue]]]:
+        """Sync all pipeline input."""
 
         pipeline_inputs = self.get_current_pipeline_inputs()
 
@@ -411,7 +485,6 @@ class Pipeline(object):
     def _set_step_inputs(
         self, step_id: str, inputs: Mapping[str, Union[uuid.UUID, None]]
     ) -> Mapping[str, Mapping[str, Mapping[str, ChangedValue]]]:
-
         changed_step_inputs = self._set_values(f"steps.{step_id}.inputs", **inputs)
         if not changed_step_inputs:
             return {}
@@ -421,7 +494,8 @@ class Pipeline(object):
         }
 
         step_outputs = self._structure.get_step_output_refs(step_id=step_id)
-        null_outputs = {k: NOT_SET_VALUE_ID for k in step_outputs.keys()}
+        # null_outputs = {k: NOT_SET_VALUE_ID for k in step_outputs.keys()}
+        null_outputs = dict.fromkeys(step_outputs, NOT_SET_VALUE_ID)
 
         changed_outputs = self.set_step_outputs(
             step_id=step_id, outputs=null_outputs, notify_listeners=False
@@ -437,7 +511,6 @@ class Pipeline(object):
         changed_outputs: Mapping[str, Mapping[str, Union[uuid.UUID, None]]],
         notify_listeners: bool = True,
     ) -> Mapping[str, Mapping[str, Mapping[str, ChangedValue]]]:
-
         results: Dict[str, Dict[str, Dict[str, ChangedValue]]] = {}
         for step_id, outputs in changed_outputs.items():
             step_results = self.set_step_outputs(
@@ -458,7 +531,6 @@ class Pipeline(object):
         outputs: Mapping[str, Union[uuid.UUID, None]],
         notify_listeners: bool = True,
     ) -> Mapping[str, Mapping[str, Mapping[str, ChangedValue]]]:
-
         # make sure pedigrees match with respective inputs?
 
         changed_step_outputs = self._set_values(f"steps.{step_id}.outputs", **outputs)
@@ -484,9 +556,9 @@ class Pipeline(object):
                     input_ref.value_name
                 ] = outputs[field_name]
 
-        for step_id, step_inputs in inputs_to_set.items():
+        for _step_id, step_inputs in inputs_to_set.items():
             changed_step_fields = self._set_step_inputs(
-                step_id=step_id, inputs=step_inputs
+                step_id=_step_id, inputs=step_inputs
             )
             dpath.merge(result, changed_step_fields)  # type: ignore
 
@@ -506,7 +578,6 @@ class Pipeline(object):
     def _set_pipeline_outputs(
         self, **outputs: Union[uuid.UUID, None]
     ) -> Mapping[str, ChangedValue]:
-
         changed_pipeline_outputs = self._set_values("pipeline.outputs", **outputs)
         return changed_pipeline_outputs
 
@@ -514,15 +585,14 @@ class Pipeline(object):
         self, alias: str, **values: Union[uuid.UUID, None]
     ) -> Dict[str, ChangedValue]:
         """Set values (value-ids) for the sub-alias-map with the specified alias path."""
-
         invalid = {}
         for k in values.keys():
             _alias = self._all_values.get_alias(alias)
             assert _alias is not None
             if k not in _alias.values_schema.keys():
-                invalid[
-                    k
-                ] = f"Invalid field '{k}'. Available fields: {', '.join(self.get_current_pipeline_inputs().keys())}"
+                invalid[k] = (
+                    f"Invalid field '{k}'. Available fields: {', '.join(self.get_current_pipeline_inputs().keys())}"
+                )
 
         if invalid:
             raise InvalidValuesException(invalid_values=invalid)
@@ -535,7 +605,6 @@ class Pipeline(object):
         changed: Dict[str, ChangedValue] = {}
 
         for field_name, new_value in values.items():
-
             current_value = self._all_values.get_alias(f"{alias}.{field_name}")
             if current_value is not None:
                 current_value_id = current_value.assoc_value
@@ -574,11 +643,24 @@ class Pipeline(object):
         """Return the object representing a step in this workflow, identified by the step id."""
         return self._structure.get_step(step_id)
 
+    def get_pipeline_inputs_schema_for_step(
+        self, step_id: str
+    ) -> Mapping[str, ValueSchema]:
+        """Return the schema for the inputs of the specified step."""
+        return self._structure.get_pipeline_inputs_schema_for_step(step_id=step_id)
+
+    @property
+    def pipeline_input_refs(self) -> Dict[str, PipelineInputRef]:
+        return self._structure.pipeline_input_refs
+
+    @property
+    def pipeline_output_refs(self) -> Dict[str, PipelineOutputRef]:
+        return self._structure.pipeline_output_refs
+
     def get_steps_by_stage(
         self,
     ) -> Mapping[int, Mapping[str, PipelineStep]]:
         """Return a all pipeline steps, ordered by stage they belong to."""
-
         if self._steps_by_stage is not None:
             return self._steps_by_stage
 
@@ -593,7 +675,6 @@ class Pipeline(object):
         return self._steps_by_stage
 
     def create_job_config_for_step(self, step_id: str) -> JobConfig:
-
         step_inputs: Mapping[str, uuid.UUID] = self.get_current_step_inputs(step_id)
         step_details: StepDetails = self.get_step_details(step_id=step_id)
         step: PipelineStep = self.get_step(step_id=step_id)
@@ -612,23 +693,22 @@ class Pipeline(object):
         return job_config
 
     def create_renderable(self, **config: Any) -> RenderableType:
-
         return PipelineInfo.create_from_pipeline(
             kiara=self._kiara, pipeline=self
         ).create_renderable(**config)
 
 
-class PipelineInfo(ItemInfo[Pipeline]):
-
-    _kiara_model_id = "info.pipeline"
+class PipelineInfo(ItemInfo):
+    _kiara_model_id: ClassVar = "info.pipeline"
 
     @classmethod
     def base_instance_class(cls) -> Type[Pipeline]:
         return Pipeline
 
     @classmethod
-    def create_from_instance(cls, kiara: "Kiara", instance: Pipeline, **kwargs):
-
+    def create_from_instance(
+        cls, kiara: "Kiara", instance: Any, **kwargs
+    ) -> "PipelineInfo":
         return cls.create_from_pipeline(kiara=kiara, pipeline=instance)
 
     @classmethod
@@ -636,9 +716,10 @@ class PipelineInfo(ItemInfo[Pipeline]):
         return "pipeline"
 
     @classmethod
-    def create_from_pipeline(cls, kiara: "Kiara", pipeline: Pipeline):
+    def create_from_pipeline(cls, kiara: "Kiara", pipeline: Pipeline) -> "PipelineInfo":
+        # doc = DocumentationMetadataModel.create(None)
 
-        doc = DocumentationMetadataModel.create(None)
+        doc = pipeline.doc
         authors = AuthorsMetadataModel()
         context = ContextMetadataModel()
 
@@ -663,15 +744,14 @@ class PipelineInfo(ItemInfo[Pipeline]):
     )
     pipeline_state: PipelineState = Field(description="The current input details.")
     # stages: Mapping[int, PipelineStage] = Field(description="Details about this pipelines stages/execution order.")
-    _kiara: "Kiara" = PrivateAttr(default=None)
-    _structure: "PipelineStructure" = PrivateAttr(default=None)
+    _kiara: Union["Kiara", None] = PrivateAttr(default=None)
+    _structure: Union["PipelineStructure", None] = PrivateAttr(default=None)
 
     @property
     def pipeline_structure(self):
         return self.pipeline_config.structure
 
     def create_pipeline_table(self, **config: Any) -> Table:
-
         table = Table(box=box.SIMPLE, show_header=False, padding=(0, 0, 0, 0))
         table.add_column("property", style="i")
         table.add_column("value")
@@ -680,12 +760,12 @@ class PipelineInfo(ItemInfo[Pipeline]):
         return table
 
     def _fill_table(self, table: Table, config: Mapping[str, Any]):
-
         include_pipeline_inputs = config.get("include_pipeline_inputs", True)
         include_pipeline_outputs = config.get("include_pipeline_outputs", True)
         include_steps = config.get("include_steps", True)
 
         if include_pipeline_inputs:
+            assert self._kiara is not None
             input_values = self._kiara.data_registry.create_valuemap(
                 data=self.pipeline_state.pipeline_inputs,
                 schema=self.pipeline_structure.pipeline_inputs_schema,
@@ -702,7 +782,6 @@ class PipelineInfo(ItemInfo[Pipeline]):
 
             fields = []
             for stage in sorted(ordered_fields.keys()):
-
                 for f in sorted(ordered_fields[stage]):
                     if f not in fields:
                         fields.append(f)
@@ -727,6 +806,7 @@ class PipelineInfo(ItemInfo[Pipeline]):
             table.add_row("steps", steps)
 
         if include_pipeline_outputs:
+            assert self._kiara is not None
             output_values = self._kiara.data_registry.load_values(
                 values=self.pipeline_state.pipeline_outputs
             )
@@ -760,7 +840,6 @@ class PipelineInfo(ItemInfo[Pipeline]):
             table.add_row("pipeline outputs", t_outputs)
 
     def create_renderable(self, **config: Any) -> RenderableType:
-
         include_details = config.get("include_details", False)
         include_doc = config.get("include_doc", False)
         include_authors = config.get("include_authors", False)
@@ -792,3 +871,64 @@ class PipelineInfo(ItemInfo[Pipeline]):
             )
 
         return table
+
+
+class PipelineGroupInfo(InfoItemGroup):
+    _kiara_model_id: ClassVar = "info.pipelines"
+
+    @classmethod
+    def base_info_class(cls) -> Type[ItemInfo]:
+        return PipelineInfo
+
+    @classmethod
+    def create_from_pipelines(
+        cls, kiara: "Kiara", group_title: Union[str, None] = None, **items: Pipeline
+    ) -> "PipelineGroupInfo":
+        p_infos = {
+            k: PipelineInfo.create_from_pipeline(kiara=kiara, pipeline=v)
+            for k, v in items.items()
+        }
+
+        op_group_info = cls(group_title=group_title, item_infos=p_infos)
+        return op_group_info
+
+    # type_name: Literal["operation_type"] = "operation_type"
+    item_infos: Mapping[str, PipelineInfo] = Field(
+        description="The pipeline info instances for each type."
+    )
+
+    def create_renderable(self, **config: Any) -> RenderableType:
+        return self._create_renderable_list(**config)
+
+    def _create_renderable_list(self, **config) -> RenderableType:
+        full_doc = config.get("full_doc", False)
+        filter = config.get("filter", [])
+
+        table = Table(box=box.SIMPLE, show_header=True)
+        table.add_column("Id", no_wrap=True, style="i")
+        table.add_column("Description")
+
+        p_info: PipelineInfo
+        for op_id, p_info in self.item_infos.items():
+            desc_str = p_info.documentation.description
+            if full_doc:
+                desc = Markdown(p_info.documentation.full_doc)
+            else:
+                desc = Markdown(p_info.documentation.description)
+
+            if filter:
+                match = True
+                for f in filter:
+                    if (
+                        f.lower() not in op_id.lower()
+                        and f.lower() not in desc_str.lower()
+                    ):
+                        match = False
+                        break
+                if match:
+                    table.add_row(op_id, desc)
+
+            else:
+                table.add_row(op_id, desc)
+
+        return table
