@@ -1,21 +1,23 @@
 # -*- coding: utf-8 -*-
-
 #  Copyright (c) 2021, University of Luxembourg / DHARPA project
 #  Copyright (c) 2021, Markus Binsteiner
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 
 from functools import lru_cache
-from typing import Any, Dict, Iterable, List, Mapping, Set, Union
+from typing import Any, ClassVar, Dict, Iterable, List, Mapping, Set, Union
 
 import networkx as nx
-from pydantic import Field, PrivateAttr, root_validator
+from pydantic import Field, PrivateAttr, model_validator
 from rich.console import RenderableType
 from rich.tree import Tree
 
+from kiara.defaults import KIARA_DEFAULT_STAGES_EXTRACTION_TYPE
 from kiara.exceptions import InvalidPipelineConfig
 from kiara.models import KiaraModel
+from kiara.models.documentation import DocumentationMetadataModel
 from kiara.models.module.pipeline import PipelineConfig, PipelineStep
+from kiara.models.module.pipeline.stages import PipelineStage
 from kiara.models.module.pipeline.value_refs import (
     PipelineInputRef,
     PipelineOutputRef,
@@ -28,15 +30,13 @@ from kiara.models.values.value_schema import ValueSchema
 
 
 def generate_pipeline_endpoint_name(step_id: str, value_name: str):
-
     return f"{step_id}__{value_name}"
 
 
 class StepInfo(KiaraModel):
+    _kiara_model_id: ClassVar = "info.pipeline_step"
 
-    _kiara_model_id = "info.pipeline_step"
-
-    step: PipelineStep = Field(description="The pipeline step object.")
+    step: PipelineStep = Field(description="The pipeline step object.", exclude=True)
     inputs: Dict[str, StepInputRef] = Field(
         description="Reference(s) to the fields that feed this steps inputs."
     )
@@ -46,85 +46,47 @@ class StepInfo(KiaraModel):
     required: bool = Field(
         description="Whether this step is always required or whether all his outputs feed into optional input fields."
     )
-    processing_stage: int = Field(
-        description="The index of the processing stage of this step."
+    doc: DocumentationMetadataModel = Field(
+        description="The step documentation.",
+        default_factory=DocumentationMetadataModel.create,
     )
+    # processing_stage: int = Field(
+    #     description="The index of the processing stage of this step."
+    # )
+    _processing_stage: Union[None, int] = PrivateAttr(default=None)
+    _structure: Union[None, "PipelineStructure"] = PrivateAttr(default=None)
 
     @property
     def step_id(self) -> str:
         return self.step.step_id
 
+    @property
+    def processing_stage(self) -> int:
+        if self._processing_stage is not None:
+            return self._processing_stage
 
-class PipelineStage(KiaraModel):
-
-    _kiara_model_id = "info.pipeline_stage"
-
-    @classmethod
-    def from_pipeline_structure(
-        cls, structure: "PipelineStructure"
-    ) -> Dict[int, "PipelineStage"]:
-        used_pipeline_inputs: Set[str] = set()
-        used_pipeline_outputs: Set[str] = set()
-        result = {}
-        for idx, stage in enumerate(structure.processing_stages, start=1):
-            stage_steps = []
-            inputs = []
-            outputs = []
-
-            for step_id in stage:
-                step = structure.get_step(step_id=step_id)
-                stage_steps.append(step.step_id)
-                for pipeline_input, ref in structure.pipeline_input_refs.items():
-                    if pipeline_input in used_pipeline_inputs:
-                        continue
-                    for con_inp in ref.connected_inputs:
-                        if con_inp.step_id == step_id and pipeline_input not in inputs:
-                            inputs.append(pipeline_input)
-                for pipeline_output, out_ref in structure.pipeline_output_refs.items():
-                    if pipeline_output in used_pipeline_outputs:
-                        continue
-                    if out_ref.connected_output.step_id == step_id:
-                        outputs.append(pipeline_output)
-
-            stage_used_inputs = list(used_pipeline_inputs)
-            stage_used_outputs = list(used_pipeline_outputs)
-
-            result[idx] = PipelineStage(
-                stage_index=idx,
-                steps=stage_steps,
-                pipeline_inputs=inputs,
-                pipeline_outputs=outputs,
-                previous_inputs=stage_used_inputs,
-                previous_outputs=stage_used_outputs,
+        if not self._structure:
+            raise Exception(
+                f"Can't look up processing stage for step '{self.step_id}': no structure assigned for those step details, set for this step info."
             )
 
-            used_pipeline_inputs.update(inputs)
-            used_pipeline_outputs.update(outputs)
+        for idx, stage in enumerate(self._structure.processing_stages, start=1):
+            if self.step_id in stage:
+                self._processing_stage = idx
+                return idx
 
-        return result
+        raise Exception(
+            f"Can't look up processing stage for step '{self.step_id}': pipeline structure does not contain step."
+        )
 
-    stage_index: int = Field(description="The index of this stage.")
-    steps: List[str] = Field(
-        description="The pipeline steps that are executed in this stage."
-    )
-    pipeline_inputs: List[str] = Field(
-        description="The pipeline inputs required for this stage."
-    )
-    pipeline_outputs: List[str] = Field(
-        description="The pipeline outputs that are ready once this stage is processed."
-    )
-    previous_inputs: List[str] = Field(
-        description="Pipeline inputs that are already set by this stage."
-    )
-    previous_outputs: List[str] = Field(
-        description="Pipeline outputs that are already computed by this stage."
-    )
+    def create_renderable(self, **config: Any) -> RenderableType:
+        return self.step.create_renderable(**config)
 
 
 class PipelineStructure(KiaraModel):
     """An object that holds one or several steps, and describes the connections between them."""
 
-    _kiara_model_id = "instance.pipeline_structure"
+    _kiara_model_id: ClassVar = "instance.pipeline_structure"
 
     pipeline_config: PipelineConfig = Field(
         description="The underlying pipeline config."
@@ -134,9 +96,9 @@ class PipelineStructure(KiaraModel):
     input_aliases: Dict[str, str] = Field(description="The input aliases.")
     output_aliases: Dict[str, str] = Field(description="The output aliases.")
 
-    @root_validator(pre=True)
-    def validate_pipeline_config(cls, values) -> Dict[str, Any]:
-
+    @model_validator(mode="before")
+    @classmethod
+    def validate_pipeline_config(cls, values: Dict[str, Any]) -> Dict[str, Any]:
         pipeline_config = values.get("pipeline_config", None)
         if not pipeline_config:
             raise ValueError("No 'pipeline_config' provided.")
@@ -158,16 +120,20 @@ class PipelineStructure(KiaraModel):
         if invalid_input_aliases:
             raise InvalidPipelineConfig(
                 f"Invalid input aliases, aliases can't contain special characters: {', '.join(invalid_input_aliases)}.",
-                config=values.get("pipeline_config", None),
+                config=values.get("pipeline_config", None),  # type: ignore
                 details=f"Invalid characters: {', '.join(invalid_input_aliases)}.",
             )
         invalid_output_aliases = [a for a in _input_aliases.values() if "." in a]
         if invalid_input_aliases:
-            raise InvalidPipelineConfig(
-                f"Invalid input aliases, aliases can't contain special characters: {', '.join(invalid_output_aliases)}.",
-                config=values.get("pipeline_config"),
-                details=f"Invalid characters: {', '.join(invalid_output_aliases)}.",
-            )
+            pc = values.get("pipeline_config", None)
+            if pc is None:
+                raise ValueError("No pipeline config provided.")
+            else:
+                raise InvalidPipelineConfig(
+                    f"Invalid input aliases, aliases can't contain special characters: {', '.join(invalid_output_aliases)}.",
+                    config=pc,
+                    details=f"Invalid characters: {', '.join(invalid_output_aliases)}.",
+                )
 
         valid_input_names = set()
         for step in _steps:
@@ -179,10 +145,10 @@ class PipelineStructure(KiaraModel):
         if invalid_input_aliases:
             msg = "Invalid input reference(s)."
             details = "Invalid reference(s):\n"
-            for iia in invalid_input_aliases:
+            for iia in sorted(invalid_input_aliases):
                 details += f" - {iia}\n"
             details += "\nMust be one of: \n"
-            for name in valid_input_names:
+            for name in sorted(valid_input_names):
                 details += f"  - {name}\n"
 
             raise InvalidPipelineConfig(
@@ -197,13 +163,12 @@ class PipelineStructure(KiaraModel):
             a for a in _output_aliases.keys() if a not in valid_output_names
         ]
         if invalid_output_names:
-
             msg = "Invalid output reference(s)."
             details = "Invalid reference(s):\n"
-            for iia in invalid_output_names:
+            for iia in sorted(invalid_output_names):
                 details += f" - {iia}\n"
             details += "\nMust be one of: \n"
-            for name in valid_output_names:
+            for name in sorted(valid_output_names):
                 details += f"  - {name}\n"
 
             raise InvalidPipelineConfig(msg, values.get("pipeline_config", {}), details)
@@ -226,7 +191,7 @@ class PipelineStructure(KiaraModel):
     _data_flow_graph_simple: nx.DiGraph = PrivateAttr(None)  # type: ignore
 
     _processing_stages: List[List[str]] = PrivateAttr(None)  # type: ignore
-    _stages_info: Mapping[int, PipelineStage] = PrivateAttr(None)  # type: ignore
+    # _stages_info: Mapping[int, PipelineStage] = PrivateAttr(None)  # type: ignore
 
     # holds details about the (current) processing steps contained in this workflow
     _steps_details: Dict[str, StepInfo] = PrivateAttr(None)  # type: ignore
@@ -244,7 +209,6 @@ class PipelineStructure(KiaraModel):
 
     @property
     def steps_details(self) -> Mapping[str, StepInfo]:
-
         if self._steps_details is None:
             self._process_steps()
         return self._steps_details  # type: ignore
@@ -257,20 +221,17 @@ class PipelineStructure(KiaraModel):
 
     @property
     def constants(self) -> Mapping[str, Any]:
-
         if self._constants is None:
             self._process_steps()
         return self._constants  # type: ignore
 
     @property
     def defaults(self) -> Mapping[str, Any]:
-
         if self._defaults is None:
             self._process_steps()
         return self._defaults  # type: ignore
 
     def get_step(self, step_id: str) -> PipelineStep:
-
         d = self.steps_details.get(step_id, None)
         if d is None:
             raise Exception(f"No step with id: {step_id}")
@@ -278,7 +239,6 @@ class PipelineStructure(KiaraModel):
         return d.step
 
     def get_step_input_refs(self, step_id: str) -> Mapping[str, StepInputRef]:
-
         d = self.steps_details.get(step_id, None)
         if d is None:
             raise Exception(f"No step with id: {step_id}")
@@ -286,7 +246,6 @@ class PipelineStructure(KiaraModel):
         return d.inputs
 
     def get_step_output_refs(self, step_id: str) -> Mapping[str, StepOutputRef]:
-
         d = self.steps_details.get(step_id, None)
         if d is None:
             raise Exception(f"No step with id: {step_id}")
@@ -294,7 +253,6 @@ class PipelineStructure(KiaraModel):
         return d.outputs
 
     def get_step_details(self, step_id: str) -> StepInfo:
-
         d = self.steps_details.get(step_id, None)
         if d is None:
             raise Exception(f"No step with id: {step_id}")
@@ -321,17 +279,90 @@ class PipelineStructure(KiaraModel):
 
     @property
     def processing_stages(self) -> List[List[str]]:
-        if self._steps_details is None:
-            self._process_steps()
+        if self._processing_stages is not None:
+            return self._processing_stages
+
+        # calculate execution order
+        # process_late = self.pipeline_config.pipeline_name == "topic_modeling"
+        processing_stages = []
+
+        processing_stages = PipelineStage.extract_stages(
+            self, stages_extraction_type=KIARA_DEFAULT_STAGES_EXTRACTION_TYPE
+        )
+
+        self._processing_stages = processing_stages
         return self._processing_stages
 
-    @property
-    def processing_stages_info(self) -> Mapping[int, PipelineStage]:
-        if self._stages_info is not None:
-            return self._stages_info
+    def extract_processing_stages(
+        self, stages_extraction_type: str = KIARA_DEFAULT_STAGES_EXTRACTION_TYPE
+    ) -> List[List[str]]:
+        """
+        Extract a list of lists of steps, representing the order of groups in which they will be executed.
 
-        self._stages_info = PipelineStage.from_pipeline_structure(self)
-        return self._stages_info
+        It is possible to extract the stages in different ways, depending on the use-case you have in mind. For most cases,
+        'late' will be appropriate. Currently available:
+        - 'late': process steps as late in the process as possible
+        - 'early': process steps as early in the process as possible
+        """
+        return PipelineStage.extract_stages(
+            self, stages_extraction_type=stages_extraction_type
+        )
+
+    def extract_processing_stages_info(
+        self, stages_extraction_type: str = KIARA_DEFAULT_STAGES_EXTRACTION_TYPE
+    ) -> List[PipelineStage]:
+        stages = self.extract_processing_stages(
+            stages_extraction_type=stages_extraction_type
+        )
+        return PipelineStage.stages_info_from_pipeline_structure(self, stages)
+
+    def get_stages_graph(
+        self,
+        stages_extraction_type: str = KIARA_DEFAULT_STAGES_EXTRACTION_TYPE,
+        flatten: bool = True,
+    ) -> nx.DiGraph:
+        """
+        Creates a networx graph that represents the processing stages of the pipeline and how they are connecte.
+
+        Arguments:
+        ---------
+            stages_extraction_type: how to extract the stages
+            flatten: if True, the nodes representing connections between stages will be removed, leaving only the edge
+
+        Returns:
+        -------
+            a networkx graph object
+        """
+        stages = self.extract_processing_stages_info(
+            stages_extraction_type=stages_extraction_type
+        )
+
+        graph: nx.DiGraph = nx.DiGraph()
+        for stage in stages:
+            fragment = stage.get_graph_fragment()
+            graph = nx.compose(graph, fragment)
+
+        if flatten:
+            to_flatten = []
+            for node_id in graph.nodes:
+                if graph.nodes[node_id]["type"] in ["stage_output", "connected_output"]:
+                    to_flatten.append(node_id)
+
+            for f in to_flatten:
+                in_edges = tuple(graph.in_edges(f))[0]  # noqa
+                out_edges = tuple(graph.out_edges(f))[0]  # noqa
+                assert in_edges[1] == out_edges[0]
+                graph.remove_edge(in_edges[0], in_edges[1])
+                graph.remove_edge(out_edges[0], out_edges[1])
+                graph.remove_node(f)
+                graph.add_edge(
+                    in_edges[0],
+                    out_edges[1],
+                    type="stage_connection",
+                    output_name=in_edges[1],
+                )
+
+        return graph
 
     @lru_cache()
     def _get_node_of_type(self, node_type: str):
@@ -374,7 +405,6 @@ class PipelineStructure(KiaraModel):
 
     @property
     def pipeline_inputs_schema(self) -> Mapping[str, ValueSchema]:
-
         schemas = {
             input_name: w_in.value_schema
             for input_name, w_in in self.pipeline_input_refs.items()
@@ -391,7 +421,6 @@ class PipelineStructure(KiaraModel):
     def get_pipeline_inputs_schema_for_step(
         self, step_id: str
     ) -> Mapping[str, ValueSchema]:
-
         result = {}
         for field, ref in self.pipeline_input_refs.items():
             for con in ref.connected_inputs:
@@ -403,7 +432,6 @@ class PipelineStructure(KiaraModel):
     def get_pipeline_outputs_schema_for_step(
         self, step_id: str
     ) -> Mapping[str, ValueSchema]:
-
         result = {}
         for field, ref in self.pipeline_output_refs.items():
             if ref.connected_output.step_id == step_id:
@@ -412,11 +440,11 @@ class PipelineStructure(KiaraModel):
         return result
 
     def get_processing_stage(self, step_id: str) -> int:
-        """Return the processing stage for the specified step_id.
+        """
+        Return the processing stage for the specified step_id.
 
         Returns the stage nr (starting with '1').
         """
-
         for index, stage in enumerate(self.processing_stages, start=1):
             if step_id in stage:
                 return index
@@ -425,18 +453,15 @@ class PipelineStructure(KiaraModel):
 
     def step_is_required(self, step_id: str) -> bool:
         """Check if the specified step is required, or can be omitted."""
-
         return self.get_step_details(step_id=step_id).required
 
     def _process_steps(self) -> None:
         """The core method of this class, it connects all the processing modules, their inputs and outputs."""
-
         steps_details: Dict[str, Any] = {}
-        execution_graph = nx.DiGraph()
+        execution_graph: nx.DiGraph = nx.DiGraph()
         execution_graph.add_node("__root__")
-        data_flow_graph = nx.DiGraph()
-        data_flow_graph_simple = nx.DiGraph()
-        processing_stages = []
+        data_flow_graph: nx.DiGraph = nx.DiGraph()
+        data_flow_graph_simple: nx.DiGraph = nx.DiGraph()
         constants = {}
         structure_defaults = {}
 
@@ -447,7 +472,6 @@ class PipelineStructure(KiaraModel):
         _temp_steps_map: Dict[str, PipelineStep] = {}
         pipeline_outputs: Dict[str, PipelineOutputRef] = {}
         for step in self.steps:
-
             _temp_steps_map[step.step_id] = step
 
             if step.step_id in steps_details.keys():
@@ -467,7 +491,6 @@ class PipelineStructure(KiaraModel):
 
             # go through all the module outputs, create points for them and connect them to pipeline outputs
             for output_name, schema in step.module.outputs_schema.items():
-
                 step_output = StepOutputRef(
                     value_name=output_name,
                     value_schema=schema,
@@ -520,7 +543,6 @@ class PipelineStructure(KiaraModel):
         # now process inputs, and connect them to the appropriate output/pipeline-input points
         existing_pipeline_input_points: Dict[str, PipelineInputRef] = {}
         for step in self.steps:
-
             other_step_dependency: Set = set()
             # go through all the inputs of a module, create input points and connect them to either
             # other module outputs, or pipeline inputs (which need to be created)
@@ -530,7 +552,6 @@ class PipelineStructure(KiaraModel):
             )
 
             for input_name, schema in step.module.inputs_schema.items():
-
                 matching_input_links: List[StepValueAddress] = []
                 is_constant = input_name in module_constants.keys()
 
@@ -614,9 +635,9 @@ class PipelineStructure(KiaraModel):
                             is_constant=is_constant,
                         )
 
-                        existing_pipeline_input_points[
-                            pipeline_input_name
-                        ] = connected_pipeline_input
+                        existing_pipeline_input_points[pipeline_input_name] = (
+                            connected_pipeline_input
+                        )
 
                         data_flow_graph.add_node(
                             connected_pipeline_input, type=PipelineInputRef.__name__
@@ -625,9 +646,9 @@ class PipelineStructure(KiaraModel):
                             connected_pipeline_input, type=PipelineInputRef.__name__
                         )
                         if is_constant:
-                            constants[
-                                pipeline_input_name
-                            ] = step.module.get_config_value("constants")[input_name]
+                            constants[pipeline_input_name] = (
+                                step.module.get_config_value("constants")[input_name]
+                            )
 
                         default_val = step.module.get_config_value("defaults").get(
                             input_name, None
@@ -657,6 +678,10 @@ class PipelineStructure(KiaraModel):
 
                 steps_details[step.step_id]["inputs"][input_name] = step_input_point
 
+                if step.doc.is_set:
+                    steps_details[step.step_id]["doc"] = step.doc
+                else:
+                    steps_details[step.step_id]["doc"] = step.module.doc
                 data_flow_graph.add_edge(step_input_point, step)
 
             if other_step_dependency:
@@ -665,53 +690,40 @@ class PipelineStructure(KiaraModel):
             else:
                 execution_graph.add_edge("__root__", step.step_id)
 
-        # calculate execution order
-        path_lengths: Dict[str, int] = {}
-
-        for step in self.steps:
-
-            step_id = step.step_id
-
-            paths = list(nx.all_simple_paths(execution_graph, "__root__", step_id))
-
-            max_steps = max(paths, key=lambda x: len(x))
-            path_lengths[step_id] = len(max_steps) - 1
-
-        if path_lengths.values():
-            max_length = max(path_lengths.values())
-
-            for i in range(1, max_length + 1):
-                stage: List[str] = [
-                    m for m, length in path_lengths.items() if length == i
-                ]
-                processing_stages.append(stage)
-                for _step_id in stage:
-                    steps_details[_step_id]["processing_stage"] = i
-                    # steps_details[_step_id]["step"].processing_stage = i
-
         self._constants = constants
         self._defaults = structure_defaults
-        self._steps_details = {
-            step_id: StepInfo(**data) for step_id, data in steps_details.items()
-        }
+        self._steps_details = {}
+        for step_id, data in steps_details.items():
+            _step = StepInfo(**data)
+            _step._structure = self
+            self._steps_details[step_id] = _step
+
         self._execution_graph = execution_graph
         self._data_flow_graph = data_flow_graph
         self._data_flow_graph_simple = data_flow_graph_simple
-        self._processing_stages = processing_stages
+        self._processing_stages = None  # type: ignore
 
         self._get_node_of_type.cache_clear()
 
-    # def info(self) -> "PipelineStructureInfo":
-    #
-    #     if self._info is not None:
-    #         return self._info
-    #
-    #     from kiara.interfaces.python_api.models.info import PipelineStructureInfo
-    #
-    #     self._info = PipelineStructureInfo.create_from_instance(kiara=None, instance=self)  # type: ignore
-    #     return self._info
+    def export_stages(self):
+        # TODO: implement different processing stages possibilities
+        processing_stages = self.processing_stages
+
+        for stage in processing_stages:
+            input_links = []
+
+            for step_id in stage:
+                step = self.get_step(step_id=step_id)
+                for input_link in step.input_links:
+                    input_links.append(input_link)
 
     def create_renderable(self, **config: Any) -> RenderableType:
+        show_pipeline_inputs_for_steps = config.get(
+            "show_pipeline_inputs_for_steps", False
+        )
+        stages_extraction_type = config.get(
+            "stages_extraction_type", KIARA_DEFAULT_STAGES_EXTRACTION_TYPE
+        )
 
         tree = Tree("pipeline")
         inputs = tree.add("inputs")
@@ -719,14 +731,25 @@ class PipelineStructure(KiaraModel):
             inputs.add(f"[i]{field_name}[i] (type: {schema.type})")
 
         steps = tree.add("steps")
-        for idx, stage in enumerate(self.processing_stages, start=1):
+        processing_stages = PipelineStage.extract_stages(
+            structure=self, stages_extraction_type=stages_extraction_type
+        )
+        for idx, stage in enumerate(processing_stages, start=1):
             stage_node = steps.add(f"stage {idx}")
             for step_id in stage:
                 step_node = stage_node.add(f"step: {step_id}")
                 step = self.get_step(step_id=step_id)
+                if show_pipeline_inputs_for_steps:
+                    pipeline_inputs = self.get_pipeline_inputs_schema_for_step(
+                        step_id=step_id
+                    )
+                    if pipeline_inputs:
+                        inps = step_node.add("pipeline inputs")
+                        for pi in pipeline_inputs:
+                            inps.add(f"[i]{pi}[i]")
                 if step.doc.is_set:
                     step_node.add(f"desc: {step.doc.description}")
-                step_node.add(f"module: {step.manifest_src.module_type}")
+                step_node.add(f"operation: {step.manifest_src.module_type}")
 
         outputs = tree.add("outputs")
         for field_name, schema in self.pipeline_outputs_schema.items():
