@@ -1,29 +1,31 @@
 # -*- coding: utf-8 -*-
+import typing
 
 #  Copyright (c) 2021, University of Luxembourg / DHARPA project
 #  Copyright (c) 2021, Markus Binsteiner
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
-
 import uuid
 from typing import Any, Dict, List, Union
 
-from pydantic import BaseModel, Extra, Field, PrivateAttr, root_validator
+from pydantic import BaseModel, ConfigDict, Field, PrivateAttr, model_validator
 
 from kiara.defaults import PIPELINE_PARENT_MARKER
 from kiara.models.values.value_schema import ValueSchema
 from kiara.utils import camel_case_to_snake_case
 
+if typing.TYPE_CHECKING:
+    pass
 
-def generate_step_alias(step_id: str, value_name):
+
+def generate_step_alias(step_id: str, value_name) -> str:
     return f"{step_id}.{value_name}"
 
 
 class StepValueAddress(BaseModel):
     """Small model to describe the address of a value of a step, within a Pipeline/PipelineStructure."""
 
-    class Config:
-        extra = Extra.forbid
+    model_config = ConfigDict(extra="forbid")
 
     step_id: str = Field(description="The id of a step within a pipeline.")
     value_name: str = Field(
@@ -40,7 +42,6 @@ class StepValueAddress(BaseModel):
         return generate_step_alias(self.step_id, self.value_name)
 
     def __eq__(self, other):
-
         if not isinstance(other, StepValueAddress):
             return False
 
@@ -51,11 +52,9 @@ class StepValueAddress(BaseModel):
         )
 
     def __hash__(self):
-
         return hash((self.step_id, self.value_name, self.sub_value))
 
     def __repr__(self):
-
         if self.sub_value:
             sub_value = f" sub_value={self.sub_value}"
         else:
@@ -67,7 +66,8 @@ class StepValueAddress(BaseModel):
 
 
 class ValueRef(BaseModel):
-    """An object that holds information about the location of a value within a pipeline (or other structure).
+    """
+    An object that holds information about the location of a value within a pipeline (or other structure).
 
     Basically, a `ValueRef` helps the containing object where in its structure the value belongs (for example so
     it can update dependent other values). A `ValueRef` object (obviously) does not contain the value itself.
@@ -83,16 +83,13 @@ class ValueRef(BaseModel):
     reference the same `Value` (in most cases)..
     """
 
-    class Config:
-        allow_mutation = True
-        extra = Extra.forbid
+    model_config = ConfigDict(frozen=False, extra="forbid")
 
     _id: uuid.UUID = PrivateAttr(default_factory=uuid.uuid4)
     value_name: str
     value_schema: ValueSchema
 
     def __eq__(self, other):
-
         if not isinstance(other, self.__class__):
             return False
 
@@ -113,7 +110,8 @@ class ValueRef(BaseModel):
 
 
 class StepInputRef(ValueRef):
-    """An input to a step.
+    """
+    An input to a step.
 
     This object can either have a 'connected_outputs' set, or a 'connected_pipeline_input', not both.
     """
@@ -130,9 +128,9 @@ class StepInputRef(ValueRef):
         description="Whether this input is a constant and can't be changed by the user."
     )
 
-    @root_validator(pre=True)
+    @model_validator(mode="before")
+    @classmethod
     def ensure_single_connected_item(cls, values):
-
         if values.get("connected_outputs", None) and values.get(
             "connected_pipeline_input"
         ):
@@ -156,8 +154,7 @@ class StepInputRef(ValueRef):
 class StepOutputRef(ValueRef):
     """An output to a step."""
 
-    class Config:
-        allow_mutation = True
+    model_config = ConfigDict(frozen=False)
 
     step_id: str = Field(description="The step id.")
     pipeline_output: Union[str, None] = Field(
@@ -207,5 +204,5 @@ class PipelineOutputRef(ValueRef):
         return generate_step_alias(PIPELINE_PARENT_MARKER, self.value_name)
 
 
-StepInputRef.update_forward_refs()
-StepOutputRef.update_forward_refs()
+StepInputRef.model_rebuild()
+StepOutputRef.model_rebuild()
