@@ -4,18 +4,18 @@
 #
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 from abc import abstractmethod
-from typing import Any, Dict, Mapping, Union, get_args
+from typing import Any, Dict, Mapping, Sequence, Union, get_args
 
 import orjson
 import structlog
-from pydantic import PrivateAttr
+from pydantic import ConfigDict, PrivateAttr
 from rich import box
 from rich.console import RenderableType
 from rich.syntax import Syntax
 from rich.table import Table
 
 from kiara.defaults import DEFAULT_ENV_HASH_KEY, ENVIRONMENT_TYPE_CATEGORY_ID
-from kiara.models import KiaraModel
+from kiara.models.metadata import KiaraMetadata
 from kiara.utils.hashing import compute_cid
 from kiara.utils.json import orjson_dumps
 from kiara.utils.output import extract_renderable
@@ -23,23 +23,19 @@ from kiara.utils.output import extract_renderable
 logger = structlog.get_logger()
 
 
-class RuntimeEnvironment(KiaraModel):
-    class Config:
-        underscore_attrs_are_private = False
-        allow_mutation = False
+class RuntimeEnvironment(KiaraMetadata):
+    model_config = ConfigDict(frozen=True)
 
     @classmethod
     def get_environment_type_name(cls) -> str:
-
-        env_type = cls.__fields__["environment_type"]
-        args = get_args(env_type.type_)
+        env_type = cls.model_fields["environment_type"]
+        args: Sequence[str] = get_args(env_type.annotation)
         assert len(args) == 1
 
         return args[0]
 
     @classmethod
     def create_environment_model(cls):
-
         try:
             type_name = cls.get_environment_type_name()
             data = cls.retrieve_environment_data()
@@ -66,7 +62,6 @@ class RuntimeEnvironment(KiaraModel):
     def _create_renderable_for_field(
         self, field_name: str, for_summary: bool = False
     ) -> Union[RenderableType, None]:
-
         return extract_renderable(getattr(self, field_name))
 
     def _retrieve_id(self) -> str:
@@ -74,7 +69,6 @@ class RuntimeEnvironment(KiaraModel):
 
     @property
     def env_hashes(self) -> Mapping[str, str]:
-
         if self._env_hashes is not None:
             return self._env_hashes
 
@@ -87,18 +81,17 @@ class RuntimeEnvironment(KiaraModel):
         return self._env_hashes
 
     def _retrieve_sub_profile_env_data(self) -> Mapping[str, Any]:
-        """Return a dictionary with data that identifies one hash profile per key.
+        """
+        Return a dictionary with data that identifies one hash profile per key.
 
         In most cases, this will only return a single-key dictionary containing all the data in that environment. But
         in some cases one might want to have several sub-hash profiles, for example a list of Python packages with and
         without version information, to have more fine-grained control about when to consider two environments functionally
         equal.
         """
-
         return {DEFAULT_ENV_HASH_KEY: self._retrieve_data_to_hash()}
 
     def create_renderable(self, **config: Any) -> RenderableType:
-
         summary = config.get("summary", False)
 
         table = Table(show_header=False, box=box.SIMPLE)
@@ -110,7 +103,7 @@ class RuntimeEnvironment(KiaraModel):
             "environment hashes", Syntax(hashes_str, "json", background_color="default")
         )
 
-        for field_name, field in self.__fields__.items():
+        for field_name, field in self.model_fields.items():
             summary_item = self._create_renderable_for_field(
                 field_name, for_summary=summary
             )
