@@ -3,7 +3,7 @@ import os
 from pathlib import Path
 from typing import Any, Dict, Iterable, List, Mapping, Union
 
-from pydantic import BaseModel, Field, validator
+from pydantic import BaseModel, Field, field_validator
 
 from kiara.exceptions import KiaraProcessingException
 from kiara.models.module import KiaraModuleConfig
@@ -12,22 +12,21 @@ from kiara.modules import KiaraModule, ValueMapSchema
 
 
 class DataExportResult(BaseModel):
-
     files: List[str] = Field(description="A list of exported files.")
 
-    @validator("files", pre=True)
+    @field_validator("files", mode="before")
+    @classmethod
     def validate_files(cls, value):
-
         if isinstance(value, str):
             value = [value]
 
         # TODO: make sure file exists
+        value = [x.as_posix() if isinstance(x, Path) else x for x in value]
 
         return value
 
 
 class DataExportModuleConfig(KiaraModuleConfig):
-
     target_profile: str = Field(
         description="The name of the target profile. Used to distinguish different target formats for the same data type."
     )
@@ -37,13 +36,11 @@ class DataExportModuleConfig(KiaraModuleConfig):
 
 
 class DataExportModule(KiaraModule):
-
     _config_cls = DataExportModuleConfig
     _module_type_name: Union[str, None] = None
 
     @classmethod
     def retrieve_supported_export_combinations(cls) -> Iterable[Mapping[str, str]]:
-
         result = []
         for attr in dir(cls):
             if (
@@ -76,7 +73,6 @@ class DataExportModule(KiaraModule):
     def create_inputs_schema(
         self,
     ) -> ValueMapSchema:
-
         source_type = self.get_config_value("source_type")
         target_profile = self.get_config_value("target_profile")
 
@@ -130,7 +126,6 @@ class DataExportModule(KiaraModule):
     def create_outputs_schema(
         self,
     ) -> ValueMapSchema:
-
         outputs = {
             "export_details": {
                 "type": "dict",
@@ -140,7 +135,6 @@ class DataExportModule(KiaraModule):
         return outputs
 
     def process(self, inputs: ValueMap, outputs: ValueMap) -> None:
-
         target_profile: str = self.get_config_value("target_profile")
         source_type: str = self.get_config_value("source_type")
 
@@ -182,7 +176,7 @@ class DataExportModule(KiaraModule):
         if export_metadata:
             metadata_file = Path(os.path.join(base_path, f"{name}.metadata"))
             value_info = source_obj.create_info()
-            value_json = value_info.json()
+            value_json = value_info.model_dump_json()
             metadata_file.write_text(value_json)
 
             result.files.append(metadata_file.as_posix())
