@@ -11,9 +11,9 @@ from typing import Any, List, Mapping, Type, Union
 import orjson
 from pydantic import Field
 
-from kiara import KiaraModuleConfig
+from kiara.api import KiaraModuleConfig
 from kiara.exceptions import KiaraProcessingException
-from kiara.models.filesystem import FileBundle, FileModel, FolderImportConfig
+from kiara.models.filesystem import FolderImportConfig, KiaraFile, KiaraFileBundle
 from kiara.models.values.value import SerializedData, ValueMap
 from kiara.modules import (
     DEFAULT_NO_IDEMPOTENT_MODULE_CHARACTERISTICS,
@@ -33,23 +33,54 @@ class ImportLocalFileModule(KiaraModule):
     def create_inputs_schema(
         self,
     ) -> ValueMapSchema:
-
-        return {"path": {"type": "string", "doc": "The local path to the file."}}
+        return {
+            "path": {
+                "type": "string",
+                "doc": "The local path to the file (absolute, or relative to current directory.",
+            }
+        }
 
     def create_outputs_schema(
         self,
     ) -> ValueMapSchema:
-
         return {"file": {"type": "file", "doc": "The loaded files."}}
 
     def _retrieve_module_characteristics(self) -> ModuleCharacteristics:
         return ModuleCharacteristics(is_idempotent=False)
 
     def process(self, inputs: ValueMap, outputs: ValueMap):
-
         path = inputs.get_value_data("path")
 
-        file = FileModel.load_file(source=path)
+        file = KiaraFile.load_file(source=path)
+        outputs.set_value("file", file)
+
+
+class CreateFileFromBytesModule(KiaraModule):
+    """Import a file from a byte array."""
+
+    _module_type_name = "create.file.from.bytes"
+
+    def create_inputs_schema(
+        self,
+    ) -> ValueMapSchema:
+        return {
+            "bytes": {"type": "bytes", "doc": "The bytes array to import."},
+            "file_name": {"type": "string", "doc": "The name of the file."},
+        }
+
+    def create_outputs_schema(
+        self,
+    ) -> ValueMapSchema:
+        return {"file": {"type": "file", "doc": "The loaded files."}}
+
+    def _retrieve_module_characteristics(self) -> ModuleCharacteristics:
+        return ModuleCharacteristics(is_idempotent=False)
+
+    def process(self, inputs: ValueMap, outputs: ValueMap):
+        bytes_value = inputs.get_value_data("bytes")
+        file_name = inputs.get_value_data("file_name")
+
+        file = KiaraFile.from_bytes(bytes_value, file_name=file_name)
         outputs.set_value("file", file)
 
 
@@ -60,7 +91,7 @@ class DeserializeFileModule(DeserializeValueModule):
 
     @classmethod
     def retrieve_supported_target_profiles(cls) -> Mapping[str, Type]:
-        return {"python_object": FileModel}
+        return {"python_object": KiaraFile}
 
     @classmethod
     def retrieve_serialized_value_type(cls) -> str:
@@ -71,7 +102,6 @@ class DeserializeFileModule(DeserializeValueModule):
         return "copy"
 
     def to__python_object(self, data: SerializedData, **config: Any):
-
         keys = list(data.get_keys())
         keys.remove("__file_metadata__")
         assert len(keys) == 1
@@ -90,20 +120,27 @@ class DeserializeFileModule(DeserializeValueModule):
 
         file: str = files[0]  # type: ignore
 
-        fm = FileModel.load_file(
+        _file_name = file_metadata.pop("file_name")
+        _file_metadata = file_metadata.pop("metadata")
+        _file_metadata_schemas = file_metadata.pop("metadata_schemas")
+
+        fm = KiaraFile.load_file(
             source=file,
-            file_name=file_metadata["file_name"],
+            file_name=_file_name,
         )
+        fm.metadata = _file_metadata
+        fm.metadata_schemas = _file_metadata_schemas
         return fm
 
 
 class ImportFileBundleConfig(KiaraModuleConfig):
-
     include_file_types: Union[None, List[str]] = Field(
-        description="File types to include.", default=None
+        description="File types to include. Type is list of strings, which will be matched using 'endswith' test.",
+        default=None,
     )
     exclude_file_types: Union[None, List[str]] = Field(
-        description="File types to include.", default=None
+        description="File types to exclude. Type is list of strings, which will be matched with the 'endswith' test.",
+        default=None,
     )
 
 
@@ -116,7 +153,6 @@ class ImportLocalFileBundleModule(KiaraModule):
     def create_inputs_schema(
         self,
     ) -> ValueMapSchema:
-
         return {
             "path": {"type": "string", "doc": "The local path of the folder to import."}
         }
@@ -124,7 +160,6 @@ class ImportLocalFileBundleModule(KiaraModule):
     def create_outputs_schema(
         self,
     ) -> ValueMapSchema:
-
         return {
             "file_bundle": {"type": "file_bundle", "doc": "The imported file bundle."}
         }
@@ -133,7 +168,6 @@ class ImportLocalFileBundleModule(KiaraModule):
         return DEFAULT_NO_IDEMPOTENT_MODULE_CHARACTERISTICS
 
     def process(self, inputs: ValueMap, outputs: ValueMap):
-
         path = inputs.get_value_data("path")
 
         include = self.get_config_value("include_file_types")
@@ -141,7 +175,7 @@ class ImportLocalFileBundleModule(KiaraModule):
 
         config = FolderImportConfig(include_files=include, exclude_files=exclude)
 
-        file_bundle = FileBundle.import_folder(source=path, import_config=config)
+        file_bundle = KiaraFileBundle.import_folder(source=path, import_config=config)
         outputs.set_value("file_bundle", file_bundle)
 
 
@@ -152,7 +186,7 @@ class DeserializeFileBundleModule(DeserializeValueModule):
 
     @classmethod
     def retrieve_supported_target_profiles(cls) -> Mapping[str, Type]:
-        return {"python_object": FileBundle}
+        return {"python_object": KiaraFileBundle}
 
     @classmethod
     def retrieve_serialized_value_type(cls) -> str:
@@ -163,7 +197,6 @@ class DeserializeFileBundleModule(DeserializeValueModule):
         return "copy"
 
     def to__python_object(self, data: SerializedData, **config: Any):
-
         keys = list(data.get_keys())
         keys.remove("__file_metadata__")
 
@@ -180,25 +213,44 @@ class DeserializeFileBundleModule(DeserializeValueModule):
 
         included_files = {}
         for rel_path in keys:
+            if "size" not in file_metadata[rel_path].keys():
+                # old style, can be removed at some point
+                # file metadata was added feb 2024
 
-            chunks = data.get_serialized_data(rel_path)
-            assert chunks.get_number_of_chunks() == 1
+                chunks = data.get_serialized_data(rel_path)
+                assert chunks.get_number_of_chunks() == 1
+                files = list(chunks.get_chunks(as_files=True, symlink_ok=True))
+                assert len(files) == 1
 
-            files = list(chunks.get_chunks(as_files=True, symlink_ok=True))
-            assert len(files) == 1
+                file: str = files[0]  # type: ignore
+                file_name = file_metadata[rel_path]["file_name"]
+                # import_time = file_metadata[rel_path]["import_time"]
+                fm = KiaraFile.load_file(source=file, file_name=file_name)
+            else:
+                fm = KiaraFile(**file_metadata[rel_path])
+
+                def _load_file():
+                    chunks = data.get_serialized_data(rel_path)
+                    assert chunks.get_number_of_chunks() == 1
+                    files = list(chunks.get_chunks(as_files=True, symlink_ok=True))
+                    assert len(files) == 1
+                    return files[0]
+
+                fm._path_resolver = _load_file
 
-            file: str = files[0]  # type: ignore
-            file_name = file_metadata[rel_path]["file_name"]
-            # import_time = file_metadata[rel_path]["import_time"]
-            fm = FileModel.load_file(source=file, file_name=file_name)
             included_files[rel_path] = fm
 
-        fb = FileBundle(
+        fb_metadata = metadata.pop("metadata")
+        fb_metadata_schemas = metadata.pop("metadata_schemas")
+
+        fb = KiaraFileBundle(
             included_files=included_files,
             bundle_name=bundle_name,
             # import_time=bundle_import_time,
             number_of_files=number_of_files,
             size=sum_size,
+            metadata=fb_metadata,
+            metadata_schemas=fb_metadata_schemas,
         )
         return fb
 
@@ -208,8 +260,7 @@ class ExportFileModule(DataExportModule):
 
     _module_type_name = "export.file"
 
-    def export__file__as__file(self, value: FileModel, base_path: str, name: str):
-
+    def export__file__as__file(self, value: KiaraFile, base_path: str, name: str):
         target_path = os.path.join(base_path, value.file_name)
 
         shutil.copy2(value.path, target_path)
@@ -225,7 +276,6 @@ class PickFileFromFileBundleModule(KiaraModule):
     def create_inputs_schema(
         self,
     ) -> ValueMapSchema:
-
         return {
             "file_bundle": {"type": "file_bundle", "doc": "The file bundle."},
             "path": {"type": "string", "doc": "The relative path of the file to pick."},
@@ -234,12 +284,10 @@ class PickFileFromFileBundleModule(KiaraModule):
     def create_outputs_schema(
         self,
     ) -> ValueMapSchema:
-
         return {"file": {"type": "file", "doc": "The file."}}
 
     def process(self, inputs: ValueMap, outputs: ValueMap):
-
-        file_bundle: FileBundle = inputs.get_value_data("file_bundle")
+        file_bundle: KiaraFileBundle = inputs.get_value_data("file_bundle")
         path: str = inputs.get_value_data("path")
 
         if path not in file_bundle.included_files.keys():
@@ -247,7 +295,7 @@ class PickFileFromFileBundleModule(KiaraModule):
                 f"Can't pick file '{path}' from file bundle: file not available."
             )
 
-        file: FileModel = file_bundle.included_files[path]
+        file: KiaraFile = file_bundle.included_files[path]
 
         outputs.set_value("file", file)
 
@@ -260,7 +308,6 @@ class PickSubBundle(KiaraModule):
     def create_inputs_schema(
         self,
     ) -> ValueMapSchema:
-
         return {
             "file_bundle": {"type": "file_bundle", "doc": "The file bundle."},
             "sub_path": {
@@ -278,8 +325,7 @@ class PickSubBundle(KiaraModule):
         }
 
     def process(self, inputs: ValueMap, outputs: ValueMap):
-
-        file_bundle: FileBundle = inputs.get_value_data("file_bundle")
+        file_bundle: KiaraFileBundle = inputs.get_value_data("file_bundle")
         sub_path: str = inputs.get_value_data("sub_path")
 
         result = {}
@@ -292,7 +338,7 @@ class PickSubBundle(KiaraModule):
                 f"Can't pick sub-folder '{sub_path}' from file bundle: no matches."
             )
 
-        new_file_bundle: FileBundle = FileBundle.create_from_file_models(
+        new_file_bundle: KiaraFileBundle = KiaraFileBundle.create_from_file_models(
             result, bundle_name=f"{file_bundle.bundle_name}_{sub_path}"
         )
 
