@@ -44,7 +44,6 @@ class PipelineModule(KiaraModule):
 
     @property
     def operation(self) -> "Operation":
-
         if self._operation is not None:
             return self._operation
 
@@ -56,7 +55,6 @@ class PipelineModule(KiaraModule):
     def create_inputs_schema(
         self,
     ) -> ValueMapSchema:
-
         pipeline_structure: PipelineStructure = self.config.structure
         inputs_schema = pipeline_structure.pipeline_inputs_schema
         return inputs_schema
@@ -68,9 +66,9 @@ class PipelineModule(KiaraModule):
         return pipeline_structure.pipeline_outputs_schema
 
     def process(self, inputs: ValueMap, outputs: ValueMapWritable, job_log: JobLog):
-
         pipeline_structure: PipelineStructure = self.config.structure
 
+        assert outputs._kiara is not None
         pipeline = Pipeline(structure=pipeline_structure, kiara=outputs._kiara)
 
         assert self._job_registry is not None
@@ -78,8 +76,14 @@ class PipelineModule(KiaraModule):
             pipeline=pipeline, job_registry=self._job_registry
         )
 
+        job_log.add_log("setting pipeline inputs")
         pipeline.set_pipeline_inputs(inputs=inputs)
-        step_details = controller.process_pipeline()
+        job_log.add_log("starting pipeline processing")
+
+        def event_callback(msg: str):
+            job_log.add_log(msg)
+
+        step_details = controller.process_pipeline(event_callback=event_callback)
 
         errors: Dict[str, Union[Exception, uuid.UUID]] = {}
         for step_id, details in step_details.items():
