@@ -7,7 +7,7 @@
 
 from typing import Any, Iterable, Mapping, Tuple, Union
 
-from pydantic import Field, validator
+from pydantic import Field, field_validator
 
 from kiara.models.module import KiaraModuleConfig
 from kiara.models.values.value import ValueMap
@@ -21,11 +21,11 @@ from kiara.utils import log_message
 
 
 class PrettyPrintConfig(KiaraModuleConfig):
-
     source_type: str = Field(description="The value type of the source value.")
     target_type: str = Field(description="The value type of the rendered value.")
 
-    @validator("source_type")
+    @field_validator("source_type")
+    @classmethod
     def validate_source_type(cls, value):
         if value == "render_config":
             raise ValueError(f"Invalid source type: {value}.")
@@ -33,13 +33,11 @@ class PrettyPrintConfig(KiaraModuleConfig):
 
 
 class PrettyPrintModule(KiaraModule):
-
     _module_type_name: str = None  # type: ignore
     _config_cls = PrettyPrintConfig
 
     @classmethod
     def retrieve_supported_render_combinations(cls) -> Iterable[Tuple[str, str]]:
-
         result = []
         for attr in dir(cls):
             if (
@@ -65,7 +63,6 @@ class PrettyPrintModule(KiaraModule):
     def create_inputs_schema(
         self,
     ) -> Mapping[str, Union[ValueSchema, Mapping[str, Any]]]:
-
         source_type = self.get_config_value("source_type")
         assert source_type not in ["target", "base_name"]
 
@@ -83,7 +80,6 @@ class PrettyPrintModule(KiaraModule):
     def create_outputs_schema(
         self,
     ) -> Mapping[str, Union[ValueSchema, Mapping[str, Any]]]:
-
         return {
             "rendered_value": {
                 "type": self.get_config_value("target_type"),
@@ -92,7 +88,6 @@ class PrettyPrintModule(KiaraModule):
         }
 
     def process(self, inputs: ValueMap, outputs: ValueMap):
-
         source_type = self.get_config_value("source_type")
         target_type = self.get_config_value("target_type")
 
@@ -113,7 +108,6 @@ class PrettyPrintModule(KiaraModule):
 
 
 class ValueTypePrettyPrintModule(KiaraModule):
-
     _module_type_name = "pretty_print.value"
     _config_cls = PrettyPrintConfig
 
@@ -123,7 +117,6 @@ class ValueTypePrettyPrintModule(KiaraModule):
     def create_inputs_schema(
         self,
     ) -> Mapping[str, Union[ValueSchema, Mapping[str, Any]]]:
-
         source_type = self.get_config_value("source_type")
         assert source_type not in ["target", "base_name"]
 
@@ -145,7 +138,6 @@ class ValueTypePrettyPrintModule(KiaraModule):
     def create_outputs_schema(
         self,
     ) -> Mapping[str, Union[ValueSchema, Mapping[str, Any]]]:
-
         return {
             "rendered_value": {
                 "type": self.get_config_value("target_type"),
@@ -154,7 +146,6 @@ class ValueTypePrettyPrintModule(KiaraModule):
         }
 
     def process(self, inputs: ValueMap, outputs: ValueMap):
-
         # source_type = self.get_config_value("source_type")
         target_type = self.get_config_value("target_type")
 
@@ -189,13 +180,12 @@ class ValueTypePrettyPrintModule(KiaraModule):
 
 
 class PrettyPrintAnyValueModule(PrettyPrintModule):
-
     _module_type_name = "pretty_print.any.value"
 
     # def pretty_print__any__as__string(self, value: Value, render_config: Dict[str, Any]):
     #
     #     data = value.data
     #     if isinstance(data, KiaraModel):
-    #         return data.json(option=orjson.OPT_INDENT_2)
+    #         return data.model_dump_json(option=orjson.OPT_INDENT_2)
     #     else:
     #         return str(data)
