@@ -10,7 +10,7 @@ from typing import Any, Iterable, Mapping, Tuple, Union
 from pydantic import Field
 
 from kiara.exceptions import KiaraProcessingException
-from kiara.models.data_types import DictModel
+from kiara.models.data_types import KiaraDict
 from kiara.models.module import KiaraModuleConfig
 from kiara.models.rendering import RenderValueResult
 from kiara.models.values.value import Value, ValueMap
@@ -24,7 +24,6 @@ from kiara.utils import log_message
 
 
 class RenderValueModuleConfig(KiaraModuleConfig):
-
     # render_scene_type: str = Field(
     #     description="The id of the model that describes (and handles) the actual rendering."
     # )
@@ -49,7 +48,6 @@ class RenderValueModuleConfig(KiaraModuleConfig):
 class RenderValueModule(KiaraModule):
     @classmethod
     def retrieve_supported_render_combinations(cls) -> Iterable[Tuple[str, str]]:
-
         result = []
         for attr in dir(cls):
             if (
@@ -69,10 +67,12 @@ class RenderValueModule(KiaraModule):
     _config_cls = RenderValueModuleConfig
     _module_type_name: str = None  # type: ignore
 
+    def _retrieve_module_characteristics(self) -> ModuleCharacteristics:
+        return DEFAULT_IDEMPOTENT_INTERNAL_MODULE_CHARACTERISTICS
+
     def create_inputs_schema(
         self,
     ) -> Mapping[str, Union[ValueSchema, Mapping[str, Any]]]:
-
         # instruction = self.get_config_value("render_scene_type")
         # model_registry = ModelRegistry.instance()
         # instr_model_cls: Type[RenderScene] = model_registry.get_model_cls(instruction, required_subclass=RenderScene)  # type: ignore
@@ -99,7 +99,6 @@ class RenderValueModule(KiaraModule):
     def create_outputs_schema(
         self,
     ) -> Mapping[str, Union[ValueSchema, Mapping[str, Any]]]:
-
         outputs = {
             "render_value_result": {
                 "type": "render_value_result",
@@ -110,13 +109,12 @@ class RenderValueModule(KiaraModule):
         return outputs
 
     def process(self, inputs: ValueMap, outputs: ValueMap) -> None:
-
         source_type = self.get_config_value("source_type")
         target_type = self.get_config_value("target_type")
 
         value: Value = inputs.get_value_obj("value")
 
-        render_scene: DictModel = inputs.get_value_data("render_config")
+        render_scene: KiaraDict = inputs.get_value_data("render_config")
         if render_scene:
             rc = render_scene.dict_data
         else:
@@ -153,7 +151,6 @@ class ValueTypeRenderModule(KiaraModule):
     def create_inputs_schema(
         self,
     ) -> Mapping[str, Union[ValueSchema, Mapping[str, Any]]]:
-
         source_type = self.get_config_value("source_type")
         assert source_type not in ["target", "base_name"]
 
@@ -175,7 +172,6 @@ class ValueTypeRenderModule(KiaraModule):
     def create_outputs_schema(
         self,
     ) -> Mapping[str, Union[ValueSchema, Mapping[str, Any]]]:
-
         outputs = {
             "render_value_result": {
                 "type": "render_value_result",
@@ -186,7 +182,6 @@ class ValueTypeRenderModule(KiaraModule):
         return outputs
 
     def process(self, inputs: ValueMap, outputs: ValueMap):
-
         source_value = inputs.get_value_obj("value")
         if not source_value.is_set:
             raise KiaraProcessingException(
@@ -196,7 +191,7 @@ class ValueTypeRenderModule(KiaraModule):
         # source_type = self.get_config_value("source_type")
         target_type = self.get_config_value("target_type")
 
-        render_scene: DictModel = inputs.get_value_data("render_config")
+        render_scene: KiaraDict = inputs.get_value_data("render_config")
 
         try:
             data_type_cls = source_value.data_type_info.data_type_class.get_class()
