@@ -47,18 +47,17 @@ class OperationType(abc.ABC, Generic[OPERATION_TYPE_DETAILS]):
     def retrieve_operation_details(
         self, operation: Union[Operation, str]
     ) -> OPERATION_TYPE_DETAILS:
-        """Retrieve operation details for provided operation.
+        """
+        Retrieve operation details for provided operation.
 
         This is really just a utility method, to make the type checker happy.
         """
-
         if isinstance(operation, str):
             operation = self.operations[operation]
 
         return operation.operation_details  # type: ignore
 
     def create_renderable(self, **config):
-
         info = OperationTypeInfo.create_from_type_class(
             kiara=None, type_cls=self.__class__
         )
