@@ -15,6 +15,7 @@ from multiformats import CID
 from kiara.exceptions import InvalidManifestException
 from kiara.interfaces.python_api.models.info import ModuleTypeInfo, ModuleTypesInfo
 from kiara.models.module.manifest import Manifest
+from kiara.utils import is_debug
 
 if TYPE_CHECKING:
     from kiara.context import Kiara
@@ -25,7 +26,6 @@ logget = structlog.getLogger()
 
 class ModuleRegistry(object):
     def __init__(self, kiara: "Kiara"):
-
         self._kiara: Kiara = kiara
 
         self._cached_modules: Dict[str, Dict[CID, KiaraModule]] = {}
@@ -45,7 +45,6 @@ class ModuleRegistry(object):
         return self._module_classes
 
     def get_module_class(self, module_type: str) -> Type["KiaraModule"]:
-
         cls = self._module_classes.get(module_type, None)
         if cls is None:
             raise InvalidManifestException(
@@ -59,7 +58,6 @@ class ModuleRegistry(object):
         return self._module_classes.keys()
 
     def get_module_type_metadata(self, type_name: str) -> ModuleTypeInfo:
-
         md = self._module_class_metadata.get(type_name, None)
         if md is None:
             md = ModuleTypeInfo.create_from_type_class(
@@ -71,7 +69,6 @@ class ModuleRegistry(object):
     def get_context_metadata(
         self, alias: Union[str, None] = None, only_for_package: Union[str, None] = None
     ) -> ModuleTypesInfo:
-
         result = {}
         for type_name in self.module_types.keys():
             md = self.get_module_type_metadata(type_name=type_name)
@@ -81,32 +78,52 @@ class ModuleRegistry(object):
             else:
                 result[type_name] = md
 
-        return ModuleTypesInfo.construct(group_alias=alias, item_infos=result)  # type: ignore
+        return ModuleTypesInfo(group_title=alias, item_infos=result)  # type: ignore
+
+    def resolve_manifest(self, manifest: Manifest) -> Manifest:
+        """Returns a cloned manifest with resolved module config."""
+
+        if manifest.is_resolved:
+            return manifest.model_copy()
+
+        m_cls = self.get_module_class(manifest.module_type)
+
+        try:
+            resolved = m_cls._resolve_module_config(**manifest.module_config)
+            resolved_dict = resolved.model_dump()
+            manifest_clone = manifest.model_copy(
+                update={"module_config": resolved_dict, "is_resolved": True}
+            )
+            return manifest_clone
+
+        except Exception as e:
+            if is_debug():
+                import traceback
+
+                traceback.print_exc()
+
+            raise InvalidManifestException(
+                f"Error while resolving module config for module '{manifest.module_type}': {e}",
+                module_type=manifest.module_type,
+                module_config=manifest.module_config,
+                parent=e,
+            )
 
     def create_module(self, manifest: Union[Manifest, str]) -> "KiaraModule":
-        """Create a [KiaraModule][kiara.module.KiaraModule] object from a module configuration.
+        """
+        Create a [KiaraModule][kiara.module.KiaraModule] object from a module configuration.
 
         Arguments:
+        ---------
             manifest: the module configuration
         """
-
         if isinstance(manifest, str):
-            manifest = Manifest.construct(module_type=manifest, module_config={})
+            manifest = Manifest(module_type=manifest, module_config={})
 
         m_cls: Type[KiaraModule] = self.get_module_class(manifest.module_type)
 
         if not manifest.is_resolved:
-            try:
-                resolved = m_cls._resolve_module_config(**manifest.module_config)
-            except Exception as e:
-                raise InvalidManifestException(
-                    f"Error while resolving module config for module '{manifest.module_type}': {e}",
-                    module_type=manifest.module_type,
-                    module_config=manifest.module_config,
-                    parent=e,
-                )
-            manifest.module_config = resolved.dict()
-            manifest.is_resolved = True
+            manifest = self.resolve_manifest(manifest)
 
         if self._cached_modules.setdefault(manifest.module_type, {}).get(
             manifest.instance_cid, None
@@ -115,11 +132,7 @@ class ModuleRegistry(object):
 
         if manifest.module_type in self.get_module_type_names():
             kiara_module = m_cls(module_config=manifest.module_config)
-            kiara_module._manifest_cache = Manifest.construct(
-                module_type=manifest.module_type,
-                module_config=manifest.module_config,
-                is_resolved=manifest.is_resolved,
-            )
+            kiara_module._manifest_cache = self.resolve_manifest(manifest)
 
         else:
             raise Exception(
