@@ -7,16 +7,25 @@
 import abc
 import datetime
 import uuid
-from typing import TYPE_CHECKING, Callable, Dict, Iterable, List, Mapping, Union
+from typing import (
+    TYPE_CHECKING,
+    Callable,
+    Dict,
+    Generic,
+    Iterable,
+    List,
+    Mapping,
+    Union,
+)
 
-import pytz
 import structlog
 
 from kiara.exceptions import NoSuchWorkflowException
 from kiara.models.events.workflow_registry import WorkflowArchiveAddedEvent
 from kiara.models.workflow import WorkflowMetadata, WorkflowState
-from kiara.registries import BaseArchive
+from kiara.registries import ARCHIVE_CONFIG_CLS, BaseArchive
 from kiara.registries.ids import ID_REGISTRY
+from kiara.utils.dates import get_current_time_incl_timezone
 
 if TYPE_CHECKING:
     from kiara.context import Kiara
@@ -24,15 +33,11 @@ if TYPE_CHECKING:
 logger = structlog.getLogger()
 
 
-class WorkflowArchive(BaseArchive):
+class WorkflowArchive(BaseArchive[ARCHIVE_CONFIG_CLS], Generic[ARCHIVE_CONFIG_CLS]):
     @classmethod
     def supported_item_types(cls) -> Iterable[str]:
         return ["workflow"]
 
-    @classmethod
-    def is_writeable(cls) -> bool:
-        return False
-
     @abc.abstractmethod
     def retrieve_all_workflow_aliases(self) -> Mapping[str, uuid.UUID]:
         pass
@@ -42,7 +47,7 @@ class WorkflowArchive(BaseArchive):
         pass
 
     @abc.abstractmethod
-    def retrieve_workflow_metadata(self, workflow_id: uuid.UUID):
+    def retrieve_workflow_metadata(self, workflow_id: uuid.UUID) -> WorkflowMetadata:
         pass
 
     # @abc.abstractmethod
@@ -53,9 +58,11 @@ class WorkflowArchive(BaseArchive):
 
     @abc.abstractmethod
     def retrieve_workflow_state(self, workflow_state_id: str) -> WorkflowState:
-        """Retrieve workflow state details for the provided state id.
+        """
+        Retrieve workflow state details for the provided state id.
 
         Arguments:
+        ---------
             workflow_id: id of the workflow
             workflow_state_id: the id of the workflow state
         """
@@ -64,9 +71,11 @@ class WorkflowArchive(BaseArchive):
     def retrieve_all_states_for_workflow(
         self, workflow_id: uuid.UUID
     ) -> Mapping[str, WorkflowState]:
-        """Retrieve workflow state details for the provided state id.
+        """
+        Retrieve workflow state details for the provided state id.
 
         Arguments:
+        ---------
             workflow_id: id of the workflow
             workflow_state_id: the id of the workflow state
         """
@@ -74,7 +83,7 @@ class WorkflowArchive(BaseArchive):
 
 class WorkflowStore(WorkflowArchive):
     @classmethod
-    def is_writeable(cls) -> bool:
+    def _is_writeable(cls) -> bool:
         return True
 
     def register_workflow(
@@ -82,7 +91,6 @@ class WorkflowStore(WorkflowArchive):
         workflow_metadata: WorkflowMetadata,
         workflow_aliases: Union[Iterable[str], None] = None,
     ):
-
         self._register_workflow_metadata(workflow_metadata=workflow_metadata)
         if workflow_aliases:
             if isinstance(workflow_aliases, str):
@@ -94,16 +102,16 @@ class WorkflowStore(WorkflowArchive):
         return workflow_metadata
 
     def unregister_alias(self, alias: str) -> bool:
-        """Unregister a workflow alias.
+        """
+        Unregister a workflow alias.
 
         Returns:
+        -------
             'True' if an alias existed and was unregistered, 'False' otherwise
         """
-
         raise NotImplementedError()
 
     def update_workflow_metadata(self, workflow_metadata: WorkflowMetadata):
-
         self._update_workflow_metadata(workflow_metadata=workflow_metadata)
 
     @abc.abstractmethod
@@ -125,7 +133,6 @@ class WorkflowStore(WorkflowArchive):
 
 class WorkflowRegistry(object):
     def __init__(self, kiara: "Kiara"):
-
         self._kiara: Kiara = kiara
         self._event_callback: Callable = self._kiara.event_registry.add_producer(self)
 
@@ -146,15 +153,11 @@ class WorkflowRegistry(object):
     def register_archive(
         self,
         archive: WorkflowArchive,
-        alias: Union[str, None] = None,
         set_as_default_store: Union[bool, None] = None,
     ):
-
-        workflow_archive_id = archive.archive_id
-        archive.register_archive(kiara=self._kiara)
-
-        if alias is None:
-            alias = str(workflow_archive_id)
+        alias = archive.archive_name
+        if not alias:
+            raise Exception("Invalid workflows archive alias: can't be empty.")
 
         if "." in alias:
             raise Exception(
@@ -166,6 +169,8 @@ class WorkflowRegistry(object):
                 f"Can't add store, workflow archive alias '{alias}' already registered."
             )
 
+        archive.register_archive(kiara=self._kiara)
+
         self._workflow_archives[alias] = archive
         is_store = False
         is_default_store = False
@@ -180,7 +185,7 @@ class WorkflowRegistry(object):
                 is_default_store = True
                 self._default_alias_store = alias
 
-        event = WorkflowArchiveAddedEvent.construct(
+        event = WorkflowArchiveAddedEvent(
             kiara_id=self._kiara.id,
             workflow_archive_id=archive.archive_id,
             workflow_archive_alias=alias,
@@ -191,7 +196,6 @@ class WorkflowRegistry(object):
 
     @property
     def default_alias_store(self) -> str:
-
         if self._default_alias_store is None:
             raise Exception("No default alias store set (yet).")
         return self._default_alias_store
@@ -213,7 +217,6 @@ class WorkflowRegistry(object):
 
     def get_aliases(self, workflow_id: uuid.UUID) -> List[str]:
         """Return all aliases for the specified workflow id."""
-
         return [
             alias
             for alias, w_id in self.workflow_aliases.items()
@@ -223,7 +226,6 @@ class WorkflowRegistry(object):
     @property
     def workflow_aliases(self) -> Dict[str, uuid.UUID]:
         """Retrieve all registered workflow aliases."""
-
         if self._all_aliases is not None:
             return self._all_aliases
 
@@ -246,7 +248,6 @@ class WorkflowRegistry(object):
 
     @property
     def all_workflow_ids(self) -> Iterable[uuid.UUID]:
-
         if self._all_workflow_ids is not None:
             return self._all_workflow_ids.keys()
 
@@ -262,7 +263,6 @@ class WorkflowRegistry(object):
         return self._all_workflow_ids.keys()
 
     def get_workflow_id(self, workflow_alias: str) -> uuid.UUID:
-
         workflow_id = self.workflow_aliases.get(workflow_alias, None)
 
         if workflow_id is None:
@@ -276,7 +276,6 @@ class WorkflowRegistry(object):
     def get_workflow_metadata(
         self, workflow: Union[str, uuid.UUID]
     ) -> WorkflowMetadata:
-
         if isinstance(workflow, str):
             try:
                 workflow_id = uuid.UUID(workflow)
@@ -306,15 +305,17 @@ class WorkflowRegistry(object):
         return workflow_details
 
     def unregister_alias(self, alias: str) -> bool:
-        """Unregister a workflow alias.
+        """
+        Unregister a workflow alias.
 
         Arguments:
+        ---------
             alias: the alias
 
         Returns:
+        -------
             whether an alias existed and was unregistered (True), or not (False)
         """
-
         try:
             self.get_workflow_id(workflow_alias=alias)
         except NoSuchWorkflowException:
@@ -332,16 +333,17 @@ class WorkflowRegistry(object):
         workflow_metadata: Union[None, WorkflowMetadata, str] = None,
         workflow_aliases: Union[Iterable[str], None] = None,
     ) -> WorkflowMetadata:
-        """Register a workflow.
+        """
+        Register a workflow.
 
         If no details are specified, a new WorkflowMetadata object will be created. If a string is provided, a new
         WorkflowMetadata object will be created that uses the string as documentation/description.
 
         Arguments:
+        ---------
             workflow_metadata: the (optional) metadata of the workflow
             workflow_aliases: (optional) aliases to register the workflow under
         """
-
         if workflow_aliases:
             for workflow_alias in workflow_aliases:
                 if workflow_alias in self.workflow_aliases.keys():
@@ -368,9 +370,9 @@ class WorkflowRegistry(object):
         )
 
         self._all_workflow_ids[workflow_metadata.workflow_id] = store_name  # type: ignore
-        self._cached_workflow_metadata_items[
-            workflow_metadata.workflow_id
-        ] = workflow_metadata
+        self._cached_workflow_metadata_items[workflow_metadata.workflow_id] = (
+            workflow_metadata
+        )
 
         if workflow_aliases:
             for workflow_alias in workflow_aliases:
@@ -384,7 +386,6 @@ class WorkflowRegistry(object):
         workflow_state_id: Union[str, None] = None,
         workflow: Union[None, uuid.UUID, str] = None,
     ) -> WorkflowState:
-
         if workflow is None and workflow_state_id is None:
             raise Exception(
                 "Can't retrieve workflow state, neither workflow nor workflow state id specified."
@@ -426,7 +427,6 @@ class WorkflowRegistry(object):
     def get_all_states_for_workflow(
         self, workflow: Union[uuid.UUID, str]
     ) -> Mapping[str, WorkflowState]:
-
         workflow_details = self.get_workflow_metadata(workflow=workflow)
 
         if self._all_workflow_ids is None:
@@ -451,11 +451,10 @@ class WorkflowRegistry(object):
         timestamp: Union[None, datetime.datetime] = None,
         set_current: bool = True,
     ) -> WorkflowMetadata:
-
         workflow_details = self.get_workflow_metadata(workflow=workflow)
 
         if timestamp is None:
-            timestamp = datetime.datetime.now(pytz.utc)
+            timestamp = get_current_time_incl_timezone()
 
         if timestamp in workflow_details.workflow_history.keys():
             if (
@@ -483,7 +482,6 @@ class WorkflowRegistry(object):
         return workflow_details
 
     def update_workflow_metadata(self, workflow_metadata: WorkflowMetadata):
-
         store_name = self.default_alias_store
         store: WorkflowStore = self.get_archive(archive_id=store_name)  # type: ignore
 
