@@ -2,7 +2,7 @@
 import shutil
 import uuid
 from pathlib import Path
-from typing import Dict, Iterable, Mapping, Union
+from typing import Any, Dict, Iterable, Mapping, Union
 
 import orjson
 
@@ -14,20 +14,48 @@ from kiara.utils.windows import fix_windows_longpath
 
 
 class FileSystemWorkflowArchive(WorkflowArchive):
-
     _archive_type_name = "filesystem_workflow_archive"
     _config_cls = FileSystemArchiveConfig  # type: ignore
 
-    def __init__(self, archive_id: uuid.UUID, config: ARCHIVE_CONFIG_CLS):
-
-        super().__init__(archive_id=archive_id, config=config)
+    def __init__(
+        self,
+        archive_name: str,
+        archive_config: ARCHIVE_CONFIG_CLS,
+        force_read_only: bool = False,
+    ):
+        super().__init__(
+            archive_name=archive_name,
+            archive_config=archive_config,
+            force_read_only=force_read_only,
+        )
 
         self._base_path: Union[Path, None] = None
         self.alias_store_path.mkdir(parents=True, exist_ok=True)
 
+    def _retrieve_archive_metadata(self) -> Mapping[str, Any]:
+        if not self.archive_metadata_path.is_file():
+            _archive_metadata = {}
+        else:
+            _archive_metadata = orjson.loads(self.archive_metadata_path.read_bytes())
+
+        archive_id = _archive_metadata.get("archive_id", None)
+        if not archive_id:
+            try:
+                _archive_id = uuid.UUID(self.workflow_store_path.name)
+                _archive_metadata["archive_id"] = str(_archive_id)
+            except Exception:
+                raise Exception(
+                    f"Could not retrieve archive id for alias archive '{self.archive_name}'."
+                )
+
+        return _archive_metadata
+
+    @property
+    def archive_metadata_path(self) -> Path:
+        return self.workflow_store_path / "store_metadata.json"
+
     @property
     def workflow_store_path(self) -> Path:
-
         if self._base_path is not None:
             return self._base_path
 
@@ -38,7 +66,6 @@ class FileSystemWorkflowArchive(WorkflowArchive):
 
     @property
     def alias_store_path(self) -> Path:
-
         return self.workflow_store_path / "aliases"
 
     def _delete_archive(self):
@@ -53,15 +80,12 @@ class FileSystemWorkflowArchive(WorkflowArchive):
         return self.workflow_store_path / "states"
 
     def get_workflow_details_path(self, workflow_id: uuid.UUID) -> Path:
-
         return self.workflow_path / str(workflow_id) / "workflow.json"
 
     def get_alias_path(self, alias: str):
-
         return self.alias_store_path / f"{alias}.alias"
 
     def retrieve_all_workflow_aliases(self) -> Mapping[str, uuid.UUID]:
-
         all_aliases = self.alias_store_path.glob("*.alias")
         result: Dict[str, uuid.UUID] = {}
         for path in all_aliases:
@@ -77,7 +101,6 @@ class FileSystemWorkflowArchive(WorkflowArchive):
         return result
 
     def retrieve_all_workflow_ids(self) -> Iterable[uuid.UUID]:
-
         all_ids = self.workflow_path.glob("*")
         result = []
         for path in all_ids:
@@ -86,7 +109,6 @@ class FileSystemWorkflowArchive(WorkflowArchive):
         return result
 
     def retrieve_workflow_metadata(self, workflow_id: uuid.UUID) -> WorkflowMetadata:
-
         workflow_path = self.get_workflow_details_path(workflow_id=workflow_id)
         if not workflow_path.exists():
             raise NoSuchWorkflowException(
@@ -103,7 +125,6 @@ class FileSystemWorkflowArchive(WorkflowArchive):
         return workflow
 
     def retrieve_workflow_state(self, workflow_state_id: str) -> WorkflowState:
-
         workflow_state_path = self.workflow_states_path / f"{workflow_state_id}.state"
 
         if not workflow_state_path.exists():
@@ -124,7 +145,6 @@ class FileSystemWorkflowArchive(WorkflowArchive):
     def retrieve_all_states_for_workflow(
         self, workflow_id: uuid.UUID
     ) -> Mapping[str, WorkflowState]:
-
         details = self.retrieve_workflow_metadata(workflow_id=workflow_id)
 
         result = {}
@@ -136,11 +156,9 @@ class FileSystemWorkflowArchive(WorkflowArchive):
 
 
 class FileSystemWorkflowStore(FileSystemWorkflowArchive, WorkflowStore):
-
     _archive_type_name = "filesystem_workflow_store"
 
     def _register_workflow_metadata(self, workflow_metadata: WorkflowMetadata):
-
         workflow_path = self.get_workflow_details_path(
             workflow_id=workflow_metadata.workflow_id
         )
@@ -152,11 +170,10 @@ class FileSystemWorkflowStore(FileSystemWorkflowArchive, WorkflowStore):
 
         workflow_path.parent.mkdir(parents=True, exist_ok=False)
 
-        workflow_json = workflow_metadata.json(option=orjson.OPT_NON_STR_KEYS)
+        workflow_json = workflow_metadata.model_dump_json()
         workflow_path.write_text(workflow_json)
 
     def _update_workflow_metadata(self, workflow_metadata: WorkflowMetadata):
-
         workflow_path = self.get_workflow_details_path(
             workflow_id=workflow_metadata.workflow_id
         )
@@ -170,7 +187,6 @@ class FileSystemWorkflowStore(FileSystemWorkflowArchive, WorkflowStore):
         workflow_path.write_text(workflow_json)
 
     def register_alias(self, workflow_id: uuid.UUID, alias: str, force: bool = False):
-
         alias_path = self.get_alias_path(alias=alias)
         if not force and alias_path.exists():
             raise Exception(
@@ -188,7 +204,6 @@ class FileSystemWorkflowStore(FileSystemWorkflowArchive, WorkflowStore):
         alias_path.symlink_to(workflow_path)
 
     def unregister_alias(self, alias: str) -> bool:
-
         alias_path = self.get_alias_path(alias=alias)
 
         if not alias_path.exists():
@@ -198,11 +213,10 @@ class FileSystemWorkflowStore(FileSystemWorkflowArchive, WorkflowStore):
         return True
 
     def add_workflow_state(self, workflow_state: WorkflowState):
-
         self.workflow_states_path.mkdir(exist_ok=True, parents=True)
         workflow_state_path = (
             self.workflow_states_path / f"{workflow_state.instance_id}.state"
         )
 
-        workflow_state_json = workflow_state.json(option=orjson.OPT_NON_STR_KEYS)
+        workflow_state_json = workflow_state.model_dump_json()
         workflow_state_path.write_text(workflow_state_json)
