@@ -2,25 +2,25 @@
 import json
 import os
 from enum import Enum
-from typing import Any, Callable, Dict, Iterable, Mapping, TypeVar, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, Mapping, TypeVar, Union
 
-import orjson
 import rich_click as click
-from click import Command, Option, Parameter
-from pydantic import BaseModel
+from click import Command, Context, Option, Parameter, option
+from rich import box
+from rich.box import Box
 from rich.console import ConsoleRenderable, Group, RichCast
 from rich.panel import Panel
 from rich.rule import Rule
 from rich.syntax import Syntax
 
-from kiara.interfaces import get_console
+from kiara.defaults import KIARA_MAIN_CONFIG_FILE, kiara_app_dirs
 
 # ======================================================================================================================
 # click helper methods
 from kiara.utils import logger
-from kiara.utils.files import get_data_from_file
-from kiara.utils.json import orjson_dumps
-from kiara.utils.output import extract_renderable
+
+if TYPE_CHECKING:
+    from pydantic import BaseModel
 
 F = TypeVar("F", bound=Callable[..., Any])
 FC = TypeVar("FC", bound=Union[Callable[..., Any], Command])
@@ -36,6 +36,20 @@ def _param_memo(f: Union[Callable[..., Any], Command], param: Parameter) -> None
         f.__click_params__.append(param)  # type: ignore
 
 
+HORIZONTALS_NO_TO_AND_BOTTOM: Box = Box(
+    """\
+    
+    
+ ── 
+    
+ ── 
+ ── 
+    
+    
+"""
+)
+
+
 # ======================================================================================================================
 def terminal_print(
     msg: Union[Any, None] = None,
@@ -44,6 +58,8 @@ def terminal_print(
     empty_line_before: bool = False,
     **config: Any,
 ) -> None:
+    from kiara.interfaces import get_console
+    from kiara.utils.output import extract_renderable
 
     if msg is None:
         msg = ""
@@ -89,7 +105,8 @@ class OutputFormat(Enum):
 
 
 def output_format_option(*param_decls: str) -> Callable[[FC], FC]:
-    """Attaches an option to the command.  All positional arguments are
+    """
+    Attaches an option to the command.  All positional arguments are
     passed as parameter declarations to :class:`Option`; all keyword
     arguments are forwarded unchanged (except ``cls``).
     This is equivalent to creating an :class:`Option` instance manually
@@ -98,7 +115,6 @@ def output_format_option(*param_decls: str) -> Callable[[FC], FC]:
     :param cls: the option class to instantiate.  This defaults to
                 :class:`Option`.
     """
-
     if not param_decls:
         param_decls = ("--format", "-f")
 
@@ -117,33 +133,48 @@ def output_format_option(*param_decls: str) -> Callable[[FC], FC]:
     return decorator
 
 
-def render_json_str(model: BaseModel):
+def render_json_str(model: "BaseModel"):
+    # import orjson
 
-    try:
-        json_str = model.json(option=orjson.OPT_INDENT_2 | orjson.OPT_NON_STR_KEYS)
-    except TypeError:
-        json_str = model.json(indent=2)
+    # TODO: pydantic refactor
+    json_str = model.model_dump_json(indent=2)
+
+    # try:
+    #     json_str = model.json(option=orjson.OPT_INDENT_2 | orjson.OPT_NON_STR_KEYS)
+    # except TypeError:
+    #     json_str = model.model_dump_json(indent=2)
 
     return json_str
 
 
-def render_json_schema_str(model: BaseModel):
+def render_json_schema_str(model: "BaseModel"):
+    #
+    # try:
+    #     json_str = model.schema_json(option=orjson.OPT_INDENT_2)
+    # except TypeError:
+    #     json_str = model.schema_json(indent=2)
 
-    try:
-        json_str = model.schema_json(option=orjson.OPT_INDENT_2)
-    except TypeError:
-        json_str = model.schema_json(indent=2)
+    import orjson
+
+    from kiara.utils.json import orjson_dumps
+
+    schema = model.model_json_schema()
+    json_str = orjson_dumps(schema, option=orjson.OPT_INDENT_2)
 
     return json_str
 
 
 def terminal_print_model(
-    *models: BaseModel,
+    *models: "BaseModel",
     format: Union[None, OutputFormat, str] = None,
     empty_line_before: Union[bool, None] = None,
     in_panel: Union[str, None] = None,
     **render_config: Any,
 ):
+    import orjson
+
+    from kiara.utils.json import orjson_dumps
+    from kiara.utils.output import extract_renderable
 
     if format is None:
         format = OutputFormat.TERMINAL
@@ -202,8 +233,10 @@ def terminal_print_model(
 
     elif format == OutputFormat.JSON_SCHEMA:
         if len(models) == 1:
+            _schema = models[0].model_json_schema()
+            schema_str = orjson_dumps(_schema, option=orjson.OPT_INDENT_2)
             syntax = Syntax(
-                models[0].schema_json(option=orjson.OPT_INDENT_2),
+                schema_str,
                 "json",
                 background_color="default",
             )
@@ -225,8 +258,8 @@ def terminal_print_model(
             )
     elif format == OutputFormat.JSON_INCL_SCHEMA:
         if len(models) == 1:
-            data = models[0].dict()
-            schema = models[0].schema()
+            data = models[0].model_dump()
+            schema = models[0].model_json_schema()
             all = {"data": data, "schema": schema}
             json_str = orjson_dumps(all, option=orjson.OPT_INDENT_2)
             syntax = Syntax(json_str, "json", background_color="default")
@@ -238,8 +271,8 @@ def terminal_print_model(
         else:
             all_data = []
             for model in models:
-                data = model.dict()
-                schema = model.schema()
+                data = model.model_dump()
+                schema = model.model_json_schema()
                 all_data.append({"data": data, "schema": schema})
             json_str = orjson_dumps(all_data, option=orjson.OPT_INDENT_2)
             # print(json_str)
@@ -251,7 +284,6 @@ def terminal_print_model(
             )
 
     elif format == OutputFormat.HTML:
-
         all_html = ""
         for model in models:
             if hasattr(model, "create_html"):
@@ -269,7 +301,6 @@ def terminal_print_model(
 def dict_from_cli_args(
     *args: str, list_keys: Union[Iterable[str], None] = None
 ) -> Dict[str, Any]:
-
     if not args:
         return {}
 
@@ -277,6 +308,12 @@ def dict_from_cli_args(
     for arg in args:
         if "=" in arg:
             key, value = arg.split("=", maxsplit=1)
+            if value.startswith("alias:"):
+                if key in config.keys():
+                    raise Exception(f"alias key already set: {key} : {value}")
+                config[key] = value
+                continue
+
             try:
                 _v = json.loads(value)
             except Exception:
@@ -284,6 +321,8 @@ def dict_from_cli_args(
             part_config = {key: _v}
         elif os.path.isfile(os.path.realpath(os.path.expanduser(arg))):
             path = os.path.realpath(os.path.expanduser(arg))
+            from kiara.utils.files import get_data_from_file
+
             part_config = get_data_from_file(path)
             assert isinstance(part_config, Mapping)
         else:
@@ -305,3 +344,101 @@ def dict_from_cli_args(
                 config[k] = v
 
     return config
+
+
+def kiara_version_option(
+    **kwargs: Any,
+) -> Callable[[FC], FC]:
+    """Add a ``--version`` option which immediately prints the version
+    number of kiara and all installed plugins.
+    """
+
+    def callback(ctx: Context, param: Parameter, value: bool) -> None:
+        from rich.table import Table
+
+        if not value or ctx.resilient_parsing:
+            return
+
+        from kiara.models.runtime_environment.python import (
+            PythonRuntimeEnvironment,
+        )
+        from kiara.registries.environment import EnvironmentRegistry
+
+        registry = EnvironmentRegistry.instance()
+        python_env: PythonRuntimeEnvironment = registry.environments["python"]  # type: ignore
+
+        kiara_version = None
+        plugins = {}
+        for pkg in python_env.packages:
+            if pkg.name == "kiara":
+                kiara_version = pkg.version
+            elif pkg.name.startswith("kiara"):
+                plugins[pkg.name] = pkg.version
+
+        table = Table(show_header=False, box=box.SIMPLE)
+        table.add_column("package")
+        table.add_column("version", style="i")
+
+        table.add_row("kiara", kiara_version)
+        table.add_row("", "")
+        for name in sorted(plugins.keys()):
+            table.add_row(name, plugins[name])
+
+        table.add_row("", "")
+        table.add_row("python", python_env.python_version)
+        terminal_print()
+        terminal_print(table, in_panel="Version information")
+
+        ctx.exit()
+
+    param_decls = ("--version", "-v")
+
+    kwargs.setdefault("is_flag", True)
+    kwargs.setdefault("expose_value", False)
+    kwargs.setdefault("is_eager", True)
+    kwargs.setdefault(
+        "help", ("Show the version of kiara and installed plugins, then exit.")
+    )
+    kwargs["callback"] = callback
+    result: Callable[[FC], FC] = option(*param_decls, **kwargs)
+    return result
+
+
+def kiara_runtime_info_option(
+    **kwargs: Any,
+) -> Callable[[FC], FC]:
+    """Add a ``--runtime-info`` option which immediately prints information about the current application environment."""
+
+    def callback(ctx: Context, param: Parameter, value: bool) -> None:
+        from rich.table import Table
+
+        if not value or ctx.resilient_parsing:
+            return
+
+        table = Table(show_header=False, box=box.SIMPLE)
+        table.add_column("key")
+        table.add_column("value", style="i")
+        config_file = KIARA_MAIN_CONFIG_FILE
+        table.add_row("config file", config_file)
+        table.add_row("", "")
+        data_path = kiara_app_dirs.user_data_dir
+        table.add_row("data path", data_path)
+        table.add_row("", "")
+        cache_path = kiara_app_dirs.user_cache_dir
+        table.add_row("cache path", cache_path)
+        table.add_row("", "")
+
+        terminal_print()
+        terminal_print(table, in_panel="Application runtime information")
+
+        ctx.exit()
+
+    param_decls = ("--runtime-info", "-ri")
+
+    kwargs.setdefault("is_flag", True)
+    kwargs.setdefault("expose_value", False)
+    kwargs.setdefault("is_eager", True)
+    kwargs.setdefault("help", ("Show current environment information, then exit."))
+    kwargs["callback"] = callback
+    result: Callable[[FC], FC] = option(*param_decls, **kwargs)
+    return result
