@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 import inspect
 import textwrap
-from typing import List, Union
+from typing import ClassVar, List, Union
 
 import click
 from rich import box
@@ -52,14 +52,16 @@ from rich_click.rich_click import (
     STYLE_OPTIONS_TABLE_SHOW_LINES,
     STYLE_REQUIRED_SHORT,
     USE_CLICK_SHORT_HELP,
-    _get_parameter_help,
-    _make_command_help,
-    _make_rich_rext,
     highlighter,
 )
+from rich_click.rich_help_rendering import (
+    _get_option_help,
+    _make_command_help,
+    _make_rich_rext,
+)
 
-from kiara import ValueMap
-from kiara.interfaces.python_api import KiaraAPI, OperationGroupInfo
+from kiara.api import ValueMap
+from kiara.interfaces.python_api.base_api import BaseAPI
 from kiara.models.module.operation import Operation
 
 # from kiara.interfaces.python_api.operation import KiaraOperation
@@ -73,19 +75,18 @@ from kiara.utils.operations import create_operation_status_renderable
 
 
 def rich_format_filter_operation_help(
-    api: KiaraAPI,
+    api: BaseAPI,
     obj: Union[click.Command, click.Group],
     ctx: click.Context,
     cmd_help: str,
     value: Union[None, str] = None,
 ) -> None:
     """Print nicely formatted help text using rich."""
-
     renderables: List[RenderableType] = []
     # Header text if we have it
     if HEADER_TEXT:
         renderables.append(
-            Padding(_make_rich_rext(HEADER_TEXT, STYLE_HEADER_TEXT), (1, 1, 0, 1))
+            Padding(_make_rich_rext(HEADER_TEXT, STYLE_HEADER_TEXT), (1, 1, 0, 1))  # type: ignore
         )
 
     # Print usage
@@ -108,6 +109,9 @@ def rich_format_filter_operation_help(
         filter_op_type: FilterOperationType = api.get_operation_type("filter")  # type: ignore
         v = api.get_value(value)
         ops = filter_op_type.find_filter_operations_for_data_type(v.data_type_name)
+
+        from kiara.interfaces.python_api.models.info import OperationGroupInfo
+
         ops_info = OperationGroupInfo.create_from_operations(
             kiara=api.context, group_title=f"{v.data_type_name} filters", **ops
         )
@@ -130,7 +134,7 @@ def rich_format_filter_operation_help(
     # Footer text if we have it
     if FOOTER_TEXT:
         renderables.append(
-            Padding(_make_rich_rext(FOOTER_TEXT, STYLE_FOOTER_TEXT), (1, 1, 0, 1))
+            Padding(_make_rich_rext(FOOTER_TEXT, STYLE_FOOTER_TEXT), (1, 1, 0, 1))  # type: ignore
         )
 
     terminal_print(Group(*renderables))
@@ -143,7 +147,8 @@ def rich_format_operation_help(
     op_inputs: ValueMap,
     cmd_help: str,
 ) -> None:
-    """Print nicely formatted help text using rich.
+    """
+    Print nicely formatted help text using rich.
 
     Based on original code from rich-cli, by @willmcgugan.
     https://github.com/Textualize/rich-cli/blob/8a2767c7a340715fc6fbf4930ace717b9b2fc5e5/src/rich_cli/__main__.py#L162-L236
@@ -152,6 +157,7 @@ def rich_format_operation_help(
     Takes a command or group and builds the help text output.
 
     Args:
+    ----
         obj (click.Command or click.Group): Command or group to build help text for
         ctx (click.Context): Click Context object
         table: a rich table, including all the inputs of the current operation
@@ -161,7 +167,7 @@ def rich_format_operation_help(
     # Header text if we have it
     if HEADER_TEXT:
         renderables.append(
-            Padding(_make_rich_rext(HEADER_TEXT, STYLE_HEADER_TEXT), (1, 1, 0, 1))
+            Padding(_make_rich_rext(HEADER_TEXT, STYLE_HEADER_TEXT), (1, 1, 0, 1))  # type: ignore
         )
 
     # Print usage
@@ -197,7 +203,6 @@ def rich_format_operation_help(
     argument_group_options = []
 
     for param in obj.get_params(ctx):
-
         # Skip positional arguments - they don't have opts or helptext and are covered in usage
         # See https://click.palletsprojects.com/en/8.0.x/documentation/#documenting-arguments
         if type(param) is click.core.Argument and not SHOW_ARGUMENTS:
@@ -209,7 +214,7 @@ def rich_format_operation_help(
 
         # Already mentioned in a config option group
         for option_group in option_groups:
-            if any([opt in option_group.get("options", []) for opt in param.opts]):
+            if any(opt in option_group.get("options", []) for opt in param.opts):
                 break
 
         # No break, no mention - add to the default group
@@ -230,10 +235,8 @@ def rich_format_operation_help(
 
     # Print each option group panel
     for option_group in option_groups:
-
         options_rows = []
         for opt in option_group.get("options", []):
-
             # Get the param
             for param in obj.get_params(ctx):
                 if any([opt in param.opts]):
@@ -258,7 +261,11 @@ def rich_format_operation_help(
 
             # Column for a metavar, if we have one
             metavar = Text(style=STYLE_METAVAR, overflow="fold")
-            metavar_str = param.make_metavar()
+            try:
+                metavar_str = param.make_metavar(ctx=ctx)
+            except Exception:
+                # for Python 3.9
+                metavar_str = param.make_metavar()  # type: ignore
 
             # Do it ourselves if this is a positional argument
             if type(param) is click.core.Argument and metavar_str == param.name.upper():  # type: ignore
@@ -289,20 +296,19 @@ def rich_format_operation_help(
 
             # Highlighter to make [ | ] and <> dim
             class MetavarHighlighter(RegexHighlighter):
-                highlights = [
+                highlights: ClassVar = [  # type: ignore
                     r"^(?P<metavar_sep>(\[|<))",
                     r"(?P<metavar_sep>\|)",
                     r"(?P<metavar_sep>(\]|>)$)",
                 ]
 
             metavar_highlighter = MetavarHighlighter()
-
             rows = [
                 required,
                 highlighter(highlighter(",".join(opt_long_strs))),
                 highlighter(highlighter(",".join(opt_short_strs))),
                 metavar_highlighter(metavar),
-                _get_parameter_help(param, ctx),  # type: ignore
+                _get_option_help(param, ctx),  # type: ignore
             ]
 
             # Remove metavar if specified in config
@@ -332,7 +338,7 @@ def rich_format_operation_help(
                 **t_styles,  # type: ignore
             )
             # Strip the required column if none are required
-            if all([x[0] == "" for x in options_rows]):
+            if all(not x[0] for x in options_rows):
                 options_rows = [x[1:] for x in options_rows]
             for row in options_rows:
                 options_table.add_row(*row)
@@ -400,7 +406,7 @@ def rich_format_operation_help(
                 else:
                     # Use short_help function argument if used, or the full help
                     helptext = cmd.short_help or cmd.help or ""
-                commands_table.add_row(command, _make_command_help(helptext))
+                commands_table.add_row(command, _make_command_help(helptext))  # type: ignore
             if commands_table.row_count > 0:
                 renderables.append(
                     Panel(
@@ -452,7 +458,7 @@ def rich_format_operation_help(
     # Footer text if we have it
     if FOOTER_TEXT:
         renderables.append(
-            Padding(_make_rich_rext(FOOTER_TEXT, STYLE_FOOTER_TEXT), (1, 1, 0, 1))
+            Padding(_make_rich_rext(FOOTER_TEXT, STYLE_FOOTER_TEXT), (1, 1, 0, 1))  # type: ignore
         )
 
     group = Group(*renderables)
