@@ -1,20 +1,20 @@
 # -*- coding: utf-8 -*-
-import os
-import os.path
 from enum import Enum
-from typing import Any, ClassVar, Dict, Union
+from typing import Any, ClassVar, Dict, Tuple, Type, Union
 
-from pydantic import BaseModel, BaseSettings, Extra, Field
+from pydantic import BaseModel, ConfigDict, Field
+from pydantic_settings import (
+    BaseSettings,
+    PydanticBaseSettingsSource,
+    SettingsConfigDict,
+)
 from rich.console import Group, RenderableType
 from rich.panel import Panel
 
-from kiara.defaults import KIARA_DEV_CONFIG_FILE
 from kiara.utils import is_develop
-from kiara.utils.files import get_data_from_file
 
 
 def log_dev_message(msg: RenderableType, title: Union[str, None] = None):
-
     if not is_develop():
         return
 
@@ -27,74 +27,68 @@ def log_dev_message(msg: RenderableType, title: Union[str, None] = None):
     terminal_print(panel)
 
 
-def dev_config_file_settings_source(settings: BaseSettings) -> Dict[str, Any]:
-    """
-    A simple settings source that loads variables from a JSON file
-    at the project's root.
-
-    Here we happen to choose to use the `env_file_encoding` from Config
-    when reading `config.json`
-    """
-
-    if os.path.exists(KIARA_DEV_CONFIG_FILE):
-        dev_config = get_data_from_file(KIARA_DEV_CONFIG_FILE)
-    else:
-        dev_config = {}
-    return dev_config
+# def dev_config_file_settings_source(settings: BaseSettings) -> Dict[str, Any]:
+#     """
+#     A simple settings source that loads variables from a JSON file
+#     at the project's root.
+#
+#     Here we happen to choose to use the `env_file_encoding` from Config
+#     when reading `config.json`
+#     """
+#     if os.path.exists(KIARA_DEV_CONFIG_FILE):
+#         dev_config: Dict[str, Any] = get_data_from_file(KIARA_DEV_CONFIG_FILE)
+#     else:
+#         dev_config = {}
+#     return dev_config
 
 
-def profile_settings_source(settings: BaseSettings) -> Dict[str, Any]:
-
-    profile_name = os.environ.get("DEVELOP", None)
-    if not profile_name:
-        profile_name = os.environ.get("develop", None)
-    if not profile_name:
-        profile_name = os.environ.get("DEV", None)
-    if not profile_name:
-        profile_name = os.environ.get("dev", None)
-    if not profile_name:
-        profile_name = os.environ.get("DEV_PROFILE", None)
-    if not profile_name:
-        profile_name = os.environ.get("dev_profile", None)
-
-    result: Dict[str, Any] = {}
-    if not profile_name:
-        return result
-
-    profile_name = profile_name.lower()
-
-    from pydantic.fields import ModelField
-
-    model: ModelField
-
-    for model in KiaraDevSettings.__fields__.values():
-        if not issubclass(model.type_, BaseModel):
-            continue
-
-        profiles = getattr(model.type_, "PROFILES", None)
-        if not profiles:
-            continue
-
-        p = profiles.get(profile_name, None)
-        if not p:
-            continue
-        result[model.name] = p
-
-    return result
+# def profile_settings_source(settings: BaseSettings) -> Dict[str, Any]:
+#
+#     profile_name = os.environ.get("DEVELOP", None)
+#     if not profile_name:
+#         profile_name = os.environ.get("develop", None)
+#     if not profile_name:
+#         profile_name = os.environ.get("DEV", None)
+#     if not profile_name:
+#         profile_name = os.environ.get("dev", None)
+#     if not profile_name:
+#         profile_name = os.environ.get("DEV_PROFILE", None)
+#     if not profile_name:
+#         profile_name = os.environ.get("dev_profile", None)
+#
+#     result: Dict[str, Any] = {}
+#     if not profile_name:
+#         return result
+#
+#     profile_name = profile_name.lower()
+#
+#     for model in KiaraDevSettings.model_fields.values():
+#         raise NotImplementedError("TODO: fix this after pydantic v2 refactoring")
+#         if not issubclass(model.type_, BaseModel):
+#             continue
+#
+#         profiles = getattr(model.type_, "PROFILES", None)
+#         if not profiles:
+#             continue
+#
+#         p = profiles.get(profile_name, None)
+#         if not p:
+#             continue
+#         result[model.name] = p
+#
+#     return result
 
 
 class DetailLevel(Enum):
-
     NONE = "none"
     MINIMAL = "minimal"
     FULL = "full"
 
 
 class PreRunMsgDetails(BaseModel):
-    class Config:
-        extra = Extra.forbid
-        validate_assignment = True
-        use_enum_values = True
+    model_config = ConfigDict(
+        extra="forbid", validate_assignment=True, use_enum_values=True
+    )
 
     pipeline_steps: bool = Field(
         description="Whether to also display information for modules that are run as part of a pipeline.",
@@ -115,10 +109,9 @@ class PreRunMsgDetails(BaseModel):
 
 
 class PostRunMsgDetails(BaseModel):
-    class Config:
-        extra = Extra.forbid
-        validate_assignment = True
-        use_enum_values = True
+    model_config = ConfigDict(
+        extra="forbid", validate_assignment=True, use_enum_values=True
+    )
 
     pipeline_steps: bool = Field(
         description="Whether to also display information for modules that are run as part of a pipeline",
@@ -143,7 +136,6 @@ class PostRunMsgDetails(BaseModel):
 
 
 class KiaraDevLogSettings(BaseModel):
-
     PROFILES: ClassVar[Dict[str, Any]] = {
         "full": {
             "log_pre_run": True,
@@ -165,11 +157,9 @@ class KiaraDevLogSettings(BaseModel):
             "post_run": {"internal_modules": True},
         },
     }
-
-    class Config:
-        extra = Extra.forbid
-        validate_assignment = True
-        use_enum_values = True
+    model_config = ConfigDict(
+        extra="forbid", validate_assignment=True, use_enum_values=True
+    )
 
     exc: DetailLevel = Field(
         description="How detailed to print exceptions", default=DetailLevel.MINIMAL
@@ -192,27 +182,52 @@ class KiaraDevLogSettings(BaseModel):
 
 
 class KiaraDevSettings(BaseSettings):
-    class Config:
+    # TODO[pydantic]: We couldn't refactor this class, please create the `model_config` manually.
+    # Check https://docs.pydantic.dev/dev-v2/migration/#changes-to-config for more information.
 
-        extra = Extra.forbid
-        validate_assignment = True
-        env_prefix = "dev_"
-        use_enum_values = True
-        env_nested_delimiter = "__"
+    model_config = SettingsConfigDict(
+        extra="forbid",
+        validate_assignment=True,
+        use_enum_values=True,
+        env_prefix="dev_",
+        env_nested_delimiter="__",
+    )
 
-        @classmethod
-        def customise_sources(
-            cls,
+    # class Config:
+    #
+    #     extra = Extra.forbid
+    #     validate_assignment = True
+    #     env_prefix = "dev_"
+    #     use_enum_values = True
+    #     env_nested_delimiter = "__"
+    #
+    #     @classmethod
+    #     def customise_sources(
+    #         cls,
+    #         init_settings,
+    #         env_settings,
+    #         file_secret_settings,
+    #     ):
+    #         return (
+    #             init_settings,
+    #             # profile_settings_source,
+    #             dev_config_file_settings_source,
+    #             env_settings,
+    #         )
+
+    @classmethod
+    def settings_customise_sources(
+        cls,
+        settings_cls: Type[BaseSettings],
+        init_settings: PydanticBaseSettingsSource,
+        env_settings: PydanticBaseSettingsSource,
+        dotenv_settings: PydanticBaseSettingsSource,
+        file_secret_settings: PydanticBaseSettingsSource,
+    ) -> Tuple[PydanticBaseSettingsSource, ...]:
+        return (
             init_settings,
             env_settings,
-            file_secret_settings,
-        ):
-            return (
-                init_settings,
-                profile_settings_source,
-                dev_config_file_settings_source,
-                env_settings,
-            )
+        )
 
     log: KiaraDevLogSettings = Field(
         description="Settings about what messages to print in 'develop' mode, and what details to include.",
