@@ -10,7 +10,6 @@ yaml = YAML(typ="safe")
 def get_data_from_url(
     url: str, content_type: Union[str, None] = None
 ) -> Mapping[str, Any]:
-
     if content_type:
         assert content_type in ["json", "yaml"]
 
@@ -35,4 +34,6 @@ def get_data_from_url(
             except Exception:
                 raise ValueError(f"Can't parse data from url '{url}'")
 
+    if not isinstance(result, Mapping):
+        raise ValueError(f"Data from url '{url}' is not a Mapping")
     return result
