@@ -7,6 +7,7 @@
 from typing import TYPE_CHECKING, Any, Iterable, Mapping, Type, Union
 
 import networkx as nx
+from pydantic import RootModel
 from pydantic.main import BaseModel
 from rich.panel import Panel
 from rich.tree import Tree
@@ -23,27 +24,25 @@ def create_pydantic_model(
     _use_pydantic_construct: bool = PYDANTIC_USE_CONSTRUCT,
     **field_values: Any,
 ):
-
     if _use_pydantic_construct:
         raise NotImplementedError()
-        return model_cls.construct(**field_values)
+        return model_cls(**field_values)
     else:
         return model_cls(**field_values)
 
 
 def retrieve_data_subcomponent_keys(data: Any) -> Iterable[str]:
-
-    if hasattr(data, "__custom_root_type__") and data.__custom_root_type__:
-        if isinstance(data.__root__, Mapping):  # type: ignore
+    if isinstance(data, RootModel):
+        if isinstance(data.root, Mapping):
             result = set()
-            for k, v in data.__root__.items():  # type: ignore
+            for k, v in data.root.items():
                 if isinstance(v, BaseModel):
                     result.add(k.split(".")[0])
             return result
         else:
             return []
     elif isinstance(data, BaseModel):
-        matches = sorted(data.__fields__.keys())
+        matches = sorted(data.model_fields.keys())
         return matches
     else:
         log_message(
@@ -54,22 +53,22 @@ def retrieve_data_subcomponent_keys(data: Any) -> Iterable[str]:
 
 def get_subcomponent_from_model(data: "KiaraModel", path: str) -> "KiaraModel":
     """Return subcomponents of a model under a specified path."""
-
     if "." in path:
         first_token, rest = path.split(".", maxsplit=1)
         sc = data.get_subcomponent(first_token)
         return sc.get_subcomponent(rest)
 
-    if hasattr(data, "__custom_root_type__") and data.__custom_root_type__:
-        if isinstance(data.__root__, Mapping):  # type: ignore
-            if path in data.__root__.keys():  # type: ignore
-                return data.__root__[path]  # type: ignore
+    # TODO: pydantic refactor
+    if isinstance(data, RootModel):
+        if isinstance(data.root, Mapping):  # type: ignore
+            if path in data.root.keys():  # type: ignore
+                return data.root[path]  # type: ignore
             else:
                 matches = {}
-                for k in data.__root__.keys():  # type: ignore
+                for k in data.root.keys():  # type: ignore
                     if k.startswith(f"{path}."):
                         rest = k[len(path) + 1 :]
-                        matches[rest] = data.__root__[k]  # type: ignore
+                        matches[rest] = data.root[k]  # type: ignore
 
                 if not matches:
                     raise KeyError(f"No child models under '{path}'.")
@@ -79,10 +78,12 @@ def get_subcomponent_from_model(data: "KiaraModel", path: str) -> "KiaraModel":
                     # return subcomponent_group
 
         else:
-            raise NotImplementedError()
+            raise NotImplementedError("Only mapping root models are supported for now.")
     else:
-        if path in data.__fields__.keys():
-            return getattr(data, path)
+        if path in data.model_fields.keys():
+            result: KiaraModel = getattr(data, path)
+            # TODO: test isinstance?
+            return result
         else:
             raise KeyError(
                 f"No subcomponent for key '{path}' in model: {data.instance_id}."
@@ -90,10 +91,9 @@ def get_subcomponent_from_model(data: "KiaraModel", path: str) -> "KiaraModel":
 
 
 def assemble_subcomponent_graph(data: "KiaraModel") -> Union[nx.DiGraph, None]:
-
     from kiara.models import KiaraModel
 
-    graph = nx.DiGraph()
+    graph: nx.DiGraph = nx.DiGraph()
 
     def assemble_graph(info_model: KiaraModel, current_node_id, level: int = 0):
         graph.add_node(current_node_id, obj=info_model, level=level)
@@ -114,12 +114,10 @@ def assemble_subcomponent_graph(data: "KiaraModel") -> Union[nx.DiGraph, None]:
 def create_subcomponent_tree_renderable(
     data: "KiaraModel", show_data: bool = False
 ) -> Tree:
-
     from kiara.models import KiaraModel
     from kiara.utils.output import extract_renderable
 
     def extract_type_string(obj: Any) -> str:
-
         if isinstance(obj, KiaraModel):
             return f"model: {obj.model_type_id}"
         elif isinstance(obj, Mapping):
@@ -128,7 +126,6 @@ def create_subcomponent_tree_renderable(
             return type(obj).__name__
 
     def assemble_tree(node: Tree, model: Any, level: int):
-
         if isinstance(model, Mapping) and model:
             for k, v in model.items():
                 child_tree = node.add(f"[b i]{k}[/b i] ({extract_type_string(v)})")
