@@ -6,6 +6,7 @@
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 
 import json
+import uuid
 from abc import ABC, abstractmethod
 from enum import Enum
 from typing import (
@@ -23,7 +24,9 @@ from typing import (
 
 import orjson
 import structlog
-from pydantic import BaseModel, Field, root_validator
+from pydantic import BaseModel, Field, model_validator
+from pydantic.v1.main import BaseModel as BaseModel1
+from pydantic_core import PydanticUndefined
 from rich import box
 from rich.console import ConsoleRenderable, Group, RenderableType, RichCast
 from rich.markdown import Markdown
@@ -38,6 +41,7 @@ from kiara.utils.json import orjson_dumps
 if TYPE_CHECKING:
     from pyarrow import Table as ArrowTable
 
+    from kiara.context import Kiara
     from kiara.models.events.pipeline import PipelineState
     from kiara.models.module.pipeline import PipelineStructure
     from kiara.models.values.value_schema import ValueSchema
@@ -47,14 +51,12 @@ log = structlog.getLogger()
 
 
 class RenderConfig(BaseModel):
-
     render_format: str = Field(description="The output format.", default="terminal")
 
 
 class OutputDetails(BaseModel):
     @classmethod
     def from_data(cls, data: Any):
-
         if isinstance(data, str):
             if "=" in data:
                 data = [data]
@@ -82,9 +84,9 @@ class OutputDetails(BaseModel):
         description="Output configuration.", default_factory=dict
     )
 
-    @root_validator(pre=True)
+    @model_validator(mode="before")
+    @classmethod
     def _set_defaults(cls, values) -> Dict[str, Any]:
-
         target: str = values.pop("target", "terminal")
         format: str = values.pop("format", None)
         if format is None:
@@ -145,7 +147,6 @@ class TabularWrap(ABC):
         max_row_height: Union[int, None] = None,
         max_cell_length: Union[int, None] = None,
     ):
-
         table_str = ""
         for cn in self.column_names:
             table_str = f"{table_str}{cn}\t"
@@ -171,7 +172,6 @@ class TabularWrap(ABC):
         max_row_height: Union[int, None] = None,
         max_cell_length: Union[int, None] = None,
     ) -> str:
-
         table_str = "<table><tr>"
         for cn in self.column_names:
             table_str = f"{table_str}<th>{cn}</th>"
@@ -199,7 +199,6 @@ class TabularWrap(ABC):
         max_cell_length: Union[int, None] = None,
         show_table_header: bool = True,
     ) -> RichTable:
-
         rich_table = RichTable(show_header=show_table_header, box=box.SIMPLE)
         if max_row_height == 1:
             overflow = "ignore"
@@ -230,16 +229,13 @@ class TabularWrap(ABC):
         max_row_height: Union[int, None] = None,
         max_cell_length: Union[int, None] = None,
     ) -> Iterator[Iterable[Any]]:
-
         if return_column_names:
             yield self.column_names
 
         num_split_rows = 2
 
         if rows_head is not None:
-
-            if rows_head < 0:
-                rows_head = 0
+            rows_head = max(rows_head, 0)
 
             if rows_head > self.num_rows:
                 rows_head = self.num_rows
@@ -262,7 +258,7 @@ class TabularWrap(ABC):
             num_rows = self.num_rows
 
         table_dict = head.to_pydict()
-        for i in range(0, num_rows):
+        for i in range(0, num_rows):  # noqa
             row = []
             for cn in self.column_names:
                 cell = table_dict[cn][i]
@@ -282,8 +278,7 @@ class TabularWrap(ABC):
                     for line in cell_str.split("\n"):
                         if len(line) > max_cell_length:
                             line = line[0:max_cell_length] + " ..."
-                        else:
-                            line = line
+
                         lines.append(line)
                     cell_str = "\n".join(lines)
 
@@ -292,7 +287,7 @@ class TabularWrap(ABC):
             yield row
 
         if num_split_rows:
-            for i in range(0, num_split_rows):
+            for i in range(0, num_split_rows):  # noqa
                 row = []
                 for _ in self.column_names:
                     row.append("...")
@@ -300,16 +295,13 @@ class TabularWrap(ABC):
 
         if rows_head:
             if rows_tail is not None:
-                if rows_tail < 0:
-                    rows_tail = 0
+                rows_tail = max(rows_tail, 0)
 
                 tail = self.slice(self.num_rows - rows_tail)
                 table_dict = tail.to_pydict()
-                for i in range(0, num_rows):
-
+                for i in range(0, num_rows):  # noqa
                     row = []
                     for cn in self.column_names:
-
                         cell = table_dict[cn][i]
                         cell_str = str(cell)
 
@@ -326,11 +318,8 @@ class TabularWrap(ABC):
                         if max_cell_length and max_cell_length > 0:
                             lines = []
                             for line in cell_str.split("\n"):
-
                                 if len(line) > max_cell_length:
                                     line = line[0:(max_cell_length)] + " ..."
-                                else:
-                                    line = line
                                 lines.append(line)
                             cell_str = "\n".join(lines)
 
@@ -338,30 +327,27 @@ class TabularWrap(ABC):
 
                     yield row
 
-        return
-
 
 class ArrowTabularWrap(TabularWrap):
     def __init__(self, table: "ArrowTable"):
         self._table: "ArrowTable" = table
         super().__init__()
 
-    def retrieve_column_names(self) -> Iterable[str]:
-        return self._table.column_names
+    def retrieve_column_names(self) -> List[str]:
+        return self._table.column_names  # type: ignore
 
     def retrieve_number_of_rows(self) -> int:
-        return self._table.num_rows
+        return self._table.num_rows  # type: ignore
 
     def slice(self, offset: int = 0, length: Union[int, None] = None):
         return self._table.slice(offset=offset, length=length)
 
-    def to_pydict(self) -> Mapping:
-        return self._table.to_pydict()
+    def to_pydict(self) -> Mapping[str, Any]:
+        return self._table.to_pydict()  # type: ignore
 
 
 class DictTabularWrap(TabularWrap):
     def __init__(self, data: Mapping[str, List[Any]]):
-
         self._data: Mapping[str, List[Any]] = data
         # TODO: assert all rows are equal length
         super().__init__()
@@ -377,7 +363,6 @@ class DictTabularWrap(TabularWrap):
         return self._data
 
     def slice(self, offset: int = 0, length: Union[int, None] = None) -> "TabularWrap":
-
         result = {}
         start = None
         end = None
@@ -390,14 +375,55 @@ class DictTabularWrap(TabularWrap):
                     end = len(self._data)
                 else:
                     end = start + length
-                    if end > len(self._data):
-                        end = len(self._data)
+                    end = min(end, len(self._data))
             result[cn] = self._data[cn][start:end]
         return DictTabularWrap(result)
 
 
 def create_table_from_base_model_cls(model_cls: Type[BaseModel]):
+    table = RichTable(box=box.SIMPLE, show_lines=True)
+    table.add_column("Field")
+    table.add_column("Type")
+    table.add_column("Description")
+    table.add_column("Required")
+    table.add_column("Default")
 
+    props = model_cls.model_json_schema().get("properties", {})
+
+    for field_name, field in sorted(model_cls.model_fields.items()):
+        row = [field_name]
+        p = props.get(field_name, None)
+        p_type = None
+        desc = ""
+        if p is not None:
+            p_type = p.get("type", None)
+            # TODO: check 'anyOf' keys
+            desc = p.get("description", "")
+
+        if p_type is None:
+            p_type = "-- check source --"
+        row.append(p_type)
+
+        row.append(desc)
+        row.append("yes" if field.is_required() else "no")
+        default = field.default
+        if callable(default):
+            default = default()
+
+        if default in [None, PydanticUndefined]:
+            default = ""
+        else:
+            try:
+                default = json.dumps(default, indent=2)
+            except Exception:
+                default = str(default)
+        row.append(default)
+        table.add_row(*row)
+
+    return table
+
+
+def create_table_from_base_model_v1_cls(model_cls: Type[BaseModel1]):
     table = RichTable(box=box.SIMPLE, show_lines=True)
     table.add_column("Field")
     table.add_column("Type")
@@ -448,7 +474,6 @@ def create_dict_from_field_schemas(
     _constants: Union[Mapping[str, Any], None] = None,
     _doc_to_string: bool = True,
 ) -> Mapping[str, List[Any]]:
-
     table: Dict[str, List[Any]] = {}
     table["field_name"] = []
     table["data_type"] = []
@@ -460,7 +485,6 @@ def create_dict_from_field_schemas(
         table["default"] = []
 
     for field_name, schema in fields.items():
-
         table["field_name"].append(field_name)
         table["data_type"].append(schema.type)
         if _doc_to_string:
@@ -496,7 +520,6 @@ def create_table_from_field_schemas(
     _show_header: bool = False,
     _constants: Union[Mapping[str, Any], None] = None,
 ) -> RichTable:
-
     table = RichTable(box=box.SIMPLE, show_header=_show_header)
     table.add_column("field name", style="i", overflow="fold")
     table.add_column("type")
@@ -510,7 +533,6 @@ def create_table_from_field_schemas(
         else:
             table.add_column("Default")
     for field_name, schema in fields.items():
-
         row: List[RenderableType] = [field_name, schema.type, schema.doc]
 
         if _add_required:
@@ -552,7 +574,6 @@ def create_value_map_status_renderable(
     render_config: Union[Mapping[str, Any], None] = None,
     fields: Union[None, Iterable[str]] = None,
 ) -> RichTable:
-
     if render_config is None:
         render_config = {}
 
@@ -561,6 +582,10 @@ def create_value_map_status_renderable(
     show_required: bool = render_config.get("show_required", True)
     show_default: bool = render_config.get("show_default", True)
     show_value_ids: bool = render_config.get("show_value_ids", False)
+    show_data: bool = render_config.get("show_data", False)
+    data_max_no_rows: Union[int, None] = render_config.get("max_no_rows", None)
+    if data_max_no_rows is None:
+        data_max_no_rows = render_config.get("max_lines", 0)
 
     table = RichTable(box=box.SIMPLE, show_header=True)
     table.add_column("field name", style="i")
@@ -579,6 +604,9 @@ def create_value_map_status_renderable(
     if show_value_ids:
         table.add_column("value id", overflow="fold")
 
+    if show_data:
+        table.add_column("data")
+
     invalid = inputs.check_invalid()
 
     if fields:
@@ -587,7 +615,6 @@ def create_value_map_status_renderable(
         field_order = sorted(inputs.keys())
 
     for field_name in field_order:
-
         value = inputs.get(field_name, None)
         if value is None:
             log.debug(
@@ -642,6 +669,17 @@ def create_value_map_status_renderable(
         if show_value_ids:
             row.append(str(inputs.get_value_obj(field_name=field_name).value_id))
 
+        if show_data:
+            render_config = dict(render_config)
+            render_config["max_no_rows"] = data_max_no_rows
+            assert value._data_registry is not None
+            data = value._data_registry.pretty_print_data(
+                value_id=value.value_id,
+                target_type="terminal_renderable",
+                **render_config,
+            )
+            row.append(data)
+
         table.add_row(*row)
 
     return table
@@ -651,19 +689,32 @@ def create_table_from_model_object(
     model: BaseModel,
     render_config: Union[Mapping[str, Any], None] = None,
     exclude_fields: Union[Set[str], None] = None,
-):
-
+) -> RichTable:
     model_cls = model.__class__
 
-    table = RichTable(box=box.SIMPLE, show_lines=True)
+    show_header: bool = True
+    show_type_column: bool = True
+    show_value_column: bool = True
+    show_desc: bool = True
+
+    if render_config:
+        show_header = render_config.get("show_header", True)
+        show_type_column = render_config.get("show_type_column", True)
+        show_value_column = render_config.get("show_value_column", True)
+        show_desc = render_config.get("show_description", True)
+
+    table = RichTable(box=box.SIMPLE, show_lines=True, show_header=show_header)
     table.add_column("Field")
-    table.add_column("Type")
-    table.add_column("Value")
-    table.add_column("Description")
+    if show_type_column:
+        table.add_column("Type")
+    if show_value_column:
+        table.add_column("Value")
+    if show_desc:
+        table.add_column("Description")
 
     props = model_cls.schema().get("properties", {})
 
-    for field_name, field in sorted(model_cls.__fields__.items()):
+    for field_name, field in sorted(model_cls.model_fields.items()):
         if exclude_fields and field_name in exclude_fields:
             continue
         row: List[RenderableType] = [field_name]
@@ -676,14 +727,16 @@ def create_table_from_model_object(
             # TODO: check 'anyOf' keys
             desc = p.get("description", "")
 
-        if p_type is None:
-            p_type = "-- check source --"
-        row.append(p_type)
+        if show_type_column:
+            if p_type is None:
+                p_type = "-- check source --"
+            row.append(p_type)
 
         data = getattr(model, field_name)
-        row.append(extract_renderable(data, render_config=render_config))
-
-        row.append(desc)
+        if show_value_column:
+            row.append(extract_renderable(data, render_config=render_config))
+        if show_desc:
+            row.append(desc)
         table.add_row(*row)
 
     return table
@@ -693,7 +746,6 @@ def extract_renderable(
     item: Any, render_config: Union[Mapping[str, Any], None] = None
 ) -> RenderableType:
     """Try to automatically find and extract or create an object that is renderable by the 'rich' library."""
-
     if render_config is None:
         render_config = {}
     else:
@@ -702,13 +754,13 @@ def extract_renderable(
     inline_models_as_json = render_config.setdefault("inline_models_as_json", True)
 
     if hasattr(item, "create_renderable"):
-        return item.create_renderable(**render_config)
+        return item.create_renderable(**render_config)  # type: ignore
     elif isinstance(item, (ConsoleRenderable, RichCast, str)):
         return item
     elif isinstance(item, BaseModel) and not inline_models_as_json:
         return create_table_from_model_object(item)
     elif isinstance(item, BaseModel):
-        return item.json(indent=2)
+        return item.model_dump_json(indent=2)
     elif isinstance(item, Mapping) and not inline_models_as_json:
         table = RichTable(show_header=False, box=box.SIMPLE)
         table.add_column("Key", style="i")
@@ -720,7 +772,7 @@ def extract_renderable(
         result = {}
         for k, v in item.items():
             if isinstance(v, BaseModel):
-                v = v.dict()
+                v = v.model_dump()
             result[k] = v
         return orjson_dumps(
             result, option=orjson.OPT_INDENT_2 | orjson.OPT_NON_STR_KEYS
@@ -748,7 +800,6 @@ def create_renderable_from_values(
     values: Mapping[str, "Value"], config: Union[Mapping[str, Any], None] = None
 ) -> RenderableType:
     """Create a renderable for this module configuration."""
-
     if config is None:
         config = {}
 
@@ -759,12 +810,15 @@ def create_renderable_from_values(
     show_pedigree = config.get("show_pedigree", False)
     show_data = config.get("show_data", False)
     show_hash = config.get("show_hash", True)
+    show_size = config.get("show_size", True)
     # show_load_config = config.get("show_load_config", False)
+    value_title = config.get("value_title", "value")
 
     table = RichTable(show_lines=True, box=box.MINIMAL_DOUBLE_HEAD)
-    table.add_column("value_id", "i")
+    table.add_column(value_title, "i")
     table.add_column("data_type")
-    table.add_column("size")
+    if show_size:
+        table.add_column("size")
     if show_hash:
         table.add_column("hash")
     if show_pedigree:
@@ -773,16 +827,19 @@ def create_renderable_from_values(
         table.add_column("data")
 
     for id, value in sorted(values.items(), key=lambda item: item[1].value_schema.type):
-        row: List[RenderableType] = [id, value.value_schema.type, str(value.value_size)]
+        row: List[RenderableType] = [id, value.value_schema.type]
+        if show_size:
+            row.append(str(value.value_size))
         if show_hash:
             row.append(str(value.value_hash))
         if show_pedigree:
             if value.pedigree == ORPHAN:
                 pedigree = "-- n/a --"
             else:
-                pedigree = value.pedigree.json(option=orjson.OPT_INDENT_2)
+                pedigree = value.pedigree.model_dump_json(indent=2)
             row.append(pedigree)
         if show_data:
+            assert value._data_registry is not None
             data = value._data_registry.pretty_print_data(
                 value_id=value.value_id, target_type="terminal_renderable", **config
             )
@@ -802,7 +859,6 @@ def create_renderable_from_values(
 def create_pipeline_steps_tree(
     pipeline_structure: "PipelineStructure", pipeline_details: "PipelineState"
 ) -> Tree:
-
     from kiara.models.module.pipeline import StepStatus
 
     steps = Tree("steps")
@@ -830,12 +886,12 @@ def create_recursive_table_from_model_object(
     model: BaseModel,
     render_config: Union[Mapping[str, Any], None] = None,
 ):
-
     if render_config is None:
         render_config = {}
 
     show_lines = render_config.get("show_lines", True)
     show_header = render_config.get("show_header", True)
+    show_description = render_config.get("show_description", False)
     model_cls = model.__class__
 
     table = RichTable(box=box.SIMPLE, show_lines=show_lines, show_header=show_header)
@@ -844,8 +900,7 @@ def create_recursive_table_from_model_object(
 
     props = model_cls.schema().get("properties", {})
 
-    for field_name in sorted(model_cls.__fields__.keys()):
-
+    for field_name in sorted(model_cls.model_fields.keys()):
         data = getattr(model, field_name)
         p = props.get(field_name, None)
         p_type = None
@@ -862,9 +917,12 @@ def create_recursive_table_from_model_object(
             data_renderable = extract_renderable(data, render_config=render_config)
             sub_model = None
         else:
+            updated_render_config = dict(render_config)
+            updated_render_config["show_header"] = False
             sub_model = create_recursive_table_from_model_object(
-                data, render_config={"show_lines": True, "show_header": False}
+                data, render_config=updated_render_config
             )
+
             data_renderable = None
 
         group = []
@@ -872,7 +930,7 @@ def create_recursive_table_from_model_object(
         if data_renderable:
             group.append(data_renderable)
             group.append("")
-        if desc:
+        if desc and show_description:
             group.append(f"[i]{desc}[/i]")
 
         if sub_model:
@@ -885,3 +943,30 @@ def create_recursive_table_from_model_object(
         table.add_row(field_name, Group(*group))
 
     return table
+
+
+def create_table_from_data_and_schema(
+    data: Mapping[str, Any], schema: Union[None, Dict[str, Any]] = None
+):
+    table = RichTable(box=box.SIMPLE, show_header=False)
+    table.add_column("key", style="i")
+    table.add_column("value")
+
+    for key, value in data.items():
+        value_renderable = extract_renderable(value)
+        table.add_row(key, value_renderable)
+
+    return table
+
+
+def create_renderable_from_value_id_map(
+    kiara: "Kiara",
+    values: Mapping[str, uuid.UUID],
+    config: Union[Mapping[str, Any], None] = None,
+) -> RenderableType:
+    """Create a renderable for a map of value ids."""
+
+    return create_value_map_status_renderable(
+        kiara.data_registry.load_values(values), render_config=config
+    )
+    # return create_renderable_from_values(kiara.data_registry.load_values(values), config=config)
