@@ -5,7 +5,7 @@
 #  Mozilla Public License, version 2.0 (see LICENSE or https://www.mozilla.org/en-US/MPL/2.0/)
 from click import Group
 
-from kiara import Kiara
+from kiara.context import Kiara
 from kiara.utils.cli import terminal_print
 
 
